/***********************************************************************/
/*   Text Categorization Framework                                     */
/*                                                                     */
/*                                                                     */
/*   Author: Alessandro Moschitti                                      */
/*   moschitti@info.uniroma2.it                                        */
/*   Date: 30.11.03                                                    */
/*                                                                     */
/*   Copyright (c) 2003  Alessandro Moschitti - All rights reserved    */
/*                                                                     */
/*   This software is available for non-commercial use only. It must   */
/*   not be modified and distributed without prior permission of the   */
/*   author. The author is not responsible for implications from the   */
/*   use of this software.                                             */
/*                                                                     */
/***********************************************************************/


// questo programma serve per tradurre i file con le informazioni linguistiche nel formato accettato da SVMlight
// in input il file con i documenti del corpus e l'oracolo ed in output, nella directory dat
// un file per ogni categoria formattato per essere accettato da SVM
// il terzo parametro indica se si sta' utilizzando il test o il train

// ricordarsi di usare il comando "unlimit" prima di lanciare il programma altrimenti il programma 
// non avra' i diritti per aprire i file di output


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAX_CATEG 10000			/*num max di categorie */
#define MAX_LEN_RIGA 80000		/*lunghezza max di riga del file di input*/
#define MAX_LEN 1000

FILE *f_input;
FILE *f_oracolo;


char tipo[MAX_LEN];

struct v_oracolo{
   char id_doc[MAX_LEN];
   char categ[MAX_LEN];
};
struct v_oracolo *vett_oracolo;

int num_oracolo=0;			/* cardinalita' di vett_oracolo */
int punt_oracolo=0;

struct v_categorie{ 			/* per memorizzare l'oragolo */
	char categ[MAX_LEN];
	int numero;	/* num di doc per la categoria */
	int positivi;	/* num di doc positivi */
	int negativi;	/* num di doc negativi */
	};
struct v_categorie vett_categorie[MAX_CATEG];
		
int num_categorie=0; 			/* cardinalita' di vatt_categorie */

struct v_filez { 			/*struttura contenente i puntatori ai file di output */
	FILE *punt;
	};
struct v_filez vett_filez[MAX_CATEG];

struct l_categorie{			/* serve per memorizzare le categorie a cui appartiene un doc */
	char categ[MAX_LEN];};
struct l_categorie lista_categorie[MAX_LEN];
int num_lista_cat;


int cat_presente(char *categoria){
int conf,pos=0;
	conf=strcmp(categoria,vett_categorie[pos].categ);
	while ((conf!=0)&&(pos<num_categorie))
	{
		pos++;	
		conf=strcmp(categoria,vett_categorie[pos].categ);
	};
	
	if (conf==0) {return(1);}
		else 	return(0);	
};


int cat_inserisci(char *categ){
strcpy(vett_categorie[num_categorie].categ,categ);
num_categorie++;
	
};


int carica_oracolo(void)
{
  while (!(feof(f_oracolo)))
  	{
    fscanf(f_oracolo,"%s",vett_oracolo[num_oracolo].id_doc);
    fscanf(f_oracolo,"%s",vett_oracolo[num_oracolo].categ);
  	//printf("\n %s : %s ",vett_oracolo[num_oracolo].id_doc,vett_oracolo[num_oracolo].categ);
	
	
	if (!cat_presente(vett_oracolo[num_oracolo].categ))
		cat_inserisci(vett_oracolo[num_oracolo].categ);
	num_oracolo++;
	
 	}
	
};

int print_categorie(void){
int i;

printf("\nNumber of category = %d \n",num_categorie);
for(i=0;i<num_categorie;i++) {
	printf("%s \n",vett_categorie[i].categ);
	
	
	}
	
	
};

int apri_filez(void){
int i;
char nome_file[MAX_LEN];

//system("unlimit"); 				/*elimina il limite del numero di file che si possono aprire dal programma*/

for(i=0;i<=num_categorie;i++){
	strcpy(nome_file,"./dat/");
	strcat(nome_file,vett_categorie[i].categ);
	strcat(nome_file,"_");
	strcat(nome_file,tipo);
	strcat(nome_file,".dat");
	
	if ((vett_filez[i].punt=fopen(nome_file,"w"))==NULL) {                
	   	printf("impossibile aprire il file di output per %s\n",vett_categorie[i].categ);       
   		exit(-50);
   		};
	}
};


int chiudi_filez(void){
int i;
char nome_file[MAX_LEN];
for(i=0;i<=num_categorie;i++){
	fclose(vett_filez[i].punt);
	}
	
};	

int cerca_appartenenze(char *id_doc, int max_oracle_items){

num_lista_cat=0;
while (punt_oracolo<max_oracle_items && strcmp(vett_oracolo[punt_oracolo].id_doc , id_doc)<=0){
	if (strcmp(vett_oracolo[punt_oracolo].id_doc , id_doc)==0){
		strcpy( lista_categorie[num_lista_cat].categ ,  vett_oracolo[punt_oracolo].categ );
		num_lista_cat++;
		//printf("]] id_doc =%s categoria=%s #lc=%d\n",id_doc,lista_categorie[num_lista_cat].categ,num_lista_cat);
		
		
 		}
	punt_oracolo++;
	}
};

int appartiene(int i){
int i_lista=0;
	
	while (i_lista<num_lista_cat){
		if (strcmp(vett_categorie[i].categ,lista_categorie[i_lista].categ)==0)
			return(1);
		i_lista++;
	}
	return(0);
};

int genera_file(int max_oracle_items){
int i_categ;
char id_doc[MAX_LEN];
char vettore[MAX_LEN_RIGA];

apri_filez();		/* apre tutti i file di output*/

for(i_categ=0;i_categ<num_categorie;i_categ++){
	vett_categorie[i_categ].numero=0;
	vett_categorie[i_categ].positivi=0;
	vett_categorie[i_categ].negativi=0;
}
	
fscanf(f_input,"%s",id_doc);	
while(!feof(f_input)){
	
	
	fgets(vettore,MAX_LEN_RIGA,f_input );
	cerca_appartenenze(id_doc,max_oracle_items);			/* cerca il documento id_doc nell'oracolo e costruisce una 
										lista di categorie a cui il documento id_doc appartiene*/
	
	//printf("= %s ",id_doc);
	for(i_categ=0;i_categ<num_categorie;i_categ++){		/* si scandiscono tutte le categorie */ 
			
		if (appartiene(i_categ)){			/*scrivi 1 sul file i-esimo*/
			fprintf(vett_filez[i_categ].punt,"1 ");
			fprintf(vett_filez[i_categ].punt,vettore);
			vett_categorie[i_categ].numero++;
			vett_categorie[i_categ].positivi++;
			
		}
		else
		{		/*scrivi 1 sul file i-esimo*/
				/* la prossima if limita gli esempi negativi*/
				/* vett_categorie[i_categ].positivi*5 
			if ((vett_categorie[i_categ].negativi<=vett_categorie[i_categ].positivi*10)||(strcmp(tipo,"test")==0)){*/
				fprintf(vett_filez[i_categ].punt,"-1 ");	/*scrivi -1 sul file i-esimo*/
				fprintf(vett_filez[i_categ].punt,vettore);
						
				vett_categorie[i_categ].negativi++;
			/*}*/
			
		};
				
	}
	//printf("\n");
	fscanf(f_input,"%s",id_doc);	
    };

    chiudi_filez();		/* chiude tutti i file di output*/
    return(0);
};

int scrivi_categorie(void){
int i_categ;	
char categ_file[MAX_LEN];
char sort_str[MAX_LEN];
FILE *fc;
strcpy(categ_file,"./dat/categorie.dis.");
strcat(categ_file,tipo);

fc=fopen(categ_file,"w");
for(i_categ=0;i_categ<num_categorie;i_categ++){
	fprintf(fc,"%s %d\n",vett_categorie[i_categ].categ,vett_categorie[i_categ].numero);
	}
	
fclose(fc);

sprintf(sort_str,"sort ./dat/categorie.dis.%s > ./dat/categorie.%s",tipo,tipo);		
system(sort_str);
		
}

int main(int argc, char *argv[])
{  

 FILE *data;

 char lastDocumentIndexString[MAX_LEN],
      command[MAX_LEN];

 long lastDocumentIndex;

if (!(argc=4)) 
    {
     printf("\nMore parameters are required! \nInsert Rocchio Data File Name, Oracle Name and the output directory\n\n");
     exit(0);
    };


if ((f_input=fopen(argv[1],"r"))==NULL) {                
   printf("Input File not found\n");       
   exit(-10);
   };
if ((f_oracolo=fopen(argv[2],"r"))==NULL) {                
   printf("Oracle not Found\n");       
   exit(-20);
   };
   
if ((strcmp(argv[3],"test")==0)||(strcmp(argv[3],"train")==0))
	strcpy(tipo,argv[3]);
else {
	printf("The third parameter indicates the type: [test,train]\n");
	exit(-30);
}
   
    printf("\nLoading Oracle..");fflush(stdout);

    sprintf(command,"wc -l %s > docNumber",argv[2]);
    system(command);
    
    if((data = fopen("docNumber","r"))==NULL){
	    printf("ERROR: problem with oracle file\n\n");
	    exit(-1);
    }   

    fscanf(data,"%s",lastDocumentIndexString); 
    fclose(data);
    lastDocumentIndex=atoi(lastDocumentIndexString);
    system("rm -f docNumber");
  
   vett_oracolo=(struct v_oracolo *) malloc(sizeof(struct v_oracolo)*(lastDocumentIndex+1));
   carica_oracolo();
   printf("done!!!\n");

   print_categorie();

   genera_file(lastDocumentIndex+1);

   scrivi_categorie();

fclose(f_input);
fclose(f_oracolo);
}
