/***********************************************************************/
/*   Text Categorization Framework                                     */
/*                                                                     */
/*                                                                     */
/*   Author: Alessandro Moschitti                                      */
/*   moschitti@info.uniroma2.it                                        */
/*   Date: 30.11.03                                                    */
/*                                                                     */
/*   Copyright (c) 2003  Alessandro Moschitti - All rights reserved    */
/*                                                                     */
/*   This software is available for non-commercial use only. It must   */
/*   not be modified and distributed without prior permission of the   */
/*   author. The author is not responsible for implications from the   */
/*   use of this software.                                             */
/*                                                                     */
/***********************************************************************/


#include<stdio.h>
#include "LearningManager.h"


#define CLASS_NAMES "class_names"
#define	TEMP_CLASS "temp_class"
#define ORDERED_CENTROIDS "ordered_centroids"

#define GENERATE_DOCUMENT_LIST "ls documents > temp/documentNames"
#define DOCUMENT_NAMES "temp/documentNames"

#define CLASSES	"classes"
#define INFO_DICTIONARY "Dict.Info"
#define OFFSET_DICTIONARY "Dict.Offset"
#define WEIGHT_DICTIONARY "Dict.Weight"
#define GLOBAL_CENTROID "globalCentroid"
#define SCALAR_PRODUCTS "StdScarti"
#define GENERAL_DOCUMENT "generalDocument"


#define TERM_LENGHT 300
#define FREAD_BUFFER_SIZE 1000
#define CENTROID_LINE_LENGHT 300
#define FREQUENCY_DIGIT 7
#define SYNTATCTIC_CLASS_DIGIT 2
#define OFFSET_DIGIT 10
#define FWRITE_BUFFER_SIZE 400
#define MAX_DOC_ID 20
#define MAX_DOC_SIZE 25000
#define MAX_K_NEAREST 500


#define ID_DOC_LENGHT 30
#define MAX_CLASS_LENGHT 30



#define MAX_CHUNK_NUMBER 15 /* Max Number of file in which the corpus has been splitted to be process by parser */
#define MAX_NUMBER_OF_DOC_PER_CLASS 100000
#define HASH_COSTANT 100000 /* HASH_COSTANT*MAX_CHUNK_NUMBER+ID_DOC_INSIDE_THE_CLASS = ID_DOC */


struct oracleCell
   {
    char idDoc[ID_DOC_LENGHT];
    char **targetClass;
   };

struct contigencyCell {
	  int wrong,correct,notRecovered;
  };
  
  
struct results {
	float precision;
	float recall;
};

struct classFiles
  {
   FILE *file,
	*ordFile;
   short status;
   int numOfDocuments;
   float *Norma;
   float currentWeight;
   char currentTerm[TERM_LENGHT];
   int numOfDocWithCurrentTermInClass;
  };
 
struct couple
  {
   int centroidNumber;
   float weight;
  };
  
struct couples
  {
   int numOfDocWithCurrentTerm;
   struct couple couple[MAX_CLASS_NUMBER];
  };

struct pair{
   char *item;
   int freq;
   float count;
   char lastDocId[MAX_DOC_ID];
  };

struct dictionaryOffset{
    char *term;
    int Offset;
   };
   
struct dictionary
{
 char term[TERM_LENGHT];
 int freq;
};

/* Document Vector */

struct docVect{
 char term[TERM_LENGHT];
 float weight;
};  


 /*  Document DB for KNN classifier */
  
 
struct docDB{
	int size;
	char docId[MAX_DOC_ID];
	struct docVect *docCentroid;
};

struct catDocDB{
	int size;
	struct docDB *db;
	};  
 
struct k_best{
	float score[MAX_K_NEAREST];
	int actualSize;
};


struct idDocDictionary
{
 char idDoc[MAX_DOC_ID];
 int numericIdDoc;
};
  
struct centroid
  {
   FILE *file;
   int status;
   char currentTerm[FREAD_BUFFER_SIZE];
   float  currentFreq;
   float currentNumberOfDoc;
  };
  

  /* struct couples setOfCouples */
  struct varStat{
	  float average,
	        stdDeviation;
  };
    
/* environment struct for classification processing */
struct CKB_env {

  FILE *weightDictFile;
  
  struct dictionaryOffset *offsetDictArray;
  
  int totalDocumentNumber;
  int differentWordNumber;
  
  char centroidNames[MAX_CLASS_NUMBER][PATH_LENGHT];
  float centroidTotalFreqs[MAX_CLASS_NUMBER];
  int  centroidNumber;
  
  struct varStat negativ[MAX_CLASS_NUMBER],
		 positiv[MAX_CLASS_NUMBER]; 
  
  float numOfTermExp[MAX_CLASS_NUMBER];
	  
  float threshold;
  
  int cuttedWords;
  float maxWeight;  
  
  int numOfCorpusDoc;
};
  
  
extern float ft(float freq);

/* Open all Centroids */
extern int openAllCentroids(char *centroid_path, char **centroid_names,int  centroid_number,short verb,
                    struct centroid *centroids, char *extension, FILE *dictionaryInfo);

/* Open all Classes*/
extern int openAllClass(char *class_path, char **class_names,int  class_number,short verb,
                    struct classFiles class[], char *extension, FILE *dictionaryInfo);

/* close all files */
extern void closeAllCentroidFiles(struct centroid *centroids,int n);

/*finds the mins among the current terms of centroids*/
extern void findMinElementOfCentroids(struct centroid *centroids, int classNumber);

/*read the next term of all centroids whoose current terms have been merged*/
extern void updateCurrentElementOfCentroids(struct centroid *centroids, int classNumber);

/*merge the mins found and retun true if exist at least a min, count even the
total number of terms*/
extern int mergeInGlobalCentroid(FILE *fileGlobalCen,FILE *fileDICT,struct centroid *centroids,
		int classNumber, int *totalTermNumber, int DF_cut);

/* init the status to 1 that is all centroids have to be read */
extern void initStatusOfCentroids(struct centroid *centroids,int classNumber);

/* load all the couples (centroidNumber,weight) in dictionary relative to a term */
extern int couplesRelativeToTerm(FILE *f1,char *term,struct couples *setOfCouples, struct dictionaryOffset
		Dict[],int n);

/* compute the number of all corpus term */
extern int computeTotalTermNumber(char *globalCentroidAbsName, short verbose, int
		*differentWordNumber);

/* write the copules (centroid, number) weight on dictionary weights and offset in Dict */ 
extern int writeCouplesOnDictionaryWeight(FILE *fileDictionaryWeight, FILE *fileDICT, 
		struct classFiles *class,int classNumber, int totalDocumentNumber,int *pointer,
		float *negativSumOfOther, char *term, struct dict_env MY_env);

extern void DictionariesBuilding(struct dict_env my_env);

extern void Classification(struct classifier_env my_env);

extern int couplesRelativeToTerm(FILE *f1,char *term,struct couples *setOfCouples, struct
		dictionaryOffset Dict[],int n);

extern int SearchInStopList(char *stopWord[],char *term,int i, int j);

extern void takeNextTermFromClasses(struct classFiles class[], int classNumber,int numOfDoc, 
			struct dictionaryOffset *globalCe, int differentTermNumber);
extern int Search(struct dictionaryOffset Dict[],char term[],int i, int j);

extern float terminologicalWeigth(char *term);
extern float terminologicalWeigthOfOtherClass(char *term);

extern int buildExtractedDocumentCentroidWithWeight(FILE *classFile,struct docVect *docCentroid,long *pos, 
				   char *documentIdOld, struct CKB_env *clKB, int catnum);
extern float ftw(float freq);

extern void loadCategoryDB (char *classPath, struct CKB_env *my_CKB_env, int whichClass, char *extension, struct docDB **db, int *numberOfDoc);

extern void aDocScalarProductWithTrainingData(struct catDocDB *docDB, struct docVect *docCentroid, int centroidSize, char *extension,
		            struct CKB_env *clKB, float *scalarProducts, struct oracleCell *oracle, int K_nearest);

extern void shutdownDB(struct docDB *db, int numberOfDoc);

