/***********************************************************************/
/*   Text Categorization Framework                                     */
/*                                                                     */
/*                                                                     */
/*   Author: Alessandro Moschitti                                      */
/*   moschitti@info.uniroma2.it                                        */
/*   Date: 30.11.03                                                    */
/*                                                                     */
/*   Copyright (c) 2003  Alessandro Moschitti - All rights reserved    */
/*                                                                     */
/*   This software is available for non-commercial use only. It must   */
/*   not be modified and distributed without prior permission of the   */
/*   author. The author is not responsible for implications from the   */
/*   use of this software.                                             */
/*                                                                     */
/***********************************************************************/



#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAX_LINE_LEN 20000
#define MAX_FILE_LINE_NO 100000
#define MAX_CHUNKS_NUMBER 1000
#define GET_LINE_EOF 4


int col2=0;

long partition(char **vect,long part_low, long part_high) {
	
	long lastsmall;
	long comp1;
	long i;
	char *median_val,*transit;

	// swap median value an first value of array
	comp1=(part_low+part_high) / 2;		
	transit=vect[part_low];
	vect[part_low]=vect[comp1];
	vect[comp1]=transit;
	median_val=vect[part_low];
	lastsmall=part_low;
	for (i=part_low+1; i<=part_high; i++) {
		if (strcmp(vect[i]+col2,median_val+col2)<0) {
			lastsmall++;
			// swap lastsmall and i
			transit=vect[lastsmall];
			vect[lastsmall]=vect[i];
			vect[i]=transit;
		}
	}
	// swap part_low and lastsmall
	transit=vect[part_low];
	vect[part_low]=vect[lastsmall];
	vect[lastsmall]=transit;

	return lastsmall;
}

void quicksort(char **vect,long qk_low, long qk_high) {
	if (qk_low < qk_high)	{
		long median=partition(vect,qk_low, qk_high);
		quicksort(vect,qk_low, median);
		quicksort(vect,median+1, qk_high);
	}
}


void PrintUsage(char *ProgramName)
{
    printf("\n %s usage :\n",ProgramName);
    printf("    %s SourceFile > DestinationFile\n",ProgramName);
    printf("   or\n");
    printf("    %s SourceFile > DestinationFile\n",ProgramName);

}


int load_chunk(FILE *fp, char **vect){
  
  int howmany=0;
  char line[MAX_LINE_LEN];

   while (!feof(fp) && howmany<MAX_FILE_LINE_NO) {
      if( fgets( line, MAX_LINE_LEN, fp ) == NULL)  {
	   if (!feof(fp))  {
		   printf("Error in reading a line\n");
		   exit(-1);
	   }
       } else
	   if((vect[howmany++]=strdup(line))==NULL){
		   printf("\n\nERROR: OUT OF MEMORY\n\n");
		   exit(-1);
	   }
   }

  return howmany;
}

int get_min(FILE *chunk_files[MAX_CHUNKS_NUMBER], char *lines[MAX_CHUNKS_NUMBER], int num_of_chunks){
    
  int i,min=0;

	for(i=0;i<num_of_chunks;i++)
	  if(lines[i] != NULL){
	     if(lines[min] != NULL){
	       if(strcmp(lines[min]+col2,lines[i]+col2)>0)min=i;
	     }
	     else min=i;	  
	 }

	return min;
}

main(int argc,char *argv[]) {

	FILE *fp,*temp1,*temp2;

	FILE *chunk_files[MAX_CHUNKS_NUMBER];

	char *lines[MAX_CHUNKS_NUMBER];

	char line[MAX_LINE_LEN];
	char firstColum[MAX_LINE_LEN];
	char filename[MAX_LINE_LEN];

 	char **vect;
	long howmany;
	long i,lineNum=0;
	int num_of_chunks,
	    current_chunk,
	    min;


	if ((fp=fopen(argv[1],"r"))==NULL) {
		printf("Source file not accessible\n");
		exit(-1);
	}


	while( fgets( line, MAX_LINE_LEN, fp ) != NULL)lineNum++;
	//printf("NUMBER OF LINES %d\n",lineNum);

	num_of_chunks=lineNum/MAX_FILE_LINE_NO+1;
        vect=(char**)malloc(sizeof(char*)*(MAX_FILE_LINE_NO));

	if(vect==NULL){printf("\n\nERROR: OUT OF MEMORY\n\n");exit(-1);}
	/* Help request  
	if ( argc<!=2) {
		PrintUsage(argv[0]);
		exit(0);
	}
	
*/

	fseek(fp,0,SEEK_SET);

	if(argc==3)col2 = atoi(argv[2]);
	
	for(current_chunk=0;current_chunk<num_of_chunks;current_chunk++){

	   howmany=load_chunk(fp,vect);
	   quicksort(vect,0,howmany-1);


	   sprintf(filename,"moka2003_temp%d",current_chunk);
	   if ((chunk_files[current_chunk]=fopen(filename,"w"))==NULL) {
		printf("%s file not openable\n",filename);
		exit(-1);
	   }
	   for (i=0; i< howmany ; i++){
		fprintf(chunk_files[current_chunk],"%s",vect[i]);
		free(vect[i]);
	   }
	   fclose(chunk_files[current_chunk]);
	   if ((chunk_files[current_chunk]=fopen(filename,"r"))==NULL) {
		printf("%s file not openable\n",filename);
		exit(-1);
	   }

	}

  	for(current_chunk=0;current_chunk<num_of_chunks;current_chunk++){
	   *line=0;
	   fgets( line, MAX_LINE_LEN, chunk_files[current_chunk] );
           if(!feof(chunk_files[current_chunk]))lines[current_chunk]=strdup(line);
	   else {/*printf("Warning a chunk empty before starting\n\n");*/lines[current_chunk]=NULL;}
	}
        
        min=get_min(chunk_files, lines, num_of_chunks);
	while(lines[min]!=NULL){
		printf("%s",lines[min]);
		free(lines[min]);*line=0;
	   	fgets( line, MAX_LINE_LEN, chunk_files[min] );
	   	if(!feof(chunk_files[min]))lines[min]=strdup(line);
           	else lines[min]=NULL;
	   	min=get_min(chunk_files, lines, num_of_chunks);
	 }

	for(current_chunk=0;current_chunk<num_of_chunks;current_chunk++)fclose(chunk_files[current_chunk]);

	fclose(fp);
	system("rm -f moka2003_temp*");
	exit(0);
}
