/***********************************************************************/
/*   Text Categorization Framework                                     */
/*                                                                     */
/*                                                                     */
/*   Author: Alessandro Moschitti                                      */
/*   moschitti@info.uniroma2.it                                        */
/*   Date: 30.11.03                                                    */
/*                                                                     */
/*   Copyright (c) 2003  Alessandro Moschitti - All rights reserved    */
/*                                                                     */
/*   This software is available for non-commercial use only. It must   */
/*   not be modified and distributed without prior permission of the   */
/*   author. The author is not responsible for implications from the   */
/*   use of this software.                                             */
/*                                                                     */
/***********************************************************************/


#include "classifier.h"
#include <math.h>
#include <string.h>
#include <stdio.h>

/**********************************************************************************************
* 					Program OPTION					      *
***********************************************************************************************/

#define ID_LENGTH 25


/*********************************************************************************************/
/***				Heading of PROCEDURES					  ****/
/*********************************************************************************************/


 
  
  

/*********************************************************************************************/
/* 					KNN CODE	   				     */
/*********************************************************************************************/

void shutdownDB(struct docDB *db, int numberOfDoc){

 int i;
 	
   for(i=0;i<numberOfDoc;i++)
	free(db[i].docCentroid);
   free(db);
}  
  
 
float documentScalarProduct(struct docVect *docCentroid1, int centroidSize1,
			     struct docVect *docCentroid2, int centroidSize2) {
  int indTermDoc1,
      indTermDoc2,
      confront;
 	  
  double 
	 scalarProduct=0,
	 docNorma1=0,
  	 docNorma2=0;
  
    /* TEST  
        printf("\n\nTEST DOC Centroid Size:%d ",centroidSize1);
  
  for(indTermDoc1=0;indTermDoc1<centroidSize2;indTermDoc1++)
           printf ("\nfirst  term: %s ---> w1 %f index %d",
                       docCentroid2[indTermDoc1].term,docCentroid2[indTermDoc1].weight,indTermDoc1);
  printf("\n\n");
  */
   
  indTermDoc1=0;
  indTermDoc2=0;
  
  while(indTermDoc2<centroidSize2 || indTermDoc1<centroidSize1){   

/* TEST printf ("  Moltiplico Word %s ---> %d \n",docCentroid1[i].term,i);*/

    
    if(indTermDoc2>=centroidSize2) confront=-1;
    else
    	if(indTermDoc1>=centroidSize1) confront=1;
    	  else confront=strcmp(docCentroid1[indTermDoc1].term,docCentroid2[indTermDoc2].term);
    
    if(confront==0){
	      
       docNorma1+=(docCentroid1[indTermDoc1].weight * docCentroid1[indTermDoc1].weight);/* Squared Norma of doc1 */
       
       docNorma2+=(docCentroid2[indTermDoc2].weight * docCentroid2[indTermDoc2].weight);

       scalarProduct+=docCentroid1[indTermDoc1].weight*docCentroid2[indTermDoc2].weight; 
       
       /* TEST  printf ("both -> term: %s ---> w1 %f w2 %f\n",
		       docCentroid1[indTermDoc1].term,docCentroid1[indTermDoc1].weight,docCentroid2[indTermDoc2].weight);
	*/
       indTermDoc1++;
       indTermDoc2++;
    }
    else if (confront<0){
            docNorma1+= docCentroid1[indTermDoc1].weight * docCentroid1[indTermDoc1].weight;
	     /* TEST    printf ("first  term: %s ---> w1 %f index %d\n",
                       docCentroid1[indTermDoc1].term,docCentroid1[indTermDoc1].weight,indTermDoc1);
	     
		*/
	    indTermDoc1++;
         }
	 else if (confront>0){
       	    docNorma2+=(docCentroid2[indTermDoc2].weight * docCentroid2[indTermDoc2].weight);
	    /* TEST  printf ("second  term: %s ---> w2 %f \n",
		       docCentroid2[indTermDoc2].term,docCentroid2[indTermDoc2].weight);
		*/
	    indTermDoc2++;
         }
	 
	 /*TEST 
	  printf (" scalarProduct  %f \n",scalarProduct);
	  printf ("  docNorma1,docNorma2 %f, %f\n\n",docNorma1,docNorma2 );
	*/
  }
     	


    /* Normalizzation of scalar product   */
     if(scalarProduct!=0) scalarProduct/=sqrt(docNorma1*docNorma2);
    /* printf ("FINAL scalarProduct  %f \n\n",scalarProduct);	*/
    
     return scalarProduct;
}

void K_maxScore(struct k_best *kbest, float score, int k){

   int i,j;
 
    i=0;
    
 /*TEST   printf("K %d  actual %d\n",k,kbest->actualSize);*/
    
   while(i<kbest->actualSize && score<kbest->score[i])i++;

   if(i<k){/* shift back the i-k product*/
     for(j=k-1;j>i;j--)kbest->score[j]=kbest->score[j-1];
     kbest->score[i]=score;
     if(kbest->actualSize<k) kbest->actualSize++;
   }   
	
}
 
	 
void loadCategoryDB (char *classPath, struct CKB_env *my_CKB_env, int whichClass, char *extension, struct docDB **db, int *numberOfDoc)
{ 
  char className [PATH_LENGHT];
  char classFileAbsName[PATH_LENGHT];
  
  struct docVect docCentroid[MAX_DOC_SIZE];
  int 	 centroidSize;
  int 	 docIndex;
  FILE *classFile;
  
  char 	docId[PATH_LENGHT];
  
  int someCriteria=0;
  
  char 	temp[PATH_LENGHT],
	command[PATH_LENGHT];
  
  long pos; /*position of class file pointer */
    
  int *centroidSizes, numOfDoc;
 
  /* -------------------------------------------- */
  /*    Load the set of document to be clusterd   */
  /* -------------------------------------------- */
  
  sprintf(classFileAbsName,"%s/%s.%s",classPath,my_CKB_env->centroidNames[whichClass],extension);
    if((classFile = fopen(classFileAbsName,"r"))==NULL) {
	     printf("ERROR : file %s to be clustered does not found\n",classFileAbsName);
	     exit(-1);
    }
   
   
  /* count the document with the function for load a single doc */
    
  centroidSizes=(int *) malloc(sizeof(int)*MAX_DOCS_IN_CLASS);
 
  numOfDoc=0;strcpy(docId,"");pos=0;
  
  while(!feof(classFile)){
	centroidSizes[numOfDoc]=
	buildExtractedDocumentCentroidWithWeight(classFile,docCentroid,&pos,docId,my_CKB_env,whichClass);
	numOfDoc++; 
  }
  printf(" num of doc %d\n", numOfDoc);
  
  *db=(struct docDB *) malloc(sizeof(struct docDB)*(numOfDoc));
   
  strcpy(docId,"");pos=0;  
   
  //printf("size of first doc %d num Of doc%d\n\n",centroidSizes[0],numOfDoc);
  fseek(classFile,pos,SEEK_SET);     
  for(docIndex=0;docIndex<numOfDoc;docIndex++){

	 (*db)[docIndex].docCentroid=(struct docVect *) malloc(sizeof(struct docVect)*centroidSizes[docIndex]+1);
 
	 (*db)[docIndex].size = 
	   buildExtractedDocumentCentroidWithWeight(classFile,(*db)[docIndex].docCentroid,&pos,(*db)[docIndex].docId,my_CKB_env,whichClass);
	 /*TEST  printf("Loaded document %s \n",
				 (*db)[docIndex].docId);*/
  }
  
  free(centroidSizes);
     
  fclose(classFile); 
  *numberOfDoc=numOfDoc; /* return the number of document in the target category */
}

		
void aDocScalarProductWithTrainingData(struct catDocDB *docDB, struct docVect *docCentroid, int centroidSize, char *extension,
		            struct CKB_env *clKB, float *scalarProducts, struct oracleCell *oracle, int K_nearest) {
	
  int catIndex,
      i,
      docIndex,
      numOfClassesOfThisDoc,
      idDocPos;
  
  struct k_best kbest[MAX_CLASS_NUMBER];
  
  float aProduct,
	som;
  
  /* reset scalar product and Norma */
  for(catIndex=0;catIndex<clKB->centroidNumber;catIndex++)
     kbest[catIndex].actualSize=scalarProducts[catIndex]=0;
   
    /* TEST   
	printf("Centroid Size:%d\n",centroidSize);*/

    for(catIndex=0;catIndex<clKB->centroidNumber;catIndex++){
	/* TEST    
       printf("\n\n\nSimilarity between test doc and cat %d of %d docs\nscalar products: \n",catIndex,docDB[catIndex].size);*/
	    
       for(i=0;i<MAX_K_NEAREST;i++)kbest->score[catIndex]=0;
       for(docIndex=0;docIndex<docDB[catIndex].size;docIndex++){  
	
	 idDocPos=searchIdDoc(docDB[catIndex].db[docIndex].docId,oracle,0,clKB->numOfCorpusDoc-1);
	 if(idDocPos==-1){
		 printf("ERROR: The Document %s is not present in the ORACLE; did you select a wrong ORACLE?\n",
				 docDB[catIndex].db[docIndex].docId);
     		 exit(-1);
	 }
	 numOfClassesOfThisDoc=0;
      	 while(oracle[idDocPos].targetClass[numOfClassesOfThisDoc]!=NULL)numOfClassesOfThisDoc++; 
	 
         aProduct = documentScalarProduct(docCentroid,centroidSize,
	            docDB[catIndex].db[docIndex].docCentroid,docDB[catIndex].db[docIndex].size)/*numOfClassesOfThisDoc*/;

	 /*TEST
	 printf("cat %d index %d  doc %s scalar product %f multeplicity %d\n",catIndex,docIndex, docDB[catIndex].db[docIndex].docId,
			 aProduct, numOfClassesOfThisDoc);		 
	 printf("%20s\t%f computed\n",docDB[catIndex].db[docIndex].docId,aProduct);
	 */
			 		 
	 K_maxScore(&(kbest[catIndex]), aProduct, K_nearest);
	 
    }
       som=0;     
       for(docIndex=0;docIndex<kbest[catIndex].actualSize;docIndex++)
	  som=som+kbest[catIndex].score[docIndex]/K_nearest;
     
       /*TEST   
       for(i=0;i<kbest[catIndex].actualSize;i++)
	  printf("\n%d) score= %f",i,kbest[catIndex].score[i]);
	  printf("\nFINAL som %f\n\n\n",som);
	   */
       scalarProducts[catIndex]=som;
      
   }
   	
  
   /* Normalizzation of scalar product   */

    /*TEST	  printf("document Norma:%f\t",docNorma);*/

    /* for(i=0;i<clKB->centroidNumber;i++){
    /*TEST	  printf("prodotto alla %d_esima classe:%f\t",i,
		      scalarProducts[i]);*/
    /*TEST	  printf("prodotto alla %d_esima classe:%f\t",i,
		      scalarProducts[i]);*/

 /* }
    /*TEST printf("count:%d\n",count);*/
}
