/***********************************************************************/
/*   Text Categorization Framework                                     */
/*                                                                     */
/*                                                                     */
/*   Author: Alessandro Moschitti                                      */
/*   moschitti@info.uniroma2.it                                        */
/*   Date: 30.11.03                                                    */
/*                                                                     */
/*   Copyright (c) 2003  Alessandro Moschitti - All rights reserved    */
/*                                                                     */
/*   This software is available for non-commercial use only. It must   */
/*   not be modified and distributed without prior permission of the   */
/*   author. The author is not responsible for implications from the   */
/*   use of this software.                                             */
/*                                                                     */
/***********************************************************************/



#include "classifier.h"
#include "base_lib.h"
#include "string.h"
#include "math.h"
#include "stdlib.h"


int countDocument(FILE *classFile)
{ 
  int	docCount;
  
  char documentIdString[TERM_LENGHT];
  char documentIdStringOld[TERM_LENGHT];
  char term[TERM_LENGHT];
  char freq[TERM_LENGHT];
   
 fscanf(classFile,"%s",documentIdString);
 strcpy(documentIdStringOld,documentIdString);
 docCount=1;

 while(!feof(classFile)){
   if(strcmp(documentIdStringOld,documentIdString)!=0){
	strcpy(documentIdStringOld,documentIdString);
	docCount++;
   } 	
   fscanf(classFile,"%s",term);
   fscanf(classFile,"%s",freq);
   fscanf(classFile,"%s",documentIdString);
 }
   
 return docCount;
  
}  
  

int classSplitting(struct class_ce_env my_env,int whichClass, char **idDocList, int extractingDoc,
		   char *extension, FILE* testset)
{
  FILE  *classFile,
  	*splittedClassFile;

  char 	classAbsFileName[PATH_LENGHT],
	splittedClassAbsFileName[PATH_LENGHT],
	testset_file_name[PATH_LENGHT],
	copyClass[PATH_LENGHT];
		
  int	
  	docIndex,
        docRequired,
	pos,
	docCounter=0;
  

  if (my_env.verbose_switch==ON)
      if(strcmp(my_env.testDocumentsIDfile,"-")==0)  
    	printf("Splitting class '%s' using %d percent\n\n",my_env.class_names[whichClass],my_env.test_set_percentage);
      else printf("Extracting documents indicated in file %s\n\n", my_env.testDocumentsIDfile);
      


  sprintf(classAbsFileName,"%s/%s.%s",my_env.class_path,my_env.class_names[whichClass],extension);  
  if((classFile=fopen(classAbsFileName,"r"))==NULL){
     if (my_env.verbose_switch==ON) printf("    Warning : file %s does not found\n",classAbsFileName);
     return 0;
  }
  

  sprintf(splittedClassAbsFileName,"%s/%s.%s",my_env.splitted_class_path,my_env.class_names[whichClass],extension);    
  
  if((splittedClassFile=fopen(splittedClassAbsFileName,"w"))==NULL){
     if (my_env.verbose_switch==ON) printf("   Warning : file %s does not found\n\n",splittedClassAbsFileName);
     return 0;
  }
  
 
  /*to improve performace fseek(file,0,SEEK_END);

  numberOfDoc=countDocument(classFile);*/
  
  if(my_env.test_set_percentage>0 || strcmp(my_env.testDocumentsIDfile,"-")!=0){
	  
	rewind(classFile);
	pos=0;

	
	for(docIndex=0;docIndex<extractingDoc;docIndex++){
	      
		                            
      	     if (extractOneDocumentForClass(classFile, splittedClassFile, testset,
          			            idDocList[docIndex], "extract", my_env.verbose_switch,
				            &pos, extension, &docCounter)){	  
             }
	     else printf("ERROR: document %s could not be extracted!\n",idDocList[docIndex]);

 	}
	 
	if (extractOneDocumentForClass(classFile, splittedClassFile, testset,
          			       "no extraction","no extraction",my_env.verbose_switch,
				       &pos, extension, &docCounter))
		printf("\nclass %s has been created!\n\n",splittedClassAbsFileName);	   
		
	else printf("ERROR: class %s has not beeen created!\n\n",splittedClassAbsFileName);
	fclose(splittedClassFile);
	fclose(classFile);
	if(my_env.test_set_percentage>=100){
		printf("Warning: all documents splitted for class %s, learning will not started\n\n",classAbsFileName);
		return 0;
	}
  } 
  else{
    fclose(splittedClassFile);  
    printf("Warning: 0 documents splitted for class %s, splitting percentage too low\n\n",classAbsFileName);
    fclose(classFile);
    sprintf(copyClass,"cp %s %s",classAbsFileName,splittedClassAbsFileName);
    system (copyClass);        
    return 1;
  }  
 
 
 return 1;
}

int extractOneDocumentForClass(FILE *classFile, FILE *splittedClassFile, FILE *testset, 
			       char *requiredDocument, char *extract_flag, int verbose, long *pos, char *extension, int *docCounter)
 {	 

  int freq;

  
  char term[TERM_LENGHT];
  char documentIdString[TERM_LENGHT];
  char documentIdStringOld[TERM_LENGHT];
  char extractedDocument[PATH_LENGHT];
  FILE 	*extractedDocumentFile,
	*temporaryClassFile;
   		  
  int first;
     
  /* Insert the procedure to find the document number documentId */
  fseek(classFile,*pos,SEEK_SET);
  fscanf(classFile,"%s",documentIdString);

 first=1;

 while(!feof(classFile) && 
       (strcmp(requiredDocument,documentIdString) >=0 || strcmp(extract_flag,"no extraction")==0)) {    
	

     /*TEST  printf("documentIdString %s\n",documentIdString); */ 
        
	/* read from the class */
     
     fscanf(classFile,"%s",term);
     fscanf(classFile,"%d\n",&freq);
     
       /* printf("required docuement %d ---- actual document %d\n",atoi(requiredDocument),atoi(documentIdString));*/
     
     if(strcmp(requiredDocument,documentIdString)==0 && strcmp(extract_flag,"no extraction")!=0){
#ifndef RANDOM_SELECTION_OF_TRAINING
	if (verbose==ON && first==1){ 
		printf("      document %s created!\n",requiredDocument);
		first=0;
	}
        fprintf(testset,"%s\t",documentIdString);
        fprintf(testset,"%s\t",term);
        fprintf(testset,"%d\n",freq);
#endif
#ifdef RANDOM_SELECTION_OF_TRAINING
	if (verbose==ON) printf("      document %s skipped!\n",documentIdString);
#endif
       *pos=ftell(classFile);
      /*TEST printf("%s %d lo scrivo su extracted\n",documentIdString,*docCounter);*/

     }
     else{
       fprintf(splittedClassFile,"%s\t",documentIdString);
       fprintf(splittedClassFile,"%s\t",term);
       fprintf(splittedClassFile,"%d\n",freq);
       *pos=ftell(classFile);
     /*TEST  printf("%s %d lo scrivo su learning\n",documentIdString,*docCounter);*/
     }
     
     strcpy(documentIdStringOld,documentIdString);
     fscanf(classFile,"%s",documentIdString);
     if(strcmp(documentIdStringOld,documentIdString)!=0){
	    /*TEST printf("siamp al doc : %d\n",*docCounter);*/
	     (*docCounter)++; 
     }
  }  
  return 1;
}
     

