/***********************************************************************/
/*   Text Categorization Framework                                     */
/*                                                                     */
/*                                                                     */
/*   Author: Alessandro Moschitti                                      */
/*   moschitti@info.uniroma2.it                                        */
/*   Date: 30.11.03                                                    */
/*                                                                     */
/*   Copyright (c) 2003  Alessandro Moschitti - All rights reserved    */
/*                                                                     */
/*   This software is available for non-commercial use only. It must   */
/*   not be modified and distributed without prior permission of the   */
/*   author. The author is not responsible for implications from the   */
/*   use of this software.                                             */
/*                                                                     */
/***********************************************************************/



#include "classifier.h"
#include "base_lib.h"



int unifieLemmaDoc(struct class_env my_env, char *classFileName, char *rugClassFIleName, 
		   int classNumber, FILE *oracle,   char *stopWord[], int size)
 {	 
      
  char documentNumber[TERM_LENGHT];
  
  char key[FREAD_BUFFER_SIZE];
  int i;
  int idTerm;
  
  struct dictionary docCentroid[MAX_DOC_SIZE];
  int 	 centroidSize,docId;
   
  long pos;
  
  FILE *f1;
  FILE *class;   
   
  if((f1=fopen(rugClassFIleName,"r"))==NULL) {
   if (my_env.verbose_switch==ON)printf("   Warning : file %s not found\n",rugClassFIleName);
      return 0;
  }
    
  if((class=fopen(classFileName,"w"))==NULL) {
   if (my_env.verbose_switch==ON)printf("   Warning : cannot create the %s file\n",classFileName);
      return 0;
  }

  pos=0;*documentNumber==0;
  while(!feof(f1)){
     centroidSize = buildCleanedDocumentCentroid(f1,docCentroid,
                    &pos,documentNumber,stopWord,size);
     /*TESTprintf("Doc. number %s built, size=%d\n",documentNumber,centroidSize);*/
		   /*  docId=atoi(documentNumber);*/
     for(idTerm=0;idTerm<centroidSize;idTerm++){
        fprintf(class,"%s\t",documentNumber);
	fprintf(class,"%s\t",docCentroid[idTerm].term);
	fprintf(class,"%d\n",docCentroid[idTerm].freq); 
     }
     if(*documentNumber!=0) fprintf(oracle,"%s\t%s\n",documentNumber,my_env.class_names[classNumber]);
  }		 

  fclose(f1);
  fclose(class);
  return 1;
}


void task0(struct class_env my_env,int whichClass,char *extension, FILE *oracle, char *stopWord[], int size)
{
  char classFileName[PATH_LENGHT];
  char rugClassFileName[PATH_LENGHT];
  char centroidFileName[PATH_LENGHT];
  char orderedCentroidFileName[PATH_LENGHT];
  char sortcommand[200],
       erasecommand[200];
  
  if (my_env.verbose_switch==ON) 
    printf("\nPurification from stop-words and Unification Lemma for class '%s' using '%s' information\n\n",my_env.class_names[whichClass],extension);

  sprintf(classFileName,"%s/%s.%s.temp",my_env.class_path,my_env.class_names[whichClass],extension);

  sprintf(rugClassFileName,"%s/%s.%s",my_env.rug_class_path,my_env.class_names[whichClass],extension);
    
  if (unifieLemmaDoc(my_env,classFileName,rugClassFileName,whichClass,oracle, stopWord,size)==0 )
	return; /*the class file does not exist*/

  sprintf(sortcommand,"./bin/mysort_merge %s > %s/%s.%s",classFileName,my_env.class_path,my_env.class_names[whichClass],extension);
  sprintf(erasecommand,"rm -f %s",classFileName);
  system(sortcommand);
  system(erasecommand);
  if (my_env.verbose_switch==ON) 
      printf("          class file=%s/%s.%s purified and ordered\n",
	  my_env.class_path,my_env.class_names[whichClass],extension);
}


void ClassUnificationProcessing(struct class_env my_env) 
{
   int whichClass;
   FILE *oracle;
   
   char command[PATH_LENGTH];
   char *stopWord[MAX_STOP_WORD];
   char stopListFileName[PATH_LENGHT]; 
   char *acceptedWords[100];
   char *acceptedWordsMoney[100];
   int numberOfSW;
  
   
   if (my_env.verbose_switch==ON) {
     /* show the environment */
     int i;
     printf("----------------------------------------------------\n\n");
     
     printf("          class_path = %s\n",my_env.class_path);
     printf("       rug_class_path = %s\n",my_env.rug_class_path);
     printf("       tmp_file_path = %s\n\n",my_env.tmp_path);
     printf("       stopList_path = %s\n\n",my_env.stopList_path);

     printf("      verbose_switch = %s\n\n",(my_env.verbose_switch==ON?"ON":"OFF"));
   
     printf("           le_switch = %s\n",(my_env.le_switch==ON?"ON":"OFF"));
     printf("           ne_switch = %s\n",(my_env.ne_switch==ON?"ON":"OFF"));
     printf("           ev_switch = %s\n\n",(my_env.ev_switch==ON?"ON":"OFF"));
   
     printf("        class_number = %d\n\n",my_env.class_number);
     for (i=0; i<my_env.class_number ; i++)
       printf("   class_names[%d] = %s\n",i,my_env.class_names[i]);
     
     printf("\n----------------------------------------------------\n\n");
  }
  
  if((oracle=fopen("notOrderedOracle","w"))==NULL) {
      printf("  ERROR: Cannot create oracle\n");
      exit(-1);
  }
  
  sprintf(stopListFileName,"%s/%s",my_env.stopList_path,STOP_WORD_LIST);    

  numberOfSW=loadStopList(stopListFileName,stopWord,my_env.verbose_switch);

/*  acceptedWords[0]="grade";  
  acceptedWords[1]="mining";
  acceptedWords[2]="plant";
  acceptedWords[3]="production";
  acceptedWords[4]="raffiney";
  acceptedWords[5]="smelter";
  acceptedWords[6]="smelting";
  acceptedWords[7]="zinc";
  acceptedWords[8]="zincor";

acceptedWordsMoney[0]="accounts";
acceptedWordsMoney[1]="auctions";
acceptedWordsMoney[2]="bank";
acceptedWordsMoney[3]="banks";
acceptedWordsMoney[4]="bills";
acceptedWordsMoney[5]="bond";
acceptedWordsMoney[6]="bounds";
acceptedWordsMoney[7]="capital";
acceptedWordsMoney[8]="currencies";
acceptedWordsMoney[9]="currency";
acceptedWordsMoney[10]="debt";
acceptedWordsMoney[11]="debts";
acceptedWordsMoney[12]="exchange";
acceptedWordsMoney[13]="financial";
acceptedWordsMoney[14]="investment";
acceptedWordsMoney[15]="investments";
acceptedWordsMoney[16]="investors";
acceptedWordsMoney[17]="issues";
acceptedWordsMoney[18]="market";
acceptedWordsMoney[19]="markets";
acceptedWordsMoney[20]="money";
acceptedWordsMoney[21]="prices";
acceptedWordsMoney[22]="share";
acceptedWordsMoney[23]="shares";
acceptedWordsMoney[24]="shortage";
acceptedWordsMoney[25]="stock";
acceptedWordsMoney[26]="stocks";
acceptedWordsMoney[27]="surpluses";
acceptedWordsMoney[28]="trading";
*/
		  		  
 for(whichClass=0;whichClass<my_env.class_number;whichClass++) {

    task0(my_env,whichClass,"le", oracle,stopWord,numberOfSW);
   /* if (my_env.ne_switch==ON)
      task0( my_env,whichClass,"ne");
    if (my_env.ev_switch==ON)
      task0( my_env,whichClass,"ev");*/
  }
  
/*  task0( my_env,46,"le", oracle,acceptedWordsMoney,29);*/
  
  fclose(oracle);
  sprintf(command,"./bin/mysort_merge notOrderedOracle  > %s ; rm -f notOrderedOracle",my_env.oracle_path);
  system(command);
  shutdownStopList(stopWord,numberOfSW);
  
}
/* end */
