#include "classifier.h"
#include "base_lib.h"
#include <math.h>
#include <stdlib.h>	
#include <unistd.h>


void task1(struct class_ce_env my_env,int whichClass,char **idDocList,int numberOfSplittingDoc,
	   char *extension, FILE *oracle, float *trainingDocs, FILE *testset)
{
  char classFileName[PATH_LENGHT];
  char centroidFileName[PATH_LENGHT];
  char orderedCentroidFileName[PATH_LENGHT];
  char sortcommand[200],
       erasecommand[200];
  short results;
  
  if (my_env.verbose_switch==ON) 
    printf("\n\n\nBuilding centroid for class '%s'...\n\n",my_env.class_names[whichClass]);
  
  results=classSplitting(my_env,whichClass,idDocList,numberOfSplittingDoc,extension,testset);          

  sprintf(classFileName,"%s/%s.%s",my_env.splitted_class_path,my_env.class_names[whichClass],extension);

  sprintf(centroidFileName,"%s/%s.%s.ce",my_env.tmp_path,my_env.class_names[whichClass],extension);

  if (my_env.verbose_switch==ON) printf("\tLearning data file=%s\n",classFileName);
      
  if(results==1){
     if (buildCentroidForClass(my_env,classFileName,centroidFileName, oracle, trainingDocs)==0)
	return; /*the class file does not exist*/
     if (my_env.verbose_switch==ON) printf("\tCentroid file=%s written\n",centroidFileName);
  
    /* sort centroid on file */
      
    sprintf(orderedCentroidFileName,"%s/%s.%s.oce",my_env.centroid_path,my_env.class_names[whichClass],extension);

    sprintf(sortcommand,"./bin/mysort_merge %s > %s",centroidFileName,orderedCentroidFileName);
   /* printf("<%s>\n\n",sortcommand);	*/
    if(system(sortcommand)==0) {
       sprintf(erasecommand,"rm %s",centroidFileName);
       system(erasecommand);
    } else 
       if (my_env.verbose_switch==ON)  
	  printf("\nWarning : Centroid %s was not ordered therefore it did not be removed %s\n",centroidFileName);
     if (my_env.verbose_switch==ON) printf("\tOrdered Centroid file=%s written\n",orderedCentroidFileName);
 }
}


float multeplicity(FILE *oracleFile, char *idDoc){

 char idDocOracle[TERM_LENGHT];
 char cat[TERM_LENGHT];
 long pos;
 int count=0;
		
   fscanf(oracleFile,"%s\t%s\n",idDocOracle,cat);
   while(!feof(oracleFile) && strcmp(idDoc,idDocOracle)!=0)
	   fscanf(oracleFile,"%s\t%s\n",idDocOracle,cat);

   
   while(!feof(oracleFile) && strcmp(idDoc,idDocOracle)==0){
	   pos=ftell(oracleFile);
	   fscanf(oracleFile,"%s\t%s\n",idDocOracle,cat);
	   count++;
   }
   
   if(feof(oracleFile) && strcmp(idDoc,idDocOracle)==0 && count==0)count=1;
   
   fseek(oracleFile,pos,SEEK_SET);
   return count;
}
	

int buildCentroidForClass(struct class_ce_env my_env, char *classFileName,char *centroidFileName, FILE *oracleFile, float *totDocNumber)
 {	 
  
  struct pair *ce;

  char term[FREAD_BUFFER_SIZE];
  char syntClass[SYNTATCTIC_CLASS_DIGIT];
  int freq;
  char stopListFileName[PATH_LENGHT];     
  char documentId[TERM_LENGHT];
  char docIdOld[TERM_LENGHT];
  int i;
  int n;
  int alreadyPresent;
  int numberOfSW;
  char command[PATH_LENGHT];
  float mult;
 
  FILE *f1;
  
  n=0;

 
  /*TEST printf("\n file %s opening...\n\n",classFileName);*/
  if((f1=fopen(classFileName,"r"))==NULL) {
   if (my_env.verbose_switch==ON)printf("   Warning : file %s not found\n",classFileName);
      return 0;
  }

  i=0;
  while(!feof(f1)){
	fgets(term,FREAD_BUFFER_SIZE - 1,f1);
	i++;
  }

  ce=(struct pair *) malloc(sizeof(struct pair)*(i+1));
  
  fseek(oracleFile,0,SEEK_SET);
  fseek(f1,0,SEEK_SET);

  /* you can use the input file f1 */
  fscanf(f1,"%s",documentId);n=0;
  strcpy(ce[n].lastDocId,"");
  strcpy(docIdOld,documentId);
  mult=multeplicity(oracleFile,documentId);
  if(mult>0)(*totDocNumber)+=1.0/mult;
  
  while(!feof(f1)) {    
	  
     fscanf(f1,"%s",term);
     fscanf(f1,"%d\n",&freq);
     alreadyPresent=0;
     for(i=0;i<n;i++)
        if (strcmp(term,ce[i].item)==0) {
            /*TEST printf("key %s already present\n",term);*/
            ce[i].freq+=freq;
	    if(strcmp(documentId,ce[i].lastDocId)!=0){
	      ce[i].count+=1.0/mult;
	      if(ce[i].count<0) printf("\nnegative on term %s of doc %s\n",ce[i].item,documentId);
	      strcpy(ce[i].lastDocId,documentId);
      	    }  
	    i=n+1;alreadyPresent=1;
        }
     if(alreadyPresent==0){
          /*TEST printf("key %s added at the end\n",term);*/
	  ce[n].item = MYstrdup(term);     
          ce[n].freq=freq;
	  ce[n].count=1.0/mult;
	  strcpy(ce[n].lastDocId,documentId);
          n++;
	  /* TESTprintf("mult of doc %s %f\n",documentId,mult);*/
     }
     
     fscanf(f1,"%s",documentId);
     if(strcmp(documentId,docIdOld)!=0 && !feof(f1)) {
	     mult=multeplicity(oracleFile,documentId);
	     strcpy(docIdOld,documentId);
	     *totDocNumber+=1.0/mult;
     }
		    
  }
  fclose(f1);
     
  /* write centroid on file */
  if((f1=fopen(centroidFileName,"w"))==NULL)
  {
     if (my_env.verbose_switch==ON) printf("   Error : file %s write permission denied\n",centroidFileName);
     return 0;
  }
  /* you can use the output file f1 */
  for(i=0;i<n;i++)
  {
    fprintf(f1,"%s\t",ce[i].item);
    fprintf(f1,"%d\t",ce[i].freq);
    fprintf(f1,"%f\n",ce[i].count);
    MYfree(ce[i].item);
  }
  free(ce);
  fclose(f1);
  return 1;
}


void randomChoice(int indexes[], int max, int needed){
    int rnd;
    int i;
 
    printf("Number of learning documents: %d\n",max);
    printf("The held out set contains %d\n",needed);
    
    for(i=0;i<max;i++)indexes[i]=0;
    for(i=0;i<needed;i++){
	rnd = random()*(float)max/(float)RAND_MAX;
	while(indexes[rnd%max]!=0) rnd++;
	indexes[rnd%max]=1;
	/*printf("documento selezionato: %d \n",rnd%max);*/
	}
}


void ClassCentroidProcessing(struct class_ce_env my_env) 
{
	
   char **idDocList;
   char **splittingDocList;
   char idDoc[PATH_LENGHT];
   char Zero[PATH_LENGHT];
   char command[PATH_LENGHT];
   char docNumString[PATH_LENGHT];
   char lastDocumentIndexString[PATH_LENGHT];
   char oraclePath[PATH_LENGHT];
   char testset_file_name[PATH_LENGHT];
   int  index=0,
	numberOfCorpusDoc=0,
	numberOfTestDoc,
	numOfSplittingDocs=0,
	*randomDocIndex,
	i,
	k,
	lastDocumentIndex,
	whichClass,
	realNumberOfDocs;
   
   float seed,
	 trainingDocs=0;

   FILE *data, *oracle, *testset;
   
    sprintf(command,"cat %s | awk '{print $1}' | sort -u | wc -l > docNumber",my_env.oracle_path);
    system(command);
    
    if((data = fopen("docNumber","r"))==NULL){
	    printf("ERROR: problem with oracle file\n\n");
	    exit(-1);
    }
    
    fscanf(data,"%s",docNumString); 
    fclose(data);

    sprintf(command,"tail -1 %s | awk '{print $1}'  > docNumber",my_env.oracle_path);
    system(command);
    
    if((data = fopen("docNumber","r"))==NULL){
	    printf("ERROR: problem with oracle file\n\n");
	    exit(-1);
    }
    
    fscanf(data,"%s",lastDocumentIndexString); 
    fclose(data);
    system("rm -f docNumber");

    lastDocumentIndex=atoi(lastDocumentIndexString);

    numberOfCorpusDoc= lastDocumentIndex;// virtually the max number of documents is the last ID
    realNumberOfDocs = atoi(docNumString);

    randomDocIndex = (int *) MYmalloc(sizeof(int)*numberOfCorpusDoc);
        
    if (my_env.verbose_switch==ON) {
     /* show the environment */
	   
     printf("----------------------------------------------------\n\n");
     
     printf("          class_path = %s\n",my_env.class_path);
     printf("       centroid_path = %s\n",my_env.centroid_path);
     printf("       tmp_file_path = %s\n\n",my_env.tmp_path);
     printf("       oracle_path   = %s\n\n",my_env.oracle_path);
     printf("      verbose_switch = %s\n\n",(my_env.verbose_switch==ON?"ON":"OFF"));
   
     /*printf("           le_switch = %s\n",(my_env.le_switch==ON?"ON":"OFF"));
     printf("           ne_switch = %s\n",(my_env.ne_switch==ON?"ON":"OFF"));
     printf("           ev_switch = %s\n\n",(my_env.ev_switch==ON?"ON":"OFF"));*/
   
     printf("       test_doc_path = %s\n\n",my_env.test_doc_path);
     printf("        class_number = %d\n\n",my_env.class_number);
     
     printf("Number of corpus docs= %d\n\n",realNumberOfDocs);
    if(strcmp(my_env.testDocumentsIDfile,"-")!=0) 
     printf("testDocumentsIDfile  = %s\n\n",my_env.testDocumentsIDfile);
     else   {
	     printf("seed for rand. split = %s\n\n",my_env.seed);    
	     printf(" test_set_percentage = %d\n",my_env.test_set_percentage);
	    }
     
     for (i=0; i<my_env.class_number ; i++)
       printf("   class_names[%d] = %s\n",i,my_env.class_names[i]);
     
     printf("\n----------------------------------------------------\n\n");
  }


  if(my_env.test_set_percentage!=0){
/* At the moment we don-t use the test set docs: they are not in the oracle*/

      if (my_env.verbose_switch==ON) {printf("automatic generation of training documentID\n");}

      idDocList=(char **)MYmalloc(numberOfCorpusDoc*(sizeof(char*)));
      for(i=0;i<numberOfCorpusDoc;i++){
	      sprintf(idDoc,"%d",i);
	      strcpy(Zero,"");
	      for(k=strlen(idDoc);k<7;k++)strcat(Zero,"0");

	      idDocList[i]=(char*)MYstrdup(strcat(Zero,idDoc));
	      numOfSplittingDocs=numberOfCorpusDoc*my_env.test_set_percentage/100;
	      /*TEST printf("%s\n",idDocList[i]);*/
	      strcpy(idDoc,"");
	      }
	      
      splittingDocList=(char **)MYmalloc(numOfSplittingDocs*(sizeof(char*)));
      if(strcmp(my_env.seed,"-")==0) seed = time(NULL);
      else seed=atoi(my_env.seed);
      printf("\n Seed for random generation %f\n\n",seed);
      srandom(seed);
      randomChoice(randomDocIndex,numberOfCorpusDoc,numOfSplittingDocs);

      index=0;
      for(i=0;i<numberOfCorpusDoc;i++)
	  if(randomDocIndex[i]==1){
	  	splittingDocList[index]=(char *)MYstrdup(idDocList[i]);	
 		/* TESTprintf("%d, %s\n",index,splittingDocList[index]);*/
		index++;
	  }
      for(i=0;i<numberOfCorpusDoc;i++)MYfree(idDocList[i]);
      MYfree(idDocList);	
  }
  else 
    if(strcmp(my_env.testDocumentsIDfile,"-")!=0){ 
//	  printf("Number of corpus doc %d\n",numberOfCorpusDoc);
	  splittingDocList=(char **)MYmalloc(numberOfCorpusDoc*(sizeof(char*)));
	  sprintf(command,"sort -u %s > ordDocIDFile",my_env.testDocumentsIDfile);
	  system(command);
	  numOfSplittingDocs = loadDocIDList("ordDocIDFile", splittingDocList, my_env.verbose_switch);
	  system("rm -f ordDocIDFile");
        }
 
 
	
  if((oracle = fopen(my_env.oracle_path,"r"))==NULL){
	    printf("ERROR: problem with oracle file\n\n");
	    exit(-1);
    }
    	
  trainingDocs=0;  

  sprintf(testset_file_name,"%s/testItems.le",my_env.test_doc_path);    
  
  if((testset=fopen(testset_file_name,"w"))==NULL){
     if (my_env.verbose_switch==ON) printf("   Warning : cannot write the testset \n");
  }

  for(whichClass=0;whichClass<my_env.class_number;whichClass++) {
    if (my_env.le_switch==ON)
      task1( my_env,whichClass,splittingDocList,numOfSplittingDocs,"le", oracle, &trainingDocs,testset);
  }

  fclose(testset);

  sprintf(command,"sort -u %s/testItems.le > %s/testItems.temp; mv %s/testItems.temp %s/testItems.le",
  	  my_env.test_doc_path,my_env.test_doc_path,my_env.test_doc_path,my_env.test_doc_path);    
  system(command);
  
  fclose(oracle);
  if(my_env.test_set_percentage!=0 || strcmp(my_env.testDocumentsIDfile,"-")!=0){
  	for(i=0;i<numOfSplittingDocs;i++)MYfree(splittingDocList[i]);
  	MYfree(splittingDocList);
	}
	
  sprintf(command,"echo %d > %s/%s",(int)(trainingDocs+0.5),my_env.dict_path,INFO_DICTIONARY);
  system(command);
  sprintf(command,"echo %d >> %s/%s",lastDocumentIndex,my_env.dict_path,INFO_DICTIONARY);	
  system(command);
 
  free(randomDocIndex);
  printf("\n\n\nCentroids of Categories built !!!\n\n\n");
  
}
/* end */




