/***********************************************************************/
/*   Text Categorization Framework                                     */
/*                                                                     */
/*                                                                     */
/*   Author: Alessandro Moschitti                                      */
/*   moschitti@info.uniroma2.it                                        */
/*   Date: 30.11.03                                                    */
/*                                                                     */
/*   Copyright (c) 2003  Alessandro Moschitti - All rights reserved    */
/*                                                                     */
/*   This software is available for non-commercial use only. It must   */
/*   not be modified and distributed without prior permission of the   */
/*   author. The author is not responsible for implications from the   */
/*   use of this software.                                             */
/*                                                                     */
/***********************************************************************/



#include "classifier.h"
#include <math.h>
#include <string.h>
#include <stdlib.h>

/**********************************************************************************************
* 					Program OPTION					      *
***********************************************************************************************/

/*
#define SCUT
#undef RDS*/

#define SCUT
#undef RDS
#undef COMBINED_SCUT_RDS

/*#undef OUTPUT_STDSCARTI*/
#define OUTPUT_STDSCARTI
#define MAX_CICLE_NUMBER 15

int n=0;
/*********************************************************************************************/
/***				Heading of PROCEDURES					  ****/
/*********************************************************************************************/

void aProduct (char *termine, int freq, struct CKB_env *clKB, float *scalarProducts, double *docNorma,int
		*numOfOtherClass);
void takeAdecision(float a[],struct CKB_env *clKB,float threshold[],short d[]);



int loadClassificationKB(struct classifier_env my_env,char *extension,struct CKB_env *clKB) {
  char etichetta[TERM_LENGHT];
  char dictAbsName[PATH_LENGHT];
  char weightDictAbsName[PATH_LENGHT];
  char infoDictionariesAbsName[PATH_LENGHT];
  int i=0;

    
  FILE *infoDictionariesFile;
  FILE *offsetDictFile;
  FILE *weightDictFile;
   
  /* -------------------------------- */
  /* Reading Dictionaries information */
  /* -------------------------------- */
  if (my_env.verbose_switch==ON) printf ("Reading Dictionaries information\n\n");
  sprintf(infoDictionariesAbsName,"%s/%s.%s",my_env.dict_path,INFO_DICTIONARY,extension);
  if((infoDictionariesFile = fopen(infoDictionariesAbsName,"r"))==NULL) {
      printf("file %s does not found\n",infoDictionariesAbsName);
      return 0;
  }
  fscanf(infoDictionariesFile,"%s\n%d\n%s\n%d\n",
		              etichetta,&(clKB->totalDocumentNumber),
		              etichetta,&(clKB->differentWordNumber));
  clKB->centroidNumber=0;
  
    /* Centroid Names reading */

  fscanf(infoDictionariesFile,"%s\n",clKB->centroidNames[clKB->centroidNumber]);
  while(!feof(infoDictionariesFile)&& strcmp(clKB->centroidNames[clKB->centroidNumber],"Max_Weight=")!=0){
    if (my_env.verbose_switch==ON) 
	   printf("centroidNames[%d]=%s\n",clKB->centroidNumber,clKB->centroidNames[clKB->centroidNumber]);
    clKB->centroidNumber++;
    fscanf(infoDictionariesFile,"%s\n",clKB->centroidNames[clKB->centroidNumber]);
  }
  
  fscanf(infoDictionariesFile,"%f\n%s",&(clKB->maxWeight),etichetta);  /* max weight reading */
  
    /* Centroid total Frequences reading 
  i=0;
  fscanf(infoDictionariesFile,"%f\n",&clKB->centroidTotalFreqs[i]);
  while(!feof(infoDictionariesFile)&&i<clKB->centroidNumber){	  
    if (my_env.verbose_switch==ON) 
	   printf("centroidTotalFreq[%d]=%f\n",i,clKB->centroidTotalFreqs[i]);
    i++;
    fscanf(infoDictionariesFile,"%f\n",&clKB->centroidTotalFreqs[i]);
  }
  */
  
    /* Average, STD Deviation reading (of positive) 
  i=0;
  fscanf(infoDictionariesFile,"%s%f\n",etichetta,&clKB->positiv[i].average);
  while(!feof(infoDictionariesFile)&&i<clKB->centroidNumber){
    fscanf(infoDictionariesFile,"%f\n",&clKB->positiv[i].stdDeviation);	  
    if (my_env.verbose_switch==ON){
	   printf("positiv[%d].average=%f\t",i,clKB->positiv[i].average);
	   printf("positiv[%d].stdDeviation=%f\n",i,clKB->positiv[i].stdDeviation);
    }
    fscanf(infoDictionariesFile,"%f\n",&clKB->positiv[++i].average);
  }*/
  
    /* Average, STD Deviation reading of negative 
  i=0;
  fscanf(infoDictionariesFile,"%s%f\n",etichetta,&clKB->negativ[i].average);
  while(!feof(infoDictionariesFile)&&i<clKB->centroidNumber){
    fscanf(infoDictionariesFile,"%f\n",&clKB->negativ[i].stdDeviation);	  
    if (my_env.verbose_switch==ON){
	   printf("negativ[%d].average=%f\t",i,clKB->negativ[i].average);
	   printf("negativ[%d].stdDeviation=%f\n",i,clKB->negativ[i].stdDeviation);
    }
    fscanf(infoDictionariesFile,"%f\n",&clKB->negativ[++i].average);
  }*/
  
  fclose(infoDictionariesFile);
  
  
  /* -------------------------------- */
  /* Reading dictionaryOffset 	      */
  /* -------------------------------- */

  sprintf(dictAbsName,"%s/%s.%s",my_env.dict_path,OFFSET_DICTIONARY,extension);
  if((offsetDictFile = fopen(dictAbsName,"r"))==NULL) {
      printf("file %s does not found\n",dictAbsName);
      return 0;
  }
  sprintf(weightDictAbsName,"%s/%s.%s",my_env.dict_path,WEIGHT_DICTIONARY,extension);
  if((clKB->weightDictFile = fopen(weightDictAbsName,"r"))==NULL) {
      printf("file %s does not found\n",weightDictAbsName);
      return 0;
  }

  if (my_env.verbose_switch==ON) printf ("\n\n\nLoading Offset Dictionary ... \n"); 
  loadDictionaryOffset(offsetDictFile,&(clKB->differentWordNumber),&clKB->offsetDictArray);
  if (my_env.verbose_switch==ON) printf ("done !\n\n\n");    

  fclose(offsetDictFile);
  
  }

short sgn(float x){
	if(x>=0) return 1;
	else return -1;
 }
  


void shutdownClassificationKB(struct CKB_env *clKB){
  int i;
  	
  for(i=0;i<clKB->differentWordNumber;i++)
	  free(clKB->offsetDictArray[i].term);
  
  free(clKB->offsetDictArray);
  fclose(clKB->weightDictFile);
  
}

void shutdownOracle(struct oracleCell *oracle, int numOfdoc){
  int cellIndex,
      classIndex;
  	
  for(cellIndex=0;cellIndex<numOfdoc;cellIndex++){
     classIndex=0;
     while(oracle[cellIndex].targetClass[classIndex]!=NULL){
	  free(oracle[cellIndex].targetClass[classIndex]);
	  classIndex++;
     }
  free(oracle[cellIndex].targetClass);
  }
  free(oracle);
}
    

	   
void aProduct (char *term, int freq, struct CKB_env *clKB, float *scalarProducts, 
	       double *docNorma, int *numOfOtherClass) {
      
  int 	j;
   int c=0;
  struct couples setOfCouples;
  int numberOfCouple,count;
 	  
  float documentTermWeight,
 	IDF;
	   

  numberOfCouple =couplesRelativeToTerm(clKB->weightDictFile,term,
			           &setOfCouples,clKB->offsetDictArray,
			           clKB->differentWordNumber); 
  if(numberOfCouple!=0){

     IDF=log(((float)clKB->totalDocumentNumber)/(float)setOfCouples.numOfDocWithCurrentTerm);     
     
     documentTermWeight=ft(freq)*IDF;
     *docNorma+=(documentTermWeight * documentTermWeight);
  
     /* TEST   
     	printf("document Norma:%f\n",*docNorma);
    	 printf("\ntermine = %s, totalDocumentNumber=%d, occuredInDocNumb=%d, IDF=%f,
			   docWeight=%f\n",term,clKB->totalDocumentNumber,
			   setOfCouples.numOfDocWithCurrentTerm,IDF,
			   documentTermWeight);*/
    
		    
          for(j=0;j<numberOfCouple;j++) {
	     
	     	scalarProducts[setOfCouples.couple[j].centroidNumber]+=
				      (documentTermWeight * setOfCouples.couple[j].weight);

 		/*weightNorma[setOfCouples.couple[j].centroidNumber]+=
		     	  (setOfCouples.couple[j].weight * setOfCouples.couple[j].weight);*/
 		/* TEST   
 		if(setOfCouples.couple[j].centroidNumber==0){ // print category 0 only
	 		printf("term=%s\tprofile weight:%f\tdocumentTermWeight %f\t",
	 	 		term,setOfCouples.couple[j].weight,documentTermWeight);
	 		printf("product category(%d):%f\n",setOfCouples.couple[j].centroidNumber,
		       		(documentTermWeight * setOfCouples.couple[j].weight));
			// printf("document Norma:%f\n",*docNorma);
			//getchar(); 
		}
	       */
		
		count++; 
     	}
   }
 }  


void aDocumentScalarProduct(struct docVect *docCentroid, int centroidSize, char *extension,
		            struct CKB_env *clKB, float *scalarProducts) {
  int i,j,
      freq,
      numOfOtherClass=0;
  
  char 	term[PATH_LENGHT];
  
  float  weightNorma[MAX_CLASS_NUMBER];
  double docNorma,prova=0;
  
  
  /* reset scalar product and Norma */
  for(i=0;i<clKB->centroidNumber;i++)
     clKB->numOfTermExp[i]=scalarProducts[i]=weightNorma[i]=0;
  
  docNorma=0;
     /* TEST   
	printf("Centroid Size:%d\n",centroidSize);*/
  for(i=0;i<centroidSize;i++){
       freq=docCentroid[i].weight;
       strcpy(term,docCentroid[i].term);
       aProduct(term,freq,clKB,scalarProducts,&docNorma,&numOfOtherClass); 
           
   }
    
  /* Normalizzation of scalar product   */

    /*TEST	  printf("document Norma:%f\t",docNorma);*/

  for(i=0;i<clKB->centroidNumber;i++){
    /*TEST	  printf("prodotto alla %d_esima classe:%f\t",i,
		      scalarProducts[i]);*/
     if(docNorma>0)scalarProducts[i]/=sqrt(docNorma);
     /*TEST	  printf("prodotto alla %d_esima classe:%f\t",i,
		      scalarProducts[i]);*/

  }
    /*TEST printf("count:%d\n",count);*/

}


void takeAdecision(float a[],struct CKB_env *clKB,float threshold[],short d[]) {
  int i,j;
  float rowMean;
    	       
  for (j=0; j<clKB->centroidNumber ; j++){
		/*TEST printf ("\n classe %d ha %f term exp",j,clKB->numOfTermExp[j]);*/					  
    #ifdef RDS
	   rowMean=0;
	   for (i=0 ; i<clKB->centroidNumber ; i++)
    		if(i!=j)rowMean+=(a[j]-a[i]);
     	    rowMean/=(clKB->centroidNumber - 1);
	   d[j]=0;
	   if(rowMean>=threshold[j])d[j]=1;
	 
    #endif	  
	   
  #ifdef SCUT 
	
	if(a[j]>= threshold[j])d[j]=1;
    	     else d[j]=0;
	
  #endif	     
    
    
    #ifdef COMBINED_SCUT_RDS
	if(j!=86 && j!=21){
	if(a[j]>= threshold[j])d[j]=1;
    	     else d[j]=0;}
	 else{ 
	 rowMean=0;   
	 for (i=0 ; i<clKB->centroidNumber ; i++)
    		if(i!=j)rowMean+=(a[j]-a[i]);
	 if(rowMean>=threshold[j])d[j]=1;
	 else d[j]=0;}
    #endif	     
    }
    
}    
    
	
void takeAdecisionRanking(float a[], struct CKB_env *clKB, int topRank, short d[]) {
  
  int i,j,indMaxScore;
  float maxScore;
  
  indMaxScore=0;
  for(i=0; i<clKB->centroidNumber ; i++)d[i]=0;
  
  for (j=0; j<topRank ; j++){
     maxScore=0;
     for(i=0; i<clKB->centroidNumber ; i++){
	 /*TESTprintf("\n*****prob. della classe %d: %f ---- sp:%f\n\n",i,prob(a[i],i),a[i]); */
    	if(maxScore<a[i] && d[i]!=1){maxScore=a[i];indMaxScore=i;}
     }	
     d[indMaxScore]=1;        
  }
}     

		
			  
int computeNumberOfCorpusDoc(FILE *oracleFile)
{
 
 int numOfCorpusDoc=0;
 char class[MAX_CLASS_LENGHT];
 char idDoc[ID_DOC_LENGHT];
 char idDocOld[ID_DOC_LENGHT];
 
 fscanf(oracleFile,"%s",idDoc);
 (*idDocOld)='\0';
 while(!feof(oracleFile)){
      
      fscanf(oracleFile,"%s",class);
      if(strcmp(idDoc,idDocOld)!=0)numOfCorpusDoc++;
      strcpy(idDocOld,idDoc);
      fscanf(oracleFile,"%s",idDoc);
      
 }
 return numOfCorpusDoc;
}


int loadOracle(struct oracleCell **oracle, char oraclePath[PATH_LENGHT]) {
    
 int numOfCorpusDoc=0,cellIndex=0,
     classIndex,
     numOfClassPerDoc;
 char class[MAX_CLASS_LENGHT];
 char idDoc[ID_DOC_LENGHT];
 char idDocOld[ID_DOC_LENGHT];
 char classes[MAX_CLASS_NUMBER][MAX_CLASS_LENGHT];
 FILE *oracleFile;
 
   if((oracleFile = fopen(oraclePath,"r"))==NULL) {
		printf("Warning : file %s does not found\n",oraclePath);
		return 0;
     }
    else printf("File %s opened\n",oraclePath);  	
 
   numOfCorpusDoc=computeNumberOfCorpusDoc(oracleFile);
   /*TEST printf ("Number of Doc = %d\n",numOfCorpusDoc);*/
   (*oracle) = (struct oracleCell *) malloc(numOfCorpusDoc * sizeof(struct oracleCell));
   rewind(oracleFile);
   
   fscanf(oracleFile,"%s",idDoc);
   /*TEST printf ("idDoc %s\n",idDoc);*/
   while(!feof(oracleFile)){
      strcpy((*oracle)[cellIndex].idDoc,idDoc);
      numOfClassPerDoc=0;
      strcpy(idDocOld,idDoc);	   
      while(!feof(oracleFile) && strcmp(idDoc,idDocOld)==0){
	      fscanf(oracleFile,"%s\n",class);
	      /*TEST printf ("class %s\n",class);*/
	      strcpy(classes[numOfClassPerDoc],class);
	      numOfClassPerDoc++;
	      fscanf(oracleFile,"%s",idDoc);
	      /*TEST printf ("idDoc %s\n",idDoc);*/
      }
      (*oracle)[cellIndex].targetClass=(char **)malloc((numOfClassPerDoc+1)*sizeof(char *));
      (*oracle)[cellIndex].targetClass[numOfClassPerDoc]=NULL;
      for(classIndex=0;classIndex<numOfClassPerDoc;classIndex++)
	  (*oracle)[cellIndex].targetClass[classIndex]=strdup(classes[classIndex]);
      cellIndex++;
   }
   /*TEST  printf ("stampa oracolo \n");
   for(cellIndex=0;cellIndex<numOfCorpusDoc;cellIndex++){
	    classIndex=0;
	    printf("Document id: %s belong to: ",(*oracle)[cellIndex].idDoc);
	    while((*oracle)[cellIndex].targetClass[classIndex]!=NULL){
		    printf("%s\t",(*oracle)[cellIndex].targetClass[classIndex]);
		    classIndex++;
	    }
	    printf("\n");
   }*/	    
	  
   fclose(oracleFile);
		   
   return numOfCorpusDoc;
}  
	    	
	

/*Performance Computation*/


int computeCellContigencyTable(char *docName, struct contigencyCell *contigencyTable, char **oracleRow,
		                short *decision,struct CKB_env *clKB) {
int class, ret,
    targetClass;	

	for(class=0;class<clKB->centroidNumber;class++){
	 targetClass=0;
	 if(strcasecmp(clKB->centroidNames[class],"unknown")!=0){
 	    while(oracleRow[targetClass]!=NULL && 
		  strcasecmp(oracleRow[targetClass],clKB->centroidNames[class])!=0)
	    {
/*TEST	     printf("target %s != class
	     %s\n",oracleRow[targetClass],clKB->centroidNames[class]);
	     */
	     targetClass++;}
	     
/*TEST	   if(decision[class]==1)printf("Category <%s> has been choosen by classifier\n",clKB->centroidNames[class]);
	     
	     if(oracleRow[targetClass]!=NULL)
		     printf("Oracle tells the target document is in <%s>\n",oracleRow[targetClass]);
	     else printf("Category <%s> is not in the oracle set\n",clKB->centroidNames[class]);
	     
*/	    if(class==0)/*printf("Not recovered %s\n",docName); */ ret=(oracleRow[targetClass]!=NULL);
	     if(decision[class]==1)
	    	  if(oracleRow[targetClass]==NULL)contigencyTable[class].wrong++;
	    	  else contigencyTable[class].correct++;
	      else if(oracleRow[targetClass]!=NULL)
	      {contigencyTable[class].notRecovered++;
	        /*if(class==0)printf("Not recovered %s\n",docName); */ 
/*TEST 	       if (class==2) printf("\n\n\n-->%s<--\n",oracleRow[targetClass]);*/}
/*TEST  
	 
	 printf("correct %d, wrong %d not retrieved %d",contigencyTable[class].correct,
			    contigencyTable[class].wrong,contigencyTable[class].notRecovered);
	 getchar();
*/ 	 
        }
     }
     return ret;
}			

void computeRecallPrecison(struct contigencyCell *contigencyTable,
			   struct CKB_env *clKB,struct results *perClassPR, short required) {
	
 int class,
     wrong=0,
     notRecovered=0,
     correct=0;
 
 float  macroAverageRecall=0,
	macroAveragePrecision=0,
	microAverageRecall,
	microAveragePrecision,
	classRec,
	classPrec;
 
  /* non fare parteciare al calcolo le seguenti classi */
  int j;

 for(class=0;class<clKB->centroidNumber;class++){
	
    if(required) printf("\n%d): %13s\tcorrect: %d\t wrong: %d\t not retrieved: %d",class,clKB->centroidNames[class],
                          contigencyTable[class].correct,contigencyTable[class].wrong,
    			  contigencyTable[class].notRecovered);
    wrong+=contigencyTable[class].wrong;
    correct+=contigencyTable[class].correct;
    notRecovered+=contigencyTable[class].notRecovered;
  }
  
 microAverageRecall=(float)correct/(float)(correct+notRecovered);
 microAveragePrecision=(float)correct/(float)(correct+wrong);
 
 if(required)printf("\n\n                   	         %d\t\t%d\t\t\t%d\n\n",correct,wrong,notRecovered);
 		    
 for(class=0;class<clKB->centroidNumber;class++){
     if(contigencyTable[class].wrong+contigencyTable[class].notRecovered==0)
	     classRec=classPrec=1;      
     else{
	
	if(contigencyTable[class].notRecovered==0)classRec=1;
	else
	   classRec=((float)contigencyTable[class].correct)/((float)contigencyTable[class].correct+
	   				    (float)contigencyTable[class].notRecovered);
	   
	if(contigencyTable[class].wrong==0)classPrec=1; 
	else
	   classPrec=((float)contigencyTable[class].correct)/((float)contigencyTable[class].correct+
			   				    (float)contigencyTable[class].wrong);   		   										
     }

    if(required)printf("%d): %13s\tRec %f\tPrec. %f\t f1: %f\n",class,clKB->centroidNames[class],
	     classRec,classPrec,2*classRec*classPrec/(classRec+classPrec));

    perClassPR[class].precision=classPrec;
    perClassPR[class].recall=classRec;
    macroAverageRecall+=classRec;
    macroAveragePrecision+=classPrec;
}

	
 if(required)printf("\n\n           MacroAverage:\tRecall: %f\tPrecision: %f\n",macroAverageRecall/clKB->centroidNumber,
		 				     macroAveragePrecision/clKB->centroidNumber);
 if(required)printf("           MicroAverage:\tRecall: %f\tPrecision: %f\tf1: %f\n\n",microAverageRecall,microAveragePrecision,
	 		microAverageRecall*2*microAveragePrecision/(microAverageRecall+microAveragePrecision));	

}


void initContigencyTable(struct contigencyCell *contigencyTable,struct CKB_env *clKB) {
	
 int class;
 
	for(class=0;class<clKB->centroidNumber;class++)
		contigencyTable[class].correct=
		contigencyTable[class].notRecovered=
		contigencyTable[class].wrong=0;
}	


char *docClassName (char *docName,char *className)
{
 int i=0;
  
 while((docName[i]>'9' || docName[i]<'0')&& i<strlen(docName))
   className[i]=docName[i++];
 className[i]='\0';
 return className;
} 

void docClassNumber (char *docName,char *className)

{
 int i=0,numberStart=0;
 while((docName[i]>'9' || docName[i]<'0')&& i<strlen(docName))i++;
 numberStart=i;
 while((docName[i]<='9' && docName[i]>='0')&& i<strlen(docName))
   className[i-numberStart]=docName[i++];
 className[i-numberStart]='\0';
} 
 
int searchIdDoc(char *idDoc,struct oracleCell *oracle,int i, int j)
{
 if(i>j)return -1;
   else
    if(strcmp((char *)oracle[i+(j-i)/2].idDoc,idDoc)==0) return i+(j-i)/2;
      else 
       if(strcmp((char *)oracle[i+(j-i)/2].idDoc,idDoc)<0)
       	 return searchIdDoc(idDoc,oracle,i+(j-i)/2+1,j);
       else
       	 return searchIdDoc(idDoc,oracle,i,i+(j-i)/2-1);
 }  
  

 
  

int initTask5(struct classifier_env my_env, char *extension, struct oracleCell *oracle,
  	      float threshold[], struct results *perClassPR, struct CKB_env *my_CKB_env,
	      float  **allScalarProducts, int print){
	 
  int whichDocument, eachCat;
  int i;
  short oracleForFirstCat;
  long start;
  
  char  documentAbsName[PATH_LENGHT];
  char  scalarProductsAbsName[PATH_LENGHT];
  char  NumericScalarProductsAbsName[PATH_LENGHT];
  char  resultingOracleAbsFile[PATH_LENGHT];
  char  className [PATH_LENGHT];  
  char  documentID [PATH_LENGHT];
  struct docVect docCentroid[MAX_DOC_SIZE];
  int 	 centroidSize;
  
  struct catDocDB docDB[MAX_CLASS_NUMBER];
  
  FILE *scalarProductsFile;
  FILE *NumericScalarProductsFile;
  FILE *documentFile;
  FILE *resultingOracle;
  

  float scalarProducts[MAX_CLASS_NUMBER];
  short decision[MAX_CLASS_NUMBER];
  
  struct contigencyCell contigencyTable[MAX_CLASS_NUMBER];
  
  
  char idDoc[PATH_LENGHT];
  int idDocPos;  
  float remain=0;
  int pos=0;
    
   initContigencyTable(contigencyTable,my_CKB_env);
  
  /* -------------------------------------------- */
  /* Open Scalar Products file and create Header  */
  /* -------------------------------------------- */ 
   
  sprintf(scalarProductsAbsName,"%s/%s_test.%s",my_env.output_path,SCALAR_PRODUCTS,"txt");
  if((scalarProductsFile = fopen(scalarProductsAbsName,"w"))==NULL) {
      if (my_env.verbose_switch==ON) printf("file %s write permission denied\n",scalarProductsAbsName);
      exit(-1);
  }
  
  sprintf(NumericScalarProductsAbsName,"%s/%s_test.number.%s",my_env.output_path,SCALAR_PRODUCTS,extension);
  if((NumericScalarProductsFile = fopen(NumericScalarProductsAbsName,"w"))==NULL) {
      if (my_env.verbose_switch==ON) printf("file %s write permission denied\n",NumericScalarProductsAbsName);
      exit(-1);
  }
  

  sprintf(resultingOracleAbsFile,"%s/resultingOrale.txt",my_env.output_path);
  if((resultingOracle = fopen(resultingOracleAbsFile,"w"))==NULL) {
      if (my_env.verbose_switch==ON) printf("file %s write permission denied\n",resultingOracleAbsFile);
      exit(-1);
  }

  //fprintf(NumericScalarProductsFile," ");
  for(i=0;i<my_CKB_env->centroidNumber;i++)
	  fprintf(NumericScalarProductsFile,"%s ",my_CKB_env->centroidNames[i]);
  fprintf(NumericScalarProductsFile,"\n");

  /* ---------------------------- */
  /* Classify a set of documents  */
  /* ---------------------------- */
  
  n=0;  
  (*allScalarProducts)= (float *) malloc(sizeof(float)*my_env.document_number*my_CKB_env->centroidNumber);
  /* printf("\n consumo %d
  bytes",sizeof(float)*my_env.document_number*my_CKB_env->centroidNumber);*/

   
   if(strcmp(my_env.method,"KNN")==0){
      if(my_env.verbose_switch==ON) printf("\n\nLoading category for KNN model");	   
      /* in order to speed-up KNN classification load the categories in memory*/
      for(eachCat=0;eachCat<my_CKB_env->centroidNumber;eachCat++){
	      
	if(my_env.verbose_switch==ON) {printf(".");fflush(stdout);}
      	loadCategoryDB(my_env.splitted_class_path,my_CKB_env,eachCat,extension,&(docDB[eachCat].db),&(docDB[eachCat].size));
//       	if(my_env.verbose_switch==ON) printf("Document DB for category %s loaded\n", my_CKB_env->centroidNames[eachCat]);
        }
      }

   if(strcmp(my_env.test_file,"fragmented")==0){
	
    /* In this case the test files are split inside a directory  */	   
    printf("\nProcessing test documents ");
    for(whichDocument=0;whichDocument<my_env.document_number;whichDocument++) {
    
     sprintf(documentAbsName,"%s/%s.%s",my_env.test_doc_path,my_env.document_names[whichDocument],extension);
     if((documentFile = fopen(documentAbsName,"r"))==NULL) {
		printf("Warning : file %s does not found\n",documentAbsName);
		return 0;
     } 		         
       
   /*  if (my_env.verbose_switch==ON) printf ("\n  Classification of %s... \n\n",documentAbsName);*/
   
   if(strcmp(my_env.method,"KNN")==0){
      start=0;strcpy(documentID,"");
      centroidSize=buildExtractedDocumentCentroidWithWeight(documentFile,docCentroid,&start,documentID,my_CKB_env,-1);
      
      aDocScalarProductWithTrainingData(docDB,docCentroid,centroidSize,extension,my_CKB_env,     
		      ((*allScalarProducts)+whichDocument*my_CKB_env->centroidNumber),oracle, my_env.K_nearest);

      }
      else{ /* default is Rocchio */ 
	 centroidSize=buildDocumentCentroid(documentFile,docCentroid,extension);	   
	 aDocumentScalarProduct(docCentroid,centroidSize,extension,my_CKB_env,
	((*allScalarProducts)+whichDocument*my_CKB_env->centroidNumber));
      }
   
   fclose(documentFile);
   if(50*(whichDocument/50)==whichDocument)printf ("%d",whichDocument); 
	   else if(15*(whichDocument/15)==whichDocument)printf ("."); 
   fflush(stdout);

  /* Threshold POLICY 
*/     
   takeAdecision(((*allScalarProducts)+whichDocument*my_CKB_env->centroidNumber),my_CKB_env,threshold,decision);

		
  /* Ranking Policy document percentage 
     takeAdecisionRanking(scalarProducts,my_CKB_env,1,decision);        
  */
   
  /**** Compute Contigency Table ****/   
     
     strcpy(idDoc,"");
    
     strcpy(idDoc,my_env.document_names[whichDocument]);
     
/*TEST		printf("\n\nIl doc <%s>  appartiene a:\n",idDoc);*/
/*TEST 		printf("seeking doc : %s\n",idDoc);
/*TEST    if (my_env.verbose_switch==ON) printf ("  Contigency Cell ... "); */
	     
#ifdef  TESTING_PHASE
  
     idDocPos=searchIdDoc(idDoc,oracle,0,my_CKB_env->numOfCorpusDoc-1);
     
/*TEST		 printf("document found in loc %d\n",idDocPos);*/
     if(idDocPos==-1)
	     printf("Warning: The Document %s is not present in the ORACLE; did you select a wrong ORACLE?\n",idDoc);
     else 
    oracleForFirstCat = computeCellContigencyTable(idDoc,contigencyTable,oracle[idDocPos].targetClass,decision,my_CKB_env);
 /*    if (my_env.verbose_switch==ON) printf ("written !\n"); */
     
#endif

#ifdef     OUTPUT_STDSCARTI
              
     /*** Score output of Classifier ***/

     fprintf(scalarProductsFile,"%s %s",
	docClassName(my_env.document_names[whichDocument],className),idDoc);
	
     fprintf(NumericScalarProductsFile,"%s",my_env.document_names[whichDocument]);

     for(i=0;i<my_CKB_env->centroidNumber;i++){
	     fprintf(scalarProductsFile," %d %f-->%d",oracleForFirstCat,((*allScalarProducts)+whichDocument*my_CKB_env->centroidNumber)[i],decision[i]);
	     if(decision[i])fprintf(resultingOracle,"%s %s\n",my_env.document_names[whichDocument],my_CKB_env->centroidNames[i]);
     }
     for(i=0;i<my_CKB_env->centroidNumber;i++)
   	 if(decision[i])fprintf(NumericScalarProductsFile," *%1.10f",((*allScalarProducts)+whichDocument*my_CKB_env->centroidNumber)[i]);
   	 else fprintf(NumericScalarProductsFile," %1.10f",((*allScalarProducts)+whichDocument*my_CKB_env->centroidNumber)[i]);

     fprintf(NumericScalarProductsFile,"\n");
     fprintf(scalarProductsFile,"\n");

#endif
    }
  }
  
  else{
      /* classify a sigle file which contains all the test documents */
	  
      /* in my_env.test_doc_path devo mettere il percorso del file di documenti */
	  

     sprintf(documentAbsName,"%s/%s.%s",my_env.test_doc_path,my_env.test_file,extension);

     if((documentFile = fopen(documentAbsName,"r"))==NULL) {
		printf("Warning : file %s does not found\n",documentAbsName);
		return 0;
     } 		
			       
     printf("\nClassify all the %s file which contains all the test documents\n",documentAbsName);
     
     pos=0;whichDocument=0;
     strcpy(documentID,"");
     while (!feof(documentFile)){
     	centroidSize=buildExtractedDocumentCentroid(documentFile,docCentroid, &pos, documentID);
     	/*if (my_env.verbose_switch==ON) printf ("\n  Classification of %s... ",documentID);*/
     	aDocumentScalarProduct(docCentroid,centroidSize,extension,my_CKB_env,((*allScalarProducts)+whichDocument*my_CKB_env->centroidNumber));
     
	takeAdecision(((*allScalarProducts)+whichDocument*my_CKB_env->centroidNumber),my_CKB_env,threshold,decision);

     /**** Compute Contigency Table ****/   

	strcpy(idDoc,"");
       /* docClassNumber(my_env.document_names[whichDocument],idDoc);*/
	strcpy(idDoc,documentID);
	my_env.document_names[whichDocument]=(char *)strdup(idDoc);

#ifdef  TESTING_PHASE
   
	idDocPos=searchIdDoc(idDoc,oracle,0,my_CKB_env->numOfCorpusDoc-1);

	if(idDocPos==-1)
		printf("Warning: The Document %s is not present in the ORACLE; did you select a wrong ORACLE?\n",idDoc);
	else 
	computeCellContigencyTable(idDoc,contigencyTable,oracle[idDocPos].targetClass,decision,my_CKB_env);
       /* if (my_env.verbose_switch==ON) printf ("written !\n");*/
#endif

	/*TEST   if (my_env.verbose_switch==ON)printf("document %s num. %d\n",documentID,	whichDocument); */
   #ifdef     OUTPUT_STDSCARTI
	/*** Output of Classifier **
	if (my_env.verbose_switch==ON) printf ("  Writing output ... "); */

	fprintf(scalarProductsFile,"%s %s",
	   docClassName(my_env.document_names[whichDocument],className),idDoc);
	fprintf(NumericScalarProductsFile,"%s",my_env.document_names[whichDocument]);
	for(i=0;i<my_CKB_env->centroidNumber;i++)
	   fprintf(scalarProductsFile," %d",decision[i]);      
	for(i=0;i<my_CKB_env->centroidNumber;i++)
   	   if(decision[i])fprintf(NumericScalarProductsFile," *%1.10f",((*allScalarProducts)+whichDocument*my_CKB_env->centroidNumber)[i]);
   	   else fprintf(NumericScalarProductsFile," %1.10f",((*allScalarProducts)+whichDocument*my_CKB_env->centroidNumber)[i]);
	fprintf(NumericScalarProductsFile,"\n");
	fprintf(scalarProductsFile,"\n");
       /* if (my_env.verbose_switch==ON) printf ("done !\n"); */
#endif
       whichDocument++;
    }
    fclose(documentFile);
  }
  /*close the Scalar Product file */
  
  fclose(scalarProductsFile);
  fclose(NumericScalarProductsFile);
  fclose(resultingOracle);
  printf("\n%s %s written\n",scalarProductsAbsName,NumericScalarProductsAbsName);

  
  /* Shutdown the Classification Knowledge Bases */
   if(strcmp(my_env.method,"KNN")==0)
     for(eachCat=0;eachCat<my_CKB_env->centroidNumber;eachCat++)
  	     shutdownDB(docDB[eachCat].db,docDB[eachCat].size);

#ifdef  TESTING_PHASE
  computeRecallPrecison(contigencyTable,my_CKB_env,perClassPR, print);
#endif  TESTING_PHASE
    printf("done!\n");
  return my_CKB_env->centroidNumber;
}



void task5(struct classifier_env my_env, char *extension, struct oracleCell *oracle,
	   float threshold[],struct results *perClassPR, struct CKB_env *my_CKB_env, float
		  *allScalarProducts, int print ){ 
  int whichDocument;
  int i;
  int r;
  
  char  documentAbsName[PATH_LENGHT];
  char  scalarProductsAbsName[PATH_LENGHT];
  char  NumericScalarProductsAbsName[PATH_LENGHT];
  char  className [PATH_LENGHT];  
  struct docVect docCentroid[MAX_DOC_SIZE];
  int 	 centroidSize;
  
  FILE *scalarProductsFile;
  FILE *NumericScalarProductsFile;
  FILE *documentFile;
  

  float scalarProducts[MAX_CLASS_NUMBER];
  short decision[MAX_CLASS_NUMBER];
  
  struct contigencyCell contigencyTable[MAX_CLASS_NUMBER];
   
  
  char 	idDoc[PATH_LENGHT];
  int idDocPos;  
  float remain=0;

    
   initContigencyTable(contigencyTable,my_CKB_env);
  
  /* ------------------------------------------------------ */
  /* Classify a set of documents with a different threshold */
  /* -------------------------------------------------------*/
   
  n=0;  
  
  for(whichDocument=0;whichDocument<my_env.document_number;whichDocument++) {
     
    
   takeAdecision((allScalarProducts+whichDocument*my_CKB_env->centroidNumber),my_CKB_env,threshold,decision);
   


 /* if( ((float)random())/ ((float)RAND_MAX) >.3)
takeAdecisionRanking((allScalarProducts+whichDocument*my_CKB_env->centroidNumber),my_CKB_env,1,decision);
/*    else takeAdecisionRanking((allScalarProducts+whichDocument*my_CKB_env->centroidNumber),my_CKB_env,2,decision) ;*/
  
  /**** Compute Contigency Table ****/   
     
  strcpy(idDoc,my_env.document_names[whichDocument]);
  
#ifdef  TESTING_PHASE
  
  idDocPos=searchIdDoc(idDoc,oracle,0,my_CKB_env->numOfCorpusDoc-1);
  if(idDocPos==-1)
	     printf("Warning: The Document %s is not present in the ORACLE; did you select a wrong ORACLE?\n",idDoc);
  else 
     computeCellContigencyTable(idDoc,contigencyTable,oracle[idDocPos].targetClass,decision,my_CKB_env);
   /*  if (my_env.verbose_switch==ON) printf ("written !\n"); */
  }
   computeRecallPrecison(contigencyTable,my_CKB_env,perClassPR,print);
#endif 
  
}

double f_measure(double p, double r){
 if(p+r==0) return 0;
 return 2*p*r/(p+r);
} 

 void Classification(struct classifier_env my_env) 
  {
   int i,
       classNumber,
       NumberOfClasses;
   float		   
	threshold[MAX_CLASS_NUMBER],
	old_gamma[MAX_CLASS_NUMBER],
	old_BEP[MAX_CLASS_NUMBER],
	old_threshold[MAX_CLASS_NUMBER];
     		
   float RP_Difference,
	 current_gamma,
	 gamma_new,
	 threshold_new,
	 BEP_new;	
  
   float
	 MagThreshold[MAX_CLASS_NUMBER],
	 MinThreshold[MAX_CLASS_NUMBER];

   struct oracleCell *oracle;
   
   struct CKB_env my_CKB_env;  
		
   struct results perClassPR[MAX_CLASS_NUMBER];
   short finish;
   int count;
   char  learningType[4];
   char  *gammaStr;
   
   float *allScalarProducts;
   char thres_string[PATH_LENGHT];
   char gamma_string[PATH_LENGHT];
   char BEP_string[PATH_LENGHT];
   char thresOfLastGamma[PATH_LENGHT];

   FILE *thresholdFile,*gammaFile;
   
      /* show the environment*/
   if (my_env.verbose_switch==ON) {
	   
        printf("     Classification with %s model\n",my_env.method);
     	if(strcmp(my_env.method,"KNN")==0)
		printf("                   k = %d\n",my_env.K_nearest);
	printf("           dict_path = %s\n",my_env.dict_path);
   	printf("         output_path = %s\n",my_env.output_path);
   	printf("       test_doc_path = %s\n",my_env.test_doc_path);
   	printf("            tmp_path = %s\n\n",my_env.tmp_path);
   
   	printf("      verbose_switch = %s\n\n",(my_env.verbose_switch==ON?"ON":"OFF"));
   
/*   	printf("           le_switch = %s\n",(my_env.le_switch==ON?"ON":"OFF"));
   	printf("           ne_switch = %s\n",(my_env.ne_switch==ON?"ON":"OFF"));
   	printf("           ev_switch = %s\n\n",(my_env.ev_switch==ON?"ON":"OFF"));
*/   
   	printf("     Number of documents = %d\n\n",my_env.document_number);
     

   	if (strcmp(my_env.test_file,"-")!=0)
	     printf("    test document in file = %s\n\n",my_env.test_file);
    	else for (i=0; i<my_env.document_number ; i++)
     	     printf("    document_names[%d] = %s\n",i,my_env.document_names[i]);
   }
   
   current_gamma=1;	
   printf("\n\n\n");
   
   if (my_env.breakEvenPoint_flag==ON) {
	 if(strcmp(my_env.method,"Rocchio")==0){
   	   if((gammaStr=getenv("gamma"))!=NULL)current_gamma=atof(gammaStr);
	   else printf("\nWARNING: the env. gamma variable is empty (1 is assumed as default)\n");
   	   printf("Best gamma Evaluation - current Gamma %f \n\n\n",current_gamma);
         }
         printf("Break-even point computation !!! \n");
   }
   else {
	  printf("Only one run (f1-measure on test-set computed)\n");
	  printf("Thresholds will be loaded from file: %s\n",my_env.threshold_file);
	}	    
   
   if (my_env.le_switch==ON)strcpy(learningType,"le");
   if (my_env.ne_switch==ON)strcpy(learningType,"ne");
   if (my_env.ev_switch==ON)strcpy(learningType,"ev");
			   			   
       
  /* -------------------------------------------- */
  /*    		Load oracle		  */
  /* -------------------------------------------- */

  my_CKB_env.numOfCorpusDoc=loadOracle(&oracle,my_env.oracle_path);
  
    
  /* -------------------------------------------- */
  /*    Load the classification Knowledge Bases   */
  /* -------------------------------------------- */

  loadClassificationKB(my_env,learningType,&my_CKB_env);
  
 
  /* -------------------------------------------- */
  /*   	Load Thresholds and gammaFile	  	  */
  /* -------------------------------------------- */

  if((thresholdFile = fopen(my_env.threshold_file,"r"))==NULL){
	   if (my_env.breakEvenPoint_flag!=ON){
		   printf("\nERROR: the thresholds file <%s> is missing turn on -BP option to an optimal evalutation on current test-documents",
				   thresholdFile);
		   exit(-1);
		}
	   for(i=0;i<MAX_CLASS_NUMBER;i++)old_threshold[i]=old_gamma[i]= old_BEP[i]=0;
	}
  else
      {
	i=0;
	if (my_env.verbose_switch==ON) printf("Loading threshold, gamma and performance form file:<%s>\n\n",
	my_env.threshold_file);
	while(!feof(thresholdFile)){
		
		/* the first field of thresholds files is the BEP threshold of last test-set evaluation (i.e. related to the last gamma)*/
		/* the second is the threshold of the best gamma tried til now */
		/* the third and forth are the best gamma and the performance associated to it */
		 
		fscanf(thresholdFile,"%s\t%s\t%s\t%s\n",thresOfLastGamma,thres_string,gamma_string,BEP_string);
	   	old_threshold[i]=threshold[i]=atof(thres_string);
		old_gamma[i]=atof(gamma_string);
		old_BEP[i]=atof(BEP_string);
		if (my_env.verbose_switch==ON) printf("%d)%f\t\t%f\t\t%f\n",i,
			old_threshold[i],
			old_gamma[i],
			old_BEP[i]);
		i++;
	   }
	fclose(thresholdFile);	
     			
  }

/* -------------------------------------------- */
/*   	Set threshold for BEP computation  	    */
/* -------------------------------------------- */


  if (my_env.breakEvenPoint_flag==ON)
	for(i=0;i<MAX_CLASS_NUMBER;i++) {
	   MagThreshold[i]=0;
	   MinThreshold[i]=1;
	   threshold[i]=(MagThreshold[i]+MinThreshold[i])/2;
	}
	   	
   
/* -------------------------------------------- */
/*   	Make a classification of all TEST_SET   */
/* -------------------------------------------- */

  finish=0;count=0;
  NumberOfClasses=initTask5(my_env,learningType,oracle,
		    threshold,perClassPR,&my_CKB_env,&allScalarProducts,my_env.verbose_switch);
  
/* -------------------------------------------- */
/* Repeat classification if BEP is requireed    */
/* -------------------------------------------- */

  while(finish==0 && count < MAX_CICLE_NUMBER && (my_env.breakEvenPoint_flag==ON)){
    finish=1;count++;
    task5(my_env,learningType,oracle,threshold,perClassPR,
		    &my_CKB_env,allScalarProducts,my_env.verbose_switch);	  
      for(classNumber=0;classNumber<NumberOfClasses;classNumber++){
	RP_Difference=(perClassPR[classNumber].recall-perClassPR[classNumber].precision);
	   
	if(RP_Difference*sgn(RP_Difference)>0.0001 || 
	      (perClassPR[classNumber].recall==perClassPR[classNumber].precision &&
			perClassPR[classNumber].recall==0)){
		
		if(RP_Difference>0)MagThreshold[classNumber]=threshold[classNumber];
		else MinThreshold[classNumber]=threshold[classNumber];
		
		if(count>MAX_CICLE_NUMBER-5 && (perClassPR[classNumber].precision<.10 ||
				perClassPR[classNumber].recall<.1))threshold[classNumber]=10000;   
	   	else{
		     threshold[classNumber]=(MinThreshold[classNumber]+MagThreshold[classNumber])/2;
	   	     finish=0;
	            } 
	 }
      
      }
 
 if(my_env.verbose_switch==ON){
    printf("\n\t\t\t Final Thresholds\n\n");     
    printf("\nThreshold[]={%f",threshold[0]);
    for(classNumber=1;classNumber<NumberOfClasses;classNumber++)
	printf(",%f",threshold[classNumber]);   
    printf("};\n");
 }

  }

/* ---------------------------------------------------------- */
/*  classification of all test-set (only one time)            */
/* ---------------------------------------------------------- */

  
  task5(my_env,learningType,oracle,threshold,perClassPR,
		    &my_CKB_env,allScalarProducts,1);
  
/*    printf("\nMinThreshold[]={%f",MinThreshold[0]);
    for(classNumber=1;classNumber<NumberOfClasses;classNumber++)
	printf(",%f",MinThreshold[classNumber]);   
    printf("};\n");
    
    printf("\nMajThreshold[]={%f",MagThreshold[0]);
    for(classNumber=1;classNumber<NumberOfClasses;classNumber++)
	printf(",%f",MagThreshold[classNumber]);   
    printf("};\n");	
    
    printf("\n\n----------------------------------------------------------------------------------------------\n\n");	*/
		    
  printf("\nThreshold[]={%f",threshold[0]);
  for(classNumber=1;classNumber<NumberOfClasses;classNumber++)
    printf(",%f",threshold[classNumber]);   
  printf("};\n");

  if (my_env.breakEvenPoint_flag==ON && current_gamma!=-1) {
	if((thresholdFile = fopen(my_env.threshold_file,"w"))==NULL) {
		printf("Warning: the threshold file %s not writeble\n",thresholdFile);
		exit(-1);
	}
	if((gammaFile = fopen(GAMMA_FILE,"w"))==NULL){
		printf("Warning: the threshold file %s not writeble\n",GAMMA_FILE);
		exit(-1);
	}		

      for(classNumber=0;classNumber<NumberOfClasses;classNumber++){
	BEP_new=f_measure(perClassPR[classNumber].recall,perClassPR[classNumber].precision);

	if(old_BEP[classNumber]>=BEP_new){
		BEP_new=old_BEP[classNumber];
	        gamma_new=old_gamma[classNumber];
	        threshold_new=old_threshold[classNumber];
	}
	else{
		gamma_new=current_gamma;
        	threshold_new=threshold[classNumber];
	}
	
	fprintf(thresholdFile,"%f\t%f\t%f\t%f\n",threshold[classNumber],threshold_new,gamma_new,BEP_new);
	fprintf(gammaFile,"%f\n",gamma_new);
      }    	 
 
	fclose(thresholdFile);
	fclose(gammaFile); 
   }
   
   /* Shutdown the Classification Knowledge Bases */

  shutdownClassificationKB(&my_CKB_env);
  shutdownOracle(oracle,my_CKB_env.numOfCorpusDoc);
}
