/***********************************************************************/
/*   Text Categorization Framework                                     */
/*                                                                     */
/*                                                                     */
/*   Author: Alessandro Moschitti                                      */
/*   moschitti@info.uniroma2.it                                        */
/*   Date: 30.11.03                                                    */
/*                                                                     */
/*   Copyright (c) 2003  Alessandro Moschitti - All rights reserved    */
/*                                                                     */
/*   This software is available for non-commercial use only. It must   */
/*   not be modified and distributed without prior permission of the   */
/*   author. The author is not responsible for implications from the   */
/*   use of this software.                                             */
/*                                                                     */
/***********************************************************************/


/**
  * LearningManager.c
  */
/*
    run usage :
                 run
	or
                 run -HELP
	to obtain usage information
	
	 	 run UNI [-CL<class_path>] [-UC<class_centroid_path>] [-TM<tmp_file_path>]
                         [-VERB] <mode> 
                         -C (ALL | <class1> <class2> ... <classN>)
	to calculate the class centroid(s).

                 run CCE [-CL<class_path>] [-CE<class_centroid_path>] [-TM<tmp_file_path>]
                         [-VERB] <mode> 
                         [-SP<percentage>] [-TD<test_doc_path>]
                         -C (ALL | <class1> <class2> ... <classN>)
	to calculate the class centroid(s).

                 run GCE [-CE<class_centroid_path>] [-GC<global_centroid_path>] [-TM<tmp_file_path>]
                         [-VERB] <mode>
	to calculate the global centroid.

                       where <mode> is one or more of [-LE] [-NE] [-EV]
                         and <percentage> is an integer value in [1..99]
			 

  correct parameters set for CCE
  ------------------------------
    run CCE -CLclass_path -CEcentroid_path -VERB  -SP2 -TDtest_doc_path -LE -EV -C class1 class2 classN
    run CCE -CLclass_path -CEcentroid_path -VERB  -SP2 -TDtest_doc_path -LE -EV -C ALL 
    run CCE -CLclass_path -CEcentroid_path -VERB  -SP2 -TDtest_doc_path -LE -EV -C ALL class1
    run CCE -CEcentroid_path -VERB  -SP2 -TDtest_doc_path -LE -EV -C class1 class2
    run CCE -CEcentroid_path -SP2 -TDtest_doc_path -EV -C class1 class2
    run CCE -EV -C class1 
    run CCE -EV -C ALL 

  wrong parameters set for CCE
  ----------------------------
    run CCE -CLclass_path -CEcentroid_path -VERB  -SP-2 -TDtest_doc_path -LE -EV -C class1 class2 classN
    run CCE -CLclass_path -CEcentroid_path -VERB  -SP0 -TDtest_doc_path -LE -EV -C class1 class2 classN
    run CCE -CLclass_path -CEcentroid_path -VERB  -SP100 -TDtest_doc_path -LE -EV -C class1 class2 classN
    run CCE -CLclass_path -CEcentroid_path -VERB  -SP110 -TDtest_doc_path -LE -EV -C class1 class2 classN
      ERROR <percentage> value is not valid

    run CCE -CEcentroid_path -SP2 -TDtest_doc_path -C class1 class2
      ERROR : <mode> parameter must be present

    run CCE -CEcentroid_path -SP2 -TDtest_doc_path -EV -C
    run CCE -CEcentroid_path -SP2 -TDtest_doc_path -EV
      ERROR : at least one class required 

    run CCE -ET -C class1 
      ERROR : Unknow or invalid parameter '-ET'

  correct parameters set for GCE
  ------------------------------
    run GCE -CEcentroid_path -VERB -LE -EV 
    run GCE -CEcentroid_path -EV 
    run GCE -VERB -LE 
    run GCE -EV 

  wrong parameters set for GCE
  ----------------------------
    run GCE -CEcentroid_path -VERB 
      ERROR : <mode> parameter must be present

    run GCE -EV -C class1 
      ERROR : Unknow or invalid parameter '-C'
    run GCE -CLclass_path -VERB
      ERROR : Unknow or invalid parameter '-CLclass_path'

*/
  
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "LearningManager.h"
#include "base_lib.h"

void PrintUsage(char *ProgramName)
{
    printf("\n%s usage :\n",ProgramName);
    
    printf("%20s\n",ProgramName);
    printf("\tor\n");
    printf("%20s -HELP\n",ProgramName);
    printf("\tto obtain usage information\n\n");
    
    printf("%20s UNI [-CL<class_path>] [-RC<rug_class_path>] \n",ProgramName);
    printf("%20s     [-TM<tmp_file_path>] [-VERB] <mode> \n","");
    printf("%20s     -C (ALL | <class1> <class2> ... <classN>)\n","");
    printf("\tto unifie lemma of the class(s).\n\n");
    
    printf("%20s CCE [-CL<class_path>] [-CE<class_centroid_path>]\n",ProgramName);
    printf("%20s     [-TM<tmp_file_path>] [-VERB] <mode> \n","");
    printf("%20s     [-SP<percentage>] [-TD<test_doc_path>]\n","");
    printf("%20s     -C (ALL | <class1> <class2> ... <classN>)\n","");
    printf("\tto calculate the class centroid(s).\n\n");
    
    printf("%20s GCE [-CE<class_centroid_path>] [-GC<global_centroid_path>] [-TM<tmp_file_path>]\n",ProgramName);
    printf("%20s     [-VERB] <mode>\n","");
    printf("\tto calculate the global centroid.\n\n");
    
    printf("%20s DIC [-CL<class_path>] [-CE<class_centroid_path>] [-GC<global_centroid_path>]\n",ProgramName);
    printf("%20s     [-DI<dictionaries_file_path>] [-TM<tmp_file_path>] [-SP<percentage>]\n","");
    printf("%20s     [-GA<integer>] [-VERB] <mode>\n","");
    printf("\tto calculate the dictionaries.\n\n");

    printf("%20s CLA [-DI<dictionaries_file_path>] [-TD<test_doc_path>]\n",ProgramName);
    printf("%20s     [-O<output_path>] [-TM<tmp_file_path>]\n",ProgramName);
    printf("%20s     [-VERB] -TR<threshold> <mode>\n","");
    printf("%20s     -D (ALL | <doc1> <doc2> ... <docM>)\n","");
    printf("\tto classify the documents.\n\n");
    
    printf("%20s CLU [-DI<dictionaries_file_path>] [-SC <splitted_class_path>] \n",ProgramName);
    printf("%20s     [-TM<tmp_file_path>]\n",ProgramName);
    printf("%20s     [-OC<OUTPUT clusters>] [-IC<INPUT clusters>]\n");
    printf("%20s     [-NC<number of Clusters>][-VERB] <mode>\n");
    printf("%20s     -D (ALL | <class1> <class2> ... <classN>)\n","");
    printf("\tto cluster the class(es).\n\n");
    
    printf("%20s   where <mode> is one or more of [-LE] [-NE] [-EV]\n","");
    printf("%20s     and <percentage> is an integer value in [1..99]\n\n","");
    printf("%20s     and <threshold> is an float value \n\n","");

}
void cutExtensions(char *fileName)
{
 char *chp;
 chp=strchr(fileName,'.');
 if (chp==NULL) {
   printf("	file name %s extension required\n",fileName);
   exit(-1); 
 }
 *chp='\0';	
}
main(int argc,char *argv[])
{ 
  int ArgNumb;
  int i;
  char *AR_DATA;
  char *DOMAIN;
  char fileNamesPath[PATH_LENGHT];
  char fileName[PATH_LENGHT];
  char command[PATH_LENGHT];
  char temp[PATH_LENGHT];
  FILE *fnames;


  /* Help request */ 
  if ( (argc==1) || ( (argc==2) && (strcmp(argv[1],"-HELP")==0) ) ) {
    PrintUsage(argv[0]);
    exit(0);
  }
  
  AR_DATA=(char *)getenv("AR_DATA");
  if (AR_DATA==NULL) {
     fprintf(stderr," ERROR : $AR_DATA environment variable undefined\n");
     exit(-1);
  }  
  /*TEST printf(" $AR_DATA = %s\n",AR_DATA);*/
  
  DOMAIN=(char *)getenv("DOMAIN");
  if (DOMAIN==NULL) {
     fprintf(stderr," ERROR : $DOMAIN environment variable undefined\n");
     exit(-1);
  }
  /*TESTprintf(" $DOMAIN = %s\n",DOMAIN);*/

  /* look at the first argument. it must be CCE or GCE*/
  if (strcmp(argv[1],"UNI")==0) {
     /*-----------------------------*/ 
     /*  class centroids evaluation */ 
     /*-----------------------------*/ 
     /*            
      *         <command> UNI [-RC<rug_class_path>] [-CL<class_path>] [-TM<tmp_file_path>]
      *                  [-VERB] <mode> 
      *                  -C (ALL | <class1> <class2> ... <classN>)
      */
			 
     /* define a variable to store the environment information */
     struct class_env current_env;
     
     /* set each environment field with a NOT valid value */
     strcpy(current_env.class_path,"-");
     strcpy(current_env.rug_class_path,"-");
     strcpy(current_env.tmp_path,"-");
     strcpy(current_env.stopList_path,"-");

     current_env.verbose_switch=(-1);

     current_env.le_switch=(-1);
     current_env.ne_switch=(-1);
     current_env.ev_switch=(-1);
         
     /* read the command line parameters until "-C" */
     for (ArgNumb=2 ; ArgNumb<argc ; ArgNumb++) {
  
       if (strncmp(argv[ArgNumb],"-CL",3)==0) {
         if (strcmp(current_env.class_path,"-")!=0) 
           {printf("Two -CL<class_path> parameters\n");exit(-1);}
         strcpy(current_env.class_path,argv[ArgNumb]+3);
	 
       } else if (strncmp(argv[ArgNumb],"-RC",3)==0) {
         if (strcmp(current_env.rug_class_path,"-")!=0) 
           {printf("Two -RC<rug_class_path> parameters\n");exit(-1);}
         strcpy(current_env.rug_class_path,argv[ArgNumb]+3);

       } else if (strncmp(argv[ArgNumb],"-SL",3)==0) {
         if (strcmp(current_env.stopList_path,"-")!=0) 
           {printf("Two -SL<stopList_path> parameters\n");exit(-1);}
         strcpy(current_env.stopList_path,argv[ArgNumb]+3);

       } else if (strncmp(argv[ArgNumb],"-TM",3)==0) {
         if (strcmp(current_env.tmp_path,"-")!=0) 
           {printf("Two -TM<tmp_file_path> parameters\n");exit(-1);}
         strcpy(current_env.tmp_path,argv[ArgNumb]+3);

       } else if (strcmp(argv[ArgNumb],"-VERB")==0) {
         if (current_env.verbose_switch!=(-1)) 
           {printf("Two -VERB parameters\n");exit(-1);}
         current_env.verbose_switch=ON;

       } else if (strcmp(argv[ArgNumb],"-LE")==0) {
         if (current_env.le_switch!=(-1)) 
           {printf("Two -LE parameters\n");exit(-1);}
         current_env.le_switch=ON;

       } else if (strcmp(argv[ArgNumb],"-NE")==0) {
         if (current_env.ne_switch!=(-1)) 
           {printf("Two -NE parameters\n");exit(-1);}
         current_env.ne_switch=ON;

       } else if (strcmp(argv[ArgNumb],"-EV")==0) {
         if (current_env.ev_switch!=(-1)) 
           {printf("Two -EV parameters\n");exit(-1);}
         current_env.ev_switch=ON;

       } else if (strcmp(argv[ArgNumb],"-C")==0) {
         break;
	 
       } else  {
        printf("ERROR : Unknow or invalid parameter '%s'\n",argv[ArgNumb]);
	PrintUsage(argv[0]);
        exit(-1);
       }
     }

     /* set default values for unspecified environment fields*/
     
     if (strcmp(current_env.class_path,"-")==0)
          sprintf(current_env.class_path,"%s/%s/%s",AR_DATA,DOMAIN,CLASSES_PATH);
     if (strcmp(current_env.rug_class_path,"-")==0)
          sprintf(current_env.rug_class_path,"%s/%s/%s",AR_DATA,DOMAIN,RUG_CLASS_PATH);
     if (strcmp(current_env.tmp_path,"-")==0)
          sprintf(current_env.tmp_path,"%s/%s/%s",AR_DATA,DOMAIN,TEMPORARY_PATH);
     if (strcmp(current_env.stopList_path,"-")==0)
	  sprintf(current_env.stopList_path,"%s/%s",AR_DATA,DOMAIN);   
      
     if (current_env.verbose_switch==(-1)) current_env.verbose_switch=OFF;
      
     if ( (current_env.le_switch==(-1)) && 
	   (current_env.ne_switch==(-1)) && 
           (current_env.ev_switch==(-1)) ) {
        printf("\nERROR : <mode> parameter must be present\n");
	PrintUsage(argv[0]);
        exit(-1);
     } 
      
     if (current_env.le_switch==(-1))   current_env.le_switch=OFF;
     if (current_env.ne_switch==(-1))   current_env.ne_switch=OFF;
     if (current_env.ev_switch==(-1))   current_env.ev_switch=OFF;
      
     /* read the class list or ALL*/
     current_env.class_number=0;
     
     ArgNumb++;  
     /* ArgNumb is now the index of the first class */
     /* name parameter or the last parameter 'ALL'  */
     if ( (ArgNumb==(argc-1)) && (strcmp(argv[ArgNumb],"ALL")==0) ) {
	
	sprintf(fileNamesPath,"%s/%s",current_env.tmp_path,FILE_OF_CLASS_NAMES);
	sprintf(command,"ls %s > %s",current_env.rug_class_path,fileNamesPath);
	system(command);	
        fnames=fopen(fileNamesPath,"r");
        if(fnames==NULL)
          {
           printf("file %s does not found\n",fileNamesPath);
           exit(-1);
          }
	  
	current_env.class_number=0;
	current_env.class_names=(char **)MYmalloc(sizeof(char *)*(MAX_CLASS_NUMBER));
        fscanf(fnames,"%s",fileName);
	if(strcmp(fileName,"")==0) printf("\tERROR: At least one class is needed/n");
       while(!feof(fnames))
         { 	  	 
	  cutExtensions(fileName);
	  for(i=0;i<current_env.class_number;i++)
	     if(strcmp(fileName,current_env.class_names[i])==0)i=MAX_CLASS_NUMBER+1;
	  if(i==current_env.class_number)
	     current_env.class_names[current_env.class_number++]=MYstrdup(fileName);
	  fscanf(fnames,"%s",fileName);
	 }
	 /* realloc with current_env.class_number size end */
	/* ------------------------------------------------------------------------- */  
     } else {
	current_env.class_names=(char **)MYmalloc(sizeof(char *)*(argc-ArgNumb)); 
        for (; ArgNumb<argc ; ArgNumb++)
	  current_env.class_names[current_env.class_number++]=MYstrdup(argv[ArgNumb]);
     }
     if ( current_env.class_number==0 ) {
        printf("\nERROR :  at least one class required\n");
        exit(-1);
     } 
     
     ClassUnificationProcessing(current_env);
       
  } else if (strcmp(argv[1],"CCE")==0) {
     /*-----------------------------*/ 
     /*  class centroids evaluation */ 
     /*-----------------------------*/ 
     /*            
      *         <command> CCE [-CL<class_path>] [-CE<centroid_path>] [-TM<tmp_file_path>]
      *                  [-VERB] <mode> 
      *                  [-SP<percentage>] [-TD<test_doc_path>]
      *                  -C (ALL | <class1> <class2> ... <classN>)
      */
			 
     /* define a variable to store the environment information */
     struct class_ce_env current_env;
     
     /* set each environment field with a NOT valid value */
     strcpy(current_env.class_path,"-");
     strcpy(current_env.centroid_path,"-");
     strcpy(current_env.tmp_path,"-");
     strcpy(current_env.class_path,"-");
     strcpy(current_env.splitted_class_path,"-");
     
     current_env.verbose_switch=(-1);

     current_env.le_switch=(-1);
     current_env.ne_switch=(-1);
     current_env.ev_switch=(-1);
     
     current_env.test_set_percentage=(-1);
     strcpy(current_env.test_doc_path,"-");
    
     /* read the command line parameters until "-C" */
     for (ArgNumb=2 ; ArgNumb<argc ; ArgNumb++) {
  
       if (strncmp(argv[ArgNumb],"-CL",3)==0) {
         if (strcmp(current_env.class_path,"-")!=0) 
           {printf("Two -CL<class_path> parameters\n");exit(-1);}
         strcpy(current_env.class_path,argv[ArgNumb]+3);
	 
       } else if (strncmp(argv[ArgNumb],"-CE",3)==0) {
         if (strcmp(current_env.centroid_path,"-")!=0) 
           {printf("Two -CE<centroid_path> parameters\n");exit(-1);}
         strcpy(current_env.centroid_path,argv[ArgNumb]+3);

       } else if (strncmp(argv[ArgNumb],"-SC",3)==0) {
         if (strcmp(current_env.centroid_path,"-")!=0) 
           {printf("Two -SP<splitted_class_path> parameters\n");exit(-1);}
         strcpy(current_env.splitted_class_path,argv[ArgNumb]+3);

       } else if (strncmp(argv[ArgNumb],"-TM",3)==0) {
         if (strcmp(current_env.tmp_path,"-")!=0) 
           {printf("Two -TM<tmp_file_path> parameters\n");exit(-1);}
         strcpy(current_env.tmp_path,argv[ArgNumb]+3);

       } else if (strcmp(argv[ArgNumb],"-VERB")==0) {
         if (current_env.verbose_switch!=(-1)) 
           {printf("Two -VERB parameters\n");exit(-1);}
         current_env.verbose_switch=ON;

       } else if (strcmp(argv[ArgNumb],"-LE")==0) {
         if (current_env.le_switch!=(-1)) 
           {printf("Two -LE parameters\n");exit(-1);}
         current_env.le_switch=ON;

       } else if (strcmp(argv[ArgNumb],"-NE")==0) {
         if (current_env.ne_switch!=(-1)) 
           {printf("Two -NE parameters\n");exit(-1);}
         current_env.ne_switch=ON;

       } else if (strcmp(argv[ArgNumb],"-EV")==0) {
         if (current_env.ev_switch!=(-1)) 
           {printf("Two -EV parameters\n");exit(-1);}
         current_env.ev_switch=ON;

       } else if (strncmp(argv[ArgNumb],"-SP",3)==0) {
         if (current_env.test_set_percentage!=(-1)) 
           {printf("Two -SP<percentage> parameters\n");exit(-1);}
         current_env.test_set_percentage=atoi(argv[ArgNumb]+3);
         if (current_env.test_set_percentage<=0 || (current_env.test_set_percentage>=100)) 
           {printf("ERROR <percentage> value is not valid\n");exit(-1);}

       } else if (strncmp(argv[ArgNumb],"-TD",3)==0) {
         if (strcmp(current_env.test_doc_path,"-")!=0) 
           {printf("Two -TD<test_doc_path> parameters\n");exit(-1);}
        strcpy(current_env.test_doc_path,argv[ArgNumb]+3);
	 
       } else if (strcmp(argv[ArgNumb],"-C")==0) {
         break;
	 
       } else  {
        printf("ERROR : Unknow or invalid parameter '%s'\n",argv[ArgNumb]);
	PrintUsage(argv[0]);
        exit(-1);
       }
     }

     /* set default values for unspecified environment fields*/
     
     if (strcmp(current_env.class_path,"-")==0)
          sprintf(current_env.class_path,"%s/%s/%s",AR_DATA,DOMAIN,CLASSES_PATH);
     if (strcmp(current_env.centroid_path,"-")==0)
          sprintf(current_env.centroid_path,"%s/%s/%s",AR_DATA,DOMAIN,CLASS_CENTROID_PATH);
     if (strcmp(current_env.tmp_path,"-")==0)
          sprintf(current_env.tmp_path,"%s/%s/%s",AR_DATA,DOMAIN,TEMPORARY_PATH);
     if (strcmp(current_env.splitted_class_path,"-")==0)
	  sprintf(current_env.splitted_class_path,"%s/%s/%s",AR_DATA,DOMAIN,SPLITTED_CLASS_PATH);   
      
     if (current_env.verbose_switch==(-1))   current_env.verbose_switch=OFF;
      
     if ( (current_env.le_switch==(-1)) && 
	   (current_env.ne_switch==(-1)) && 
           (current_env.ev_switch==(-1)) ) {
        printf("\nERROR : <mode> parameter must be present\n");
	PrintUsage(argv[0]);
        exit(-1);
     } 
      
     if (current_env.le_switch==(-1))   current_env.le_switch=OFF;
     if (current_env.ne_switch==(-1))   current_env.ne_switch=OFF;
     if (current_env.ev_switch==(-1))   current_env.ev_switch=OFF;
      
     if (current_env.test_set_percentage==(-1)) current_env.test_set_percentage=0;
      
     if (strcmp(current_env.test_doc_path,"-")==0)
         sprintf(current_env.test_doc_path,"%s/%s/%s",AR_DATA,DOMAIN,TEST_DOC_PATH);
     
     /* read the class list or ALL*/
     current_env.class_number=0;
     
     ArgNumb++;  
     /* ArgNumb is now the index of the first class */
     /* name parameter or the last parameter 'ALL'  */
     if ( (ArgNumb==(argc-1)) && (strcmp(argv[ArgNumb],"ALL")==0) ) {
	/* identify all the class name in 'current_env.class_path' */
	/* ------------------------------------------------------------------------- */  
	/* Alessandro modifica fino ad 'end' */
	sprintf(fileNamesPath,"%s/%s",current_env.tmp_path,FILE_OF_CLASS_NAMES);
	sprintf(command,"ls %s > %s",current_env.class_path,fileNamesPath);
	system(command);	
        fnames=fopen(fileNamesPath,"r");
        if(fnames==NULL)
          {
           printf("file %s does not found\n",fileNamesPath);
           exit(-1);
          }
	  
	current_env.class_number=0;
	current_env.class_names=(char **)MYmalloc(sizeof(char *)*(MAX_CLASS_NUMBER));
        fscanf(fnames,"%s",fileName); 
       while(!feof(fnames))
         { 	  	 
	  cutExtensions(fileName);
	  for(i=0;i<current_env.class_number;i++)
	     if(strcmp(fileName,current_env.class_names[i])==0)i=MAX_CLASS_NUMBER+1;
	  if(i==current_env.class_number)
	     current_env.class_names[current_env.class_number++]=MYstrdup(fileName);
	  fscanf(fnames,"%s",fileName);
	 }
	 /* realloc with current_env.class_number size end */
	/* ------------------------------------------------------------------------- */  
     } else {
	current_env.class_names=(char **)MYmalloc(sizeof(char *)*(argc-ArgNumb)); 
        for (; ArgNumb<argc ; ArgNumb++)
	  current_env.class_names[current_env.class_number++]=MYstrdup(argv[ArgNumb]);
     }
     if ( current_env.class_number==0 ) {
        printf("\nERROR :  at least one class required\n");
        exit(-1);
     } 
     
     ClassCentroidProcessing(current_env);
       
  } else if (strcmp(argv[1],"GCE")==0) {
	  
     /*-----------------------------*/ 
     /* global centroids evaluation */ 
     /*-----------------------------*/ 
     /*            
      *           <command> GCE [-CE<class_centroid_path>] [-GC<global_centroid_path>]
      *                  [-VERB] <mode>
      */
	  
     /* define a variable to store the environment information */
     struct global_ce_env current_env;
     
     /* set each environment field with a NOT valid value */
     strcpy(current_env.centroid_path,"-");
     strcpy(current_env.global_centroid_path,"-");
     strcpy(current_env.tmp_path,"-");
     
     current_env.verbose_switch=(-1);

     current_env.le_switch=(-1);
     current_env.ne_switch=(-1);
     current_env.ev_switch=(-1);
     
     
     /* read the command line parameters until "-C" */
     for (ArgNumb=2 ; ArgNumb<argc ; ArgNumb++) {
  
       if (strncmp(argv[ArgNumb],"-CE",3)==0) {
         if (strcmp(current_env.centroid_path,"-")!=0) 
           {printf("Two -CE<centroid_path> parameters\n");exit(-1);}
         strcpy(current_env.centroid_path,argv[ArgNumb]+3);

       } else if (strncmp(argv[ArgNumb],"-GC",3)==0) {
         if (strcmp(current_env.global_centroid_path,"-")!=0) 
           {printf("Two -GC<global_centroid_path> parameters\n");exit(-1);}
         strcpy(current_env.global_centroid_path,argv[ArgNumb]+3);

       } else if (strncmp(argv[ArgNumb],"-TM",3)==0) {
         if (strcmp(current_env.tmp_path,"-")!=0) 
           {printf("Two -TM<tmp_file_path> parameters\n");exit(-1);}
         strcpy(current_env.tmp_path,argv[ArgNumb]+3);

       } else if (strcmp(argv[ArgNumb],"-VERB")==0) {
         if (current_env.verbose_switch!=(-1)) 
           {printf("Two -VERB parameters\n");exit(-1);}
         current_env.verbose_switch=ON;

       } else if (strcmp(argv[ArgNumb],"-LE")==0) {
         if (current_env.le_switch!=(-1)) 
           {printf("Two -LE parameters\n");exit(-1);}
         current_env.le_switch=ON;

       } else if (strcmp(argv[ArgNumb],"-NE")==0) {
         if (current_env.ne_switch!=(-1)) 
           {printf("Two -NE parameters\n");exit(-1);}
         current_env.ne_switch=ON;

       } else if (strcmp(argv[ArgNumb],"-EV")==0) {
         if (current_env.ev_switch!=(-1)) 
           {printf("Two -EV parameters\n");exit(-1);}
         current_env.ev_switch=ON;

       } else {
        printf("ERROR : Unknow or invalid parameter '%s'\n",argv[ArgNumb]);
	PrintUsage(argv[0]);
        exit(-1);
       }
     }

     /* set default values for unspecified environment fields*/
     
     if (strcmp(current_env.centroid_path,"-")==0)
          sprintf(current_env.centroid_path,"%s/%s/%s",AR_DATA,DOMAIN,CLASS_CENTROID_PATH);
     if (strcmp(current_env.global_centroid_path,"-")==0)
          sprintf(current_env.global_centroid_path,"%s/%s/%s",AR_DATA,DOMAIN,GLOBAL_CENTROID_PATH);
     if (strcmp(current_env.tmp_path,"-")==0)
          sprintf(current_env.tmp_path,"%s/%s/%s",AR_DATA,DOMAIN,TEMPORARY_PATH);
      
     if (current_env.verbose_switch==(-1))   current_env.verbose_switch=OFF;
      
     if ( (current_env.le_switch==(-1)) && 
	   (current_env.ne_switch==(-1)) && 
           (current_env.ev_switch==(-1)) ) {
        printf("\nERROR : <mode> parameter must be present\n");
	PrintUsage(argv[0]);
        exit(-1);
     } 
      
     if (current_env.le_switch==(-1))   current_env.le_switch=OFF;
     if (current_env.ne_switch==(-1))   current_env.ne_switch=OFF;
     if (current_env.ev_switch==(-1))   current_env.ev_switch=OFF;
           
     /* identify all the class name looking at centroids in 'current_env.centroid_path' */
     
     /* ------------------------------------------------------------------------- */  
     /* Alessandro modifica fino ad 'end' */  
        current_env.centroid_number=0;
        sprintf(fileNamesPath,"%s/%s",current_env.tmp_path,FILE_OF_CENTROID_NAMES);
	sprintf(command,"ls %s > %s",current_env.centroid_path,fileNamesPath);	
	system(command);	
	fnames=fopen(fileNamesPath,"r");
        if(fnames==NULL)
          {
           printf("file %s does not found\n",fileNamesPath);
           exit(-1);
          }
	  
	current_env.centroid_names=(char **)MYmalloc(sizeof(char *)*(MAX_CLASS_NUMBER));
        fscanf(fnames,"%s",fileName);
        while(!feof(fnames))
         { 	  	 
	  cutExtensions(fileName);
	  for(i=0;i<current_env.centroid_number;i++)
	     if(strcmp(fileName,current_env.centroid_names[i])==0)i=MAX_CLASS_NUMBER+1;
	  if(i==current_env.centroid_number)
	     current_env.centroid_names[current_env.centroid_number++]=MYstrdup(fileName);
	  fscanf(fnames,"%s",fileName);
	 }
      /* end */
     /* ------------------------------------------------------------------------- */  
    
     if ( current_env.centroid_number==0 ) {
        printf("\nERROR :  at least one centroid required\n");
        exit(-1);
     } 
     GlobalCentroidProcessing(current_env);
	  
  } else if (strcmp(argv[1],"DIC")==0) {
	  
     /*-----------------------------*/ 
     /* Dictionaries Generation     */ 
     /*-----------------------------*/ 
     /*            
      *           <command> DIC [-SC<splitted_class_centroid_path>] [-GC<global_centroid_path>]
      *                         [-DI<dictionaries_file_path>] [-TM<tmp_file_path>]
      *                         [-VERB] <mode>
      */
	  
     /* define a variable to store the environment information */
     struct dict_env current_env;
     
       /* set each environment field with a NOT valid value */
     
     strcpy(current_env.splitted_class_path,"-");
     strcpy(current_env.global_centroid_path,"-");
     strcpy(current_env.dict_path,"-");
     strcpy(current_env.tmp_path,"-");
     strcpy(current_env.gamma,"-");
     
     current_env.verbose_switch=(-1);
     current_env.le_switch=(-1);
     current_env.ne_switch=(-1);
     current_env.ev_switch=(-1);
     
     
     /* read the command line parameters until "-C" */
     for (ArgNumb=2 ; ArgNumb<argc ; ArgNumb++) {
  
        if (strncmp(argv[ArgNumb],"-SC",3)==0) {
         if (strcmp(current_env.splitted_class_path,"-")!=0) 
           {printf("Two -SC<class_path> parameters\n");exit(-1);}
         strcpy(current_env.splitted_class_path,argv[ArgNumb]+3);
	 
       } else if (strcmp(argv[ArgNumb],"-VERB")==0){
         if (current_env.verbose_switch!=(-1)) 
           {printf("Two -VERB parameters\n");exit(-1);}
         current_env.verbose_switch=ON;
	 
       }  else if (strncmp(argv[ArgNumb],"-GC",3)==0) {
         if (strcmp(current_env.global_centroid_path,"-")!=0) 
           {printf("Two -GC<global_centroid_path> parameters\n");exit(-1);}
         strcpy(current_env.global_centroid_path,argv[ArgNumb]+3);

       } else if (strncmp(argv[ArgNumb],"-DI",3)==0) {
         if (strcmp(current_env.dict_path,"-")!=0) 
           {printf("Two -DI<dictionaries_file_path> parameters\n");exit(-1);}
         strcpy(current_env.dict_path,argv[ArgNumb]+3);

       } else if (strncmp(argv[ArgNumb],"-TM",3)==0) {
         if (strcmp(current_env.tmp_path,"-")!=0) 
           {printf("Two -TM<tmp_file_path> parameters\n");exit(-1);}
         strcpy(current_env.tmp_path,argv[ArgNumb]+3);

       } else if (strcmp(argv[ArgNumb],"-LE")==0) {
         if (current_env.le_switch!=(-1)) 
           {printf("Two -LE parameters\n");exit(-1);}
         current_env.le_switch=ON;
	 
       } else if (strcmp(argv[ArgNumb],"-NE")==0) {
         if (current_env.ne_switch!=(-1)) 
           {printf("Two -NE parameters\n");exit(-1);}
         current_env.ne_switch=ON;

       } else if (strcmp(argv[ArgNumb],"-EV")==0) {
         if (current_env.ev_switch!=(-1)) 
           {printf("Two -EV parameters\n");exit(-1);}
         current_env.ev_switch=ON;

       } else if (strncmp(argv[ArgNumb],"-GA",3)==0) {
         if (strcmp(current_env.gamma,"-")!=0) 
           {printf("Two -GA parameters\n");exit(-1);}
         strcpy(current_env.gamma,argv[ArgNumb]+3);
       }  else {
        printf("ERROR : Unknow or invalid parameter '%s'\n",argv[ArgNumb]);
	PrintUsage(argv[0]);
        exit(-1);
       }
     }

     /* set default values for unspecified environment fields*/
     
     if (strcmp(current_env.global_centroid_path,"-")==0)
          sprintf(current_env.global_centroid_path,"%s/%s/%s",AR_DATA,DOMAIN,GLOBAL_CENTROID_PATH);
     if (strcmp(current_env.dict_path,"-")==0)
          sprintf(current_env.dict_path,"%s/%s/%s",AR_DATA,DOMAIN,DICTIONARIES_PATH);
     if (strcmp(current_env.tmp_path,"-")==0)
          sprintf(current_env.tmp_path,"%s/%s/%s",AR_DATA,DOMAIN,TEMPORARY_PATH);
     if (strcmp(current_env.splitted_class_path,"-")==0)
          sprintf(current_env.splitted_class_path,"%s/%s/%s",AR_DATA,DOMAIN,SPLITTED_CLASS_PATH);
     if (strcmp(current_env.splitted_class_path,"-")==0) strcpy(current_env.gamma,"1");
      
     if (current_env.verbose_switch==(-1))   current_env.verbose_switch=OFF; 
     
     if ( (current_env.le_switch==(-1)) && 
	   (current_env.ne_switch==(-1)) && 
           (current_env.ev_switch==(-1)) ) {
        printf("\nERROR : <mode> parameter must be present\n");
	PrintUsage(argv[0]);
        exit(-1);
     } 
      
     if (current_env.le_switch==(-1))   current_env.le_switch=OFF;
     if (current_env.ne_switch==(-1))   current_env.ne_switch=OFF;
     if (current_env.ev_switch==(-1))   current_env.ev_switch=OFF;
               
     /* identify all the class name looking at centroids in 'current_env.centroid_path' */
     
     /* ------------------------------------------------------------------------- */  
 
        current_env.centroid_number=0;
        sprintf(fileNamesPath,"%s/%s",current_env.tmp_path,FILE_OF_CENTROID_NAMES);
	system(command);	
	fnames=fopen(fileNamesPath,"r");
        if(fnames==NULL)
          {
           printf("file %s does not found\n",fileNamesPath);
           exit(-1);
          }
	  
	current_env.centroid_names=(char **)MYmalloc(sizeof(char *)*(MAX_CLASS_NUMBER));
        fscanf(fnames,"%s",fileName);
        while(!feof(fnames))
         { 	  	 
	  cutExtensions(fileName);
	  for(i=0;i<current_env.centroid_number;i++)
	     if(strcmp(fileName,current_env.centroid_names[i])==0)i=MAX_CLASS_NUMBER+1;
	  if(i==current_env.centroid_number)
	     current_env.centroid_names[current_env.centroid_number++]=MYstrdup(fileName);
	  fscanf(fnames,"%s",fileName);
	 }
	 
	 
	 sprintf(fileNamesPath,"%s/%s",current_env.tmp_path,FILE_OF_CLASS_NAMES);
	sprintf(command,"ls %s > %s",current_env.splitted_class_path,fileNamesPath);
	system(command);	
        fnames=fopen(fileNamesPath,"r");
        if(fnames==NULL)
          {
           printf("file %s does not found\n",fileNamesPath);
           exit(-1);
          }
	  
	current_env.class_number=0;
	current_env.class_names=(char **)MYmalloc(sizeof(char *)*(MAX_CLASS_NUMBER));
        fscanf(fnames,"%s",fileName); 
       while(!feof(fnames))
         { 	  	 
	  cutExtensions(fileName);
	  for(i=0;i<current_env.class_number;i++)
	     if(strcmp(fileName,current_env.class_names[i])==0)i=MAX_CLASS_NUMBER+1;
	  if(i==current_env.class_number)
	     current_env.class_names[current_env.class_number++]=MYstrdup(fileName);
	  fscanf(fnames,"%s",fileName);
	 }
      /* end */
     /* ------------------------------------------------------------------------- */  
     if ( current_env.centroid_number==0 ) {
        printf("\nERROR :  at least one centroid required\n");
        exit(-1);
     }
     
     if ( current_env.class_number==0 ) {
        printf("\nError :  at least one class required to compute the thresholds classification\n");
        exit(-1);
     } 
     DictionariesBuilding(current_env);
	  
  } else if (strcmp(argv[1],"CLA")==0) {
     /*-----------------------------*/ 
     /*  class centroids evaluation */ 
     /*-----------------------------*/ 
     /*            
      *         <command> CLA [-DI<dictionaries_file_path>] [-TM<tmp_file_path>] [-TD<test_doc_path>]
      *                       [-VERB] <mode> 
      *                       -D (ALL | <doc1> <doc2> ... <docM>)
      */
			 
     /* define a variable to store the environment information */
     struct classifier_env current_env;
     
    
     /* set each environment field with a NOT valid value */
     strcpy(current_env.dict_path,"-");
     strcpy(current_env.test_doc_path,"-");
     strcpy(current_env.tmp_path,"-");
     strcpy(current_env.output_path,"-");
     strcpy(current_env.oracle_path,"-");
     
     current_env.verbose_switch=(-1);

     current_env.le_switch=(-1);
     current_env.ne_switch=(-1);
     current_env.ev_switch=(-1);
     current_env.breakEvenPoint_flag=0;

     /* read the command line parameters until "-C" */
     for (ArgNumb=2 ; ArgNumb<argc ; ArgNumb++) {
  
       if (strncmp(argv[ArgNumb],"-BP",3)==0) current_env.breakEvenPoint_flag=1;
       else	 
        if (strncmp(argv[ArgNumb],"-DI",3)==0) {
         if (strcmp(current_env.dict_path,"-")!=0) 
           {printf("Two -DI<dictionaries_file_path> parameters\n");exit(-1);}
         strcpy(current_env.dict_path,argv[ArgNumb]+3);
	 
       } else if (strncmp(argv[ArgNumb],"-TD",3)==0) {
         if (strcmp(current_env.test_doc_path,"-")!=0) 
           {printf("Two -TD<test_doc_path> parameters\n");exit(-1);}
        strcpy(current_env.test_doc_path,argv[ArgNumb]+3);

       } else if (strncmp(argv[ArgNumb],"-OR",3)==0) {
         if (strcmp(current_env.oracle_path,"-")!=0) 
           {printf("Two -OR<oracle_path> parameters\n");exit(-1);}
         strcpy(current_env.oracle_path,argv[ArgNumb]+3);

       } else if (strncmp(argv[ArgNumb],"-O",2)==0) {
         if (strcmp(current_env.output_path,"-")!=0) 
           {printf("Two -O<output_path> parameters\n");exit(-1);}
        strcpy(current_env.output_path,argv[ArgNumb]+2);

       } else if (strncmp(argv[ArgNumb],"-TM",3)==0) {
         if (strcmp(current_env.tmp_path,"-")!=0) 
           {printf("Two -TM<tmp_file_path> parameters\n");exit(-1);}
         strcpy(current_env.tmp_path,argv[ArgNumb]+3);

       } else if (strcmp(argv[ArgNumb],"-VERB")==0) {
         if (current_env.verbose_switch!=(-1)) 
           {printf("Two -VERB parameters\n");exit(-1);}
         current_env.verbose_switch=ON;

       } else if (strcmp(argv[ArgNumb],"-LE")==0) {
         if (current_env.le_switch!=(-1)) 
           {printf("Two -LE parameters\n");exit(-1);}
         current_env.le_switch=ON;

       } else if (strcmp(argv[ArgNumb],"-NE")==0) {
         if (current_env.ne_switch!=(-1)) 
           {printf("Two -NE parameters\n");exit(-1);}
         current_env.ne_switch=ON;

       } else if (strcmp(argv[ArgNumb],"-EV")==0) {
         if (current_env.ev_switch!=(-1)) 
           {printf("Two -EV parameters\n");exit(-1);}
         current_env.ev_switch=ON;
 
       } else if (strcmp(argv[ArgNumb],"-D")==0) {
         break;
	 
       } else  {
        printf("ERROR : Unknow or invalid parameter '%s'\n",argv[ArgNumb]);
	PrintUsage(argv[0]);
        exit(-1);
       }
     }

     /* set default values for unspecified environment fields*/
     
     if (strcmp(current_env.oracle_path,"-")==0){
          printf("\n ERROR: oracle file is needed");
	  exit(-1);
     }	  
     if (strcmp(current_env.dict_path,"-")==0){
          printf("\n ERROR: specific path of dictionaries needed");
	  exit(-1);
     }	  
     if (strcmp(current_env.test_doc_path,"-")==0){
          printf("\n ERROR: specific path of test doc needed");
	  exit(-1);
     }	  
     if (strcmp(current_env.tmp_path,"-")==0)
          sprintf(current_env.tmp_path,"%s/%s/%s",AR_DATA,DOMAIN,TEMPORARY_PATH);
     if (strcmp(current_env.output_path,"-")==0)
          sprintf(current_env.output_path,"%s/%s/%s",AR_DATA,DOMAIN,OUTPUT_PATH);
      
     if (current_env.verbose_switch==(-1))   current_env.verbose_switch=OFF;
      
     if ( (current_env.le_switch==(-1)) && 
	   (current_env.ne_switch==(-1)) && 
           (current_env.ev_switch==(-1)) ) {
        printf("\nERROR : <mode> parameter must be present\n");
	PrintUsage(argv[0]);
        exit(-1);
     } 
     
      
     if (current_env.le_switch==(-1))   current_env.le_switch=OFF;
     if (current_env.ne_switch==(-1))   current_env.ne_switch=OFF;
     if (current_env.ev_switch==(-1))   current_env.ev_switch=OFF;
      
     /* read the class list or ALL*/
     current_env.document_number=0;
     
     ArgNumb++;  
     /* ArgNumb is now the index of the first doc */
     /* name parameter or the last parameter 'ALL'  */
     if ( (ArgNumb==(argc-1)) && (strcmp(argv[ArgNumb],"ALL")==0) ) {
	/* identify all the class name in 'current_env.class_path' */
	/* ------------------------------------------------------------------------- */  
	/* Alessandro modifica fino ad 'end' */
	sprintf(fileNamesPath,"%s/%s",current_env.tmp_path,FILE_OF_TEST_DOC_NAMES);
	sprintf(command,"ls %s > %s",current_env.test_doc_path,fileNamesPath);
	system(command);	
        fnames=fopen(fileNamesPath,"r");
        if(fnames==NULL)
          {
           printf("file %s does not found\n",fileNamesPath);
           exit(-1);
          }
	  
	current_env.document_number=0;
	current_env.document_names=(char **)MYmalloc(sizeof(char *)*(MAX_TEST_DOC_NUMBER));
        fscanf(fnames,"%s",fileName); 
        while(!feof(fnames)) { 	  	 
	  cutExtensions(fileName);
	  current_env.document_names[current_env.document_number++]=MYstrdup(fileName);
	  fscanf(fnames,"%s",fileName);
        }
	 /* realloc with current_env.class_number size end */
	/* ------------------------------------------------------------------------- */  
     } else {
	current_env.document_names=(char **)MYmalloc(sizeof(char *)*(argc-ArgNumb)); 
        for (; ArgNumb<argc ; ArgNumb++)
	  current_env.document_names[current_env.document_number++]=MYstrdup(argv[ArgNumb]);
     }
     if ( current_env.document_number==0 ) {
        printf("\nERROR :  at least one document required\n");
        exit(-1);
     } 
   /*  Classification(current_env);*/
     
  } 
  if (strcmp(argv[1],"CLU")==0) {
     /*-----------------------------*/ 
     /*  clustering */ 
     /*-----------------------------*/ 
     /*            
      *         
      *	CLU [-DI<dictionaries_file_path>] [-CL <class dir>]
      * [-TM<tmp_file_path>] -OC<OUTPUT clusters dir> -IC<INPUT clusters dir> [-VERB]  <mode>
      * [-NC<number of clusters>] -C ALL | <class1> <class2> ... <classN>
      *
      */
			 
     /* define a variable to store the environment information */
     struct clustering_env current_env;
     
    
     /* set each environment field with a NOT valid value */
     strcpy(current_env.dict_path,"-");
     strcpy(current_env.tmp_path,"-");
     strcpy(current_env.splitted_class_path,"-");
     strcpy(current_env.out_clusters_path,"-");
     strcpy(current_env.in_clusters_path,"-");
     strcpy(current_env.weighting,"-");
     
     current_env.verbose_switch=(-1);

     current_env.le_switch=(-1);
     current_env.ne_switch=(-1);
     current_env.ev_switch=(-1);
     current_env.numberOfClusters=-1;

     /* read the command line parameters until "-C" */
     for (ArgNumb=2 ; ArgNumb<argc ; ArgNumb++) {
  
       	 
        if (strncmp(argv[ArgNumb],"-DI",3)==0) {
         if (strcmp(current_env.dict_path,"-")!=0) 
           {printf("Two -DI<dictionaries_file_path> parameters\n");exit(-1);}
         strcpy(current_env.dict_path,argv[ArgNumb]+3);
	 
       } else if (strncmp(argv[ArgNumb],"-SC",3)==0) {
         if (strcmp(current_env.splitted_class_path,"-")!=0) 
           {printf("Two -SC<class_path> parameters\n");exit(-1);}
         strcpy(current_env.splitted_class_path,argv[ArgNumb]+3);
	 
       }  else if (strncmp(argv[ArgNumb],"-W",2)==0) {
         if (strcmp(current_env.weighting,"-")!=0) 
           {printf("Two -W<weighting> parameters\n");exit(-1);}
         strcpy(current_env.weighting,"YES");
	 
       } else if (strncmp(argv[ArgNumb],"-OC",3)==0) {
         if (strcmp(current_env.out_clusters_path,"-")!=0) 
           {printf("Two -OC<out_clusters_path> parameters\n");exit(-1);}
        strcpy(current_env.out_clusters_path,argv[ArgNumb]+3);

       } else if (strncmp(argv[ArgNumb],"-IC",3)==0) {
         if (strcmp(current_env.in_clusters_path,"-")!=0) 
           {printf("Two -IC<in_clusters_path> parameters\n");exit(-1);}
        strcpy(current_env.in_clusters_path,argv[ArgNumb]+3);

       } else if (strncmp(argv[ArgNumb],"-NC",3)==0) {
         if (current_env.numberOfClusters>0) 
           {printf("Two -TM<number of clusters> parameters\n");exit(-1);}
            current_env.numberOfClusters=atoi(argv[ArgNumb]+3);

       } else if (strcmp(argv[ArgNumb],"-VERB")==0) {
         if (current_env.verbose_switch!=(-1)) 
           {printf("Two -VERB parameters\n");exit(-1);}
         current_env.verbose_switch=ON;

       } else if (strcmp(argv[ArgNumb],"-LE")==0) {
         if (current_env.le_switch!=(-1)) 
           {printf("Two -LE parameters\n");exit(-1);}
         current_env.le_switch=ON;

       } else if (strcmp(argv[ArgNumb],"-NE")==0) {
         if (current_env.ne_switch!=(-1)) 
           {printf("Two -NE parameters\n");exit(-1);}
         current_env.ne_switch=ON;

       } else if (strcmp(argv[ArgNumb],"-EV")==0) {
         if (current_env.ev_switch!=(-1)) 
           {printf("Two -EV parameters\n");exit(-1);}
         current_env.ev_switch=ON;
 
       } else if (strcmp(argv[ArgNumb],"-D")==0) {
         break;
	 
       } else  {
        printf("ERROR : Unknow or invalid parameter '%s'\n",argv[ArgNumb]);
	PrintUsage(argv[0]);
        exit(-1);
       }
     }

     /* set default values for unspecified environment fields*/     
     
          
     /*if (strcmp(current_env.out_clusters_path,"-")==0){
          printf("\n ERROR: output cluster path is needed");
	  exit(-1);
     }	 */ 
     if (strcmp(current_env.dict_path,"-")==0){
          printf("\n ERROR: specific path of dictionaries needed");
	  exit(-1);
     }	  
   /*  if (strcmp(current_env.in_clusters_path,"-")==0){
          printf("\n ERROR: input cluster path needed");
	  exit(-1);
     }*/	  
     if (strcmp(current_env.tmp_path,"-")==0)
          sprintf(current_env.tmp_path,"%s/%s/%s",AR_DATA,DOMAIN,TEMPORARY_PATH);
     if (strcmp(current_env.output_path,"-")==0)
          sprintf(current_env.output_path,"%s/%s/%s",AR_DATA,DOMAIN,OUTPUT_PATH);
      
     if (current_env.verbose_switch==(-1))   current_env.verbose_switch=OFF;
      
     if ( (current_env.le_switch==(-1)) && 
	   (current_env.ne_switch==(-1)) && 
           (current_env.ev_switch==(-1)) ) {
        printf("\nERROR : <mode> parameter must be present\n");
	PrintUsage(argv[0]);
        exit(-1);
     } 
     
      
     if (current_env.le_switch==(-1))   current_env.le_switch=OFF;
     if (current_env.ne_switch==(-1))   current_env.ne_switch=OFF;
     if (current_env.ev_switch==(-1))   current_env.ev_switch=OFF;
      
     /* read the number of clusters
	
     sprintf(fileNamesPath,"%s/%s",current_env.tmp_path,FILE_OF_CLUSTER_NAMES);
     sprintf(command,"ls %s > %s",current_env.in_clusters_path,fileNamesPath);
     system(command);	
     fnames=fopen(fileNamesPath,"r");
     if(fnames==NULL)
       {
        printf("file %s does not found\n",fileNamesPath);
        exit(-1);
       }
	  
	current_env.numberOfClusters=0;
        fscanf(fnames,"%s",fileName); 
        while(!feof(fnames)) { 	
	  current_env.numberOfClusters++;  	 
	  fscanf(fnames,"%s",fileName);
        }
	 /* realloc with current_env.class_number size end */
	
     KNN_classifier(current_env,0); /*at moment cluster only the first class*/
     
  } else {
     printf("\nERROR : First argument must be CCE, GCE, DIC, CLU or CLA\n");
     PrintUsage(argv[0]);
     exit(-1);
  }
  exit(0);
}
