/***********************************************************************/
/*   Text Categorization Framework                                     */
/*                                                                     */
/*                                                                     */
/*   Author: Alessandro Moschitti                                      */
/*   moschitti@info.uniroma2.it                                        */
/*   Date: 30.11.03                                                    */
/*                                                                     */
/*   Copyright (c) 2003  Alessandro Moschitti - All rights reserved    */
/*                                                                     */
/*   This software is available for non-commercial use only. It must   */
/*   not be modified and distributed without prior permission of the   */
/*   author. The author is not responsible for implications from the   */
/*   use of this software.                                             */
/*                                                                     */
/***********************************************************************/


#include "classifier.h"
#include <math.h>
#include <string.h>

/**********************************************************************************************
* 					Program OPTION					      *
***********************************************************************************************/

#define OUTPUT_STDSCARTI
#define MAX_CICLE_NUMBER 25


/*********************************************************************************************/
/***				Heading of PROCEDURES					  ****/
/*********************************************************************************************/


/*********************************************************************************************/
/***				Load KB info file prendere da dictionary		  ****/
/*********************************************************************************************/


int loadClassificationKB(struct clustering_env my_env, char *extension, struct CKB_env *clKB) {
  char etichetta[TERM_LENGHT];
  char dictAbsName[PATH_LENGHT];
  char weightDictAbsName[PATH_LENGHT];
  char infoDictionariesAbsName[PATH_LENGHT];
  int i=0;

    
  FILE *infoDictionariesFile;
  FILE *offsetDictFile;
  FILE *weightDictFile;
   
  /* -------------------------------- */
  /* Reading Dictionaries information */
  /* -------------------------------- */
  if (my_env.verbose_switch==ON) printf ("Reading Dictionaries information\n");
  
  sprintf(infoDictionariesAbsName,"%s/%s.%s",my_env.dict_path,INFO_DICTIONARY,extension);
  if((infoDictionariesFile = fopen(infoDictionariesAbsName,"r"))==NULL) {
      printf("file %s does not found\n",infoDictionariesAbsName);
      return 0;
  }
  fscanf(infoDictionariesFile,"%s\n%d\n%s\n%d\n",
		              etichetta,&(clKB->totalDocumentNumber),
		              etichetta,&(clKB->differentWordNumber));
  clKB->centroidNumber=0;
  
    /* Centroid Names reading */

  fscanf(infoDictionariesFile,"%s\n",clKB->centroidNames[clKB->centroidNumber]);
  while(!feof(infoDictionariesFile)&& strcmp(clKB->centroidNames[clKB->centroidNumber],"Max_Weight=")!=0){
    if (my_env.verbose_switch==ON) 
	   printf("centroidNames[%d]=%s\n",clKB->centroidNumber,clKB->centroidNames[clKB->centroidNumber]);
    clKB->centroidNumber++;
    fscanf(infoDictionariesFile,"%s\n",clKB->centroidNames[clKB->centroidNumber]);
  }
  
  fscanf(infoDictionariesFile,"%f\n%s",&(clKB->maxWeight),etichetta);  /* max weight reading */
  
    /* Centroid total Frequences reading */
  i=0;
  fscanf(infoDictionariesFile,"%f\n",&clKB->centroidTotalFreqs[i]);
  while(!feof(infoDictionariesFile)&&i<clKB->centroidNumber){	  
    if (my_env.verbose_switch==ON) 
	   printf("centroidTotalFreq[%d]=%f\n",i,clKB->centroidTotalFreqs[i]);
    i++;
    fscanf(infoDictionariesFile,"%f\n",&clKB->centroidTotalFreqs[i]);
  }
  
  
    /* Average, STD Deviation reading (of positive) */
  i=0;
  fscanf(infoDictionariesFile,"%s%f\n",etichetta,&clKB->positiv[i].average);
  while(!feof(infoDictionariesFile)&&i<clKB->centroidNumber){
    fscanf(infoDictionariesFile,"%f\n",&clKB->positiv[i].stdDeviation);	  
    if (my_env.verbose_switch==ON){
	   printf("positiv[%d].average=%f\t",i,clKB->positiv[i].average);
	   printf("positiv[%d].stdDeviation=%f\n",i,clKB->positiv[i].stdDeviation);
    }
    fscanf(infoDictionariesFile,"%f\n",&clKB->positiv[++i].average);
  }
  
    /* Average, STD Deviation reading of negative */
  i=0;
  fscanf(infoDictionariesFile,"%s%f\n",etichetta,&clKB->negativ[i].average);
  while(!feof(infoDictionariesFile)&&i<clKB->centroidNumber){
    fscanf(infoDictionariesFile,"%f\n",&clKB->negativ[i].stdDeviation);	  
    if (my_env.verbose_switch==ON){
	   printf("negativ[%d].average=%f\t",i,clKB->negativ[i].average);
	   printf("negativ[%d].stdDeviation=%f\n",i,clKB->negativ[i].stdDeviation);
    }
    fscanf(infoDictionariesFile,"%f\n",&clKB->negativ[++i].average);
  }
  
  fclose(infoDictionariesFile);
  
  
  /* -------------------------------- */
  /* Reading dictionaryOffset 	      */
  /* -------------------------------- */

  sprintf(dictAbsName,"%s/%s.%s",my_env.dict_path,OFFSET_DICTIONARY,extension);
  if((offsetDictFile = fopen(dictAbsName,"r"))==NULL) {
      printf("file %s does not found\n",dictAbsName);
      return 0;
  }
  sprintf(weightDictAbsName,"%s/%s.%s",my_env.dict_path,WEIGHT_DICTIONARY,extension);
  if((clKB->weightDictFile = fopen(weightDictAbsName,"r"))==NULL) {
      printf("file %s does not found\n",weightDictAbsName);
      return 0;
  }

  if (my_env.verbose_switch==ON) printf ("loading Offset Dictionary ... "); 
  loadDictionaryOffset(offsetDictFile,clKB->differentWordNumber,&clKB->offsetDictArray);
  if (my_env.verbose_switch==ON) printf ("done !\n");    

  fclose(offsetDictFile);
  
  }

short sgn(float x){
	if(x>=0) return 1;
	else return -1;
 }
  

/* unifica prendere da centroid.c */ 
 
int unifieLemma(char *term, int freq, struct dictionary *centroid, int centroidSize){
  
  int i;
       for(i=0;i<centroidSize;i++)
	  if (strcmp(term,centroid[i].term)==0) {
              /*TEST printf("key %s already present\n",term);*/
              centroid[i].freq+=freq;
	      i=centroidSize+2;
	  }  
       if(i<centroidSize+2){
               /*TEST printf("key %s added at the end\n",term);*/
	       
	       strcpy(centroid[centroidSize].term,term);     
               centroid[centroidSize].freq=freq;
	       centroidSize++;
       }
   return centroidSize;    
   } 
   
/* prendere sempre da centroid.c */
   
   
int buildDocumentCentroid(FILE *documentFile, 
		struct dictionary *docCentroid){
	  
   char 
	documentId[TERM_LENGHT],
      	term[TERM_LENGHT];   
   
   char  documentAbsName[PATH_LENGHT];
   
   int freq,centroidSize=0;    
     
	     
     fscanf(documentFile,"%s",documentId);
     while(!feof(documentFile)){    
       fscanf(documentFile,"%s",term);
       fscanf(documentFile,"%d",&freq);
       /*TEST printf("%s\n",documentId);
       printf("%s\n",term);
       printf("%d\n\n",freq);*/ 
              		                
       centroidSize=unifieLemma(term,freq,docCentroid,centroidSize);  
       fscanf(documentFile,"%s",documentId);
    }
       
   return centroidSize; 
        
}  
  

void shutdownClassificationKB(struct CKB_env *clKB){
  int i;
  	
  for(i=0;i<clKB->differentWordNumber;i++)
	  MYfree(clKB->offsetDictArray[i].term);
  
  MYfree(clKB->offsetDictArray);
  fclose(clKB->weightDictFile);
  
}

void shutdownOracle(struct oracleCell *oracle, int numOfdoc){
  int cellIndex,
      classIndex;
  	
  for(cellIndex=0;cellIndex<numOfdoc;cellIndex++){
     classIndex=0;
     while(oracle[cellIndex].targetClass[classIndex]!=NULL){
	  MYfree(oracle[cellIndex].targetClass[classIndex]);
	  classIndex++;
     }
  MYfree(oracle[cellIndex].targetClass);
  }
  MYfree(oracle);
}


int computeNumberOfCorpusDoc(FILE *oracleFile)
{
 
 int numOfCorpusDoc=0;
 char class[MAX_CLASS_LENGHT];
 char idDoc[ID_DOC_LENGHT];
 char idDocOld[ID_DOC_LENGHT];
 
 fscanf(oracleFile,"%s",idDoc);
 (*idDocOld)='\0';
 while(!feof(oracleFile)){
      
      fscanf(oracleFile,"%s",class);
      if(strcmp(idDoc,idDocOld)!=0)numOfCorpusDoc++;
      strcpy(idDocOld,idDoc);
      fscanf(oracleFile,"%s",idDoc);
      
 }
 return numOfCorpusDoc;
}


int loadOracle(struct oracleCell **oracle, char oraclePath[PATH_LENGHT]) {
    
 int numOfCorpusDoc=0,cellIndex=0,
     classIndex,
     numOfClassPerDoc;
 char class[MAX_CLASS_LENGHT];
 char idDoc[ID_DOC_LENGHT];
 char idDocOld[ID_DOC_LENGHT];
 char classes[MAX_CLASS_NUMBER][MAX_CLASS_LENGHT];
 FILE *oracleFile;
 
   if((oracleFile = fopen(oraclePath,"r"))==NULL) {
		printf("Warning : file %s does not found\n",oraclePath);
		return 0;
     }
    else printf("\tfile %s opened\n",oraclePath);  	
 
   numOfCorpusDoc=computeNumberOfCorpusDoc(oracleFile);
   /*TEST printf ("Number of Doc = %d\n",numOfCorpusDoc);*/
   (*oracle) = (struct oracleCell *) malloc(numOfCorpusDoc * sizeof(struct oracleCell));
   rewind(oracleFile);
   
   fscanf(oracleFile,"%s",idDoc);
   /*TEST printf ("idDoc %s\n",idDoc);*/
   while(!feof(oracleFile)){
      strcpy((*oracle)[cellIndex].idDoc,idDoc);
      numOfClassPerDoc=0;
      strcpy(idDocOld,idDoc);	   
      while(!feof(oracleFile) && strcmp(idDoc,idDocOld)==0){
	      fscanf(oracleFile,"%s\n",class);
	      /*TEST printf ("class %s\n",class);*/
	      strcpy(classes[numOfClassPerDoc],class);
	      numOfClassPerDoc++;
	      fscanf(oracleFile,"%s",idDoc);
	      /*TEST printf ("idDoc %s\n",idDoc);*/
      }
      (*oracle)[cellIndex].targetClass=(char **)malloc((numOfClassPerDoc+1)*sizeof(char *));
      (*oracle)[cellIndex].targetClass[numOfClassPerDoc]=NULL;
      for(classIndex=0;classIndex<numOfClassPerDoc;classIndex++)
	  (*oracle)[cellIndex].targetClass[classIndex]=strdup(classes[classIndex]);
      cellIndex++;
   }
   /*TEST  printf ("stampa oracolo \n");
   for(cellIndex=0;cellIndex<numOfCorpusDoc;cellIndex++){
	    classIndex=0;
	    printf("Document id: %s belong to: ",(*oracle)[cellIndex].idDoc);
	    while((*oracle)[cellIndex].targetClass[classIndex]!=NULL){
		    printf("%s\t",(*oracle)[cellIndex].targetClass[classIndex]);
		    classIndex++;
	    }
	    printf("\n");
   }*/	    
	  
   fclose(oracleFile);
		   
   return numOfCorpusDoc;
}  
	    	
		
char *docClassName (char *docName,char *className)
{
 int i=0;
  
 while((docName[i]>'9' || docName[i]<'0')&& i<strlen(docName))
   className[i]=docName[i++];
 className[i]='\0';
 return className;
} 

void docClassNumber (char *docName,char *className)

{
 int i=0,numberStart=0;
 while((docName[i]>'9' || docName[i]<'0')&& i<strlen(docName))i++;
 numberStart=i;
 while((docName[i]<='9' && docName[i]>='0')&& i<strlen(docName))
   className[i-numberStart]=docName[i++];
 className[i-numberStart]='\0';
} 
 
int searchIdDoc(char *idDoc,struct oracleCell *oracle,int i, int j)
{
 if(i>j)return -1;
   else
    if(strcmp((char *)oracle[i+(j-i)/2].idDoc,idDoc)==0) return i+(j-i)/2;
      else 
       if(strcmp((char *)oracle[i+(j-i)/2].idDoc,idDoc)<0)
       	 return searchIdDoc(idDoc,oracle,i+(j-i)/2+1,j);
       else
       	 return searchIdDoc(idDoc,oracle,i,i+(j-i)/2-1);
 }
 


/*********************************************************************************************/
/* 					Clustering CODE	   				     */
/*********************************************************************************************/

void shutdownDB(struct docDB *db, int numberOfDoc){

 int i;
 	
   for(i=0;i<=numberOfDoc;i++)
	MYfree(db[i].docCentroid);
   MYfree(db);
}  
  
 
float documentScalarProduct(struct docVect *docCentroid1, int centroidSize1,
			     struct docVect *docCentroid2, int centroidSize2) {
  int indTermDoc1,
      indTermDoc2,
      confront;
 	  
  double documentTermWeight1,
	 documentTermWeight2,
	 scalarProduct=0,
	 docNorma1=0,
  	 docNorma2=0;
  
     /* TEST   
	printf("Centroid Size:%d ",centroidSize2);*/
  
  indTermDoc1=0;
  indTermDoc2=0;
  
  while(indTermDoc2<centroidSize2 || indTermDoc1<centroidSize1){   

/* TEST printf ("  Moltiplico Word %s ---> %d \n",docCentroid1[i].term,i);*/
    
    if(indTermDoc2>centroidSize2) confront=-1;
    else
    	if(indTermDoc1>centroidSize1) confront=1;
    	  else confront=strcmp(docCentroid1[indTermDoc1].term,docCentroid2[indTermDoc2].term);
    
    if(confront==0){
	      
       docNorma1+=(docCentroid1[indTermDoc1].weight * docCentroid1[indTermDoc1].weight);/* Squared Norma of doc1 */
       
       docNorma2+=(docCentroid2[indTermDoc2].weight * docCentroid2[indTermDoc2].weight);

       scalarProduct+=docCentroid1[indTermDoc1].weight*docCentroid2[indTermDoc2].weight; /** setOfCouples.couple[j].weight)*/;
       
       /* TEST  printf ("  term: %s ---> w1 %f w2 %f scalar product %f\n",
		       docCentroid1[indTermDoc1].term,docCentroid1[indTermDoc1].weight,docCentroid2[indTermDoc2].weight
		       ,scalarProduct);*/
       indTermDoc1++;
       indTermDoc2++;
    }
    else if (confront<0){
            docNorma1+= docCentroid1[indTermDoc1].weight * docCentroid1[indTermDoc1].weight;
	    indTermDoc1++;
         }
	 else if (confront>0){
       	    docNorma2+=(docCentroid2[indTermDoc2].weight * docCentroid2[indTermDoc2].weight);
	    indTermDoc2++;
         }
  }
     	
/* printf ("  documentTermWeight1,documentTermWeight2 %f, %f\n",documentTermWeight1,documentTermWeight2 ); 
printf (" scalarProduct  %f \n",scalarProduct);*/

	/*weightNorma[setOfCouples.couple[j].centroidNumber]+=
		     	  (setOfCouples.couple[j].weight * setOfCouples.couple[j].weight);*/

  
    /* Normalizzation of scalar product   */
     if(scalarProduct!=0) scalarProduct/=sqrt(docNorma1*docNorma2);
     
     /*TEST printf ("  docNorma1,docNorma2 %f, %f\n",docNorma1,docNorma2 );*/
     
    
     return scalarProduct;
}

float maxVect(float *vect, int n){
 int i,
     indMax;
 float max=0;
 
 for(i=0;i<n;i++)
	 if(max<=vect[i]){max=vect[i];indMax=i;}
 vect[indMax]=-1;
 return max;
 }	 

 
 float minVect(float *vect, int n){
 int i,
     indMin;
 float min=0;
 
 for(i=0;i<n;i++)
	 if(min<=vect[i]&&min>=0){min=vect[i];indMin=i;}
 vect[indMin]=-1;
 return min;
 }	 
	 
	 

int chooseClusterForDoc(struct numericCluster *cluster[],struct docDB *db, char *aSetDocId,
			struct docVect *aSetDoc, int setDocSize, struct clustering_env my_env){
		  
     int indexTerm,
   	 numberOfDoc,
	 bestClusterIndex,
	 clusterIndex,
	 docIndex,
	 k_neighbour;
     
     float max,maxV,
	    scalarProduct, 
	    clusterSimilarity,
	    similarity[MAX_DOCS_IN_CLASS];
     
     struct numericCluster *aClust;
     int clusterDocSize;
     
     
			
    bestClusterIndex=0; max=0;
    for(clusterIndex=0;clusterIndex<my_env.numberOfClusters;clusterIndex++) {
	    
      if (my_env.verbose_switch==ON) printf ("  Measure cluster %d \n",clusterIndex);
      for(docIndex=0;docIndex<MAX_DOCS_IN_CLASS;docIndex++)similarity[MAX_DOCS_IN_CLASS]=0;

      clusterSimilarity=0;
      numberOfDoc=0;
      if(aClust=NULL)printf("\n<<<<<Warning: Cluster %d is empty>>>>>\n");
      else{
	   aClust=cluster[clusterIndex];

	   do { 
	       if (my_env.verbose_switch==ON) 
		     printf ("   Similarity with doc %s of size %d = ",
			     db[aClust->docNum].docId,db[aClust->docNum].size); 

	      /**** il doc da cluster non fa parte del cluster ***/
	       if(strcmp(db[aClust->docNum].docId,aSetDocId)!=0 || cluster[clusterIndex]->next==NULL){
		   scalarProduct = documentScalarProduct(aSetDoc,setDocSize,db[aClust->docNum].docCentroid,
			  				 db[aClust->docNum].size);
		   clusterSimilarity += scalarProduct;
		   if (my_env.verbose_switch==ON) 
			 printf (" %f \n",scalarProduct);


		   similarity[numberOfDoc]=scalarProduct;
		   numberOfDoc++;
  	       }
	       else if (my_env.verbose_switch==ON) printf("same doc --> not computed!\n");
	       aClust=aClust->next;

	   } while(aClust!=NULL);
   	}
	 
/*      if(numberOfDoc>0)clusterSimilarity/=(float)numberOfDoc;*/
      clusterSimilarity=0;
      
      k_neighbour=150;
      if(k_neighbour>numberOfDoc)k_neighbour=numberOfDoc;
      for(docIndex=0;docIndex<k_neighbour;docIndex++)
	      clusterSimilarity+=((maxV=maxVect(similarity,numberOfDoc))/k_neighbour);
           
      
      if(k_neighbour<numberOfDoc){
	k_neighbour=numberOfDoc-k_neighbour;
	if(k_neighbour>150) k_neighbour=150;    
      	for(docIndex=0;docIndex<k_neighbour;docIndex++)
	      clusterSimilarity+=((minVect(similarity,numberOfDoc)-maxV)/k_neighbour);
	}
      
  /*    clusterSimilarity/=k_neighbour;*/
      
      if (my_env.verbose_switch==ON) printf (">>Overall clusters similarity %d ---> %f\n\n",
	      clusterIndex,clusterSimilarity);
      if (clusterSimilarity>max){
	  max=clusterSimilarity;
	  bestClusterIndex=clusterIndex;
      }
     
    }
    
    if (my_env.verbose_switch==ON)
	    printf ("\n\t\t\t*** Cluster %d has been choosen ***\n\n\n",bestClusterIndex);
    return bestClusterIndex;
}

  
void putDocInCluster(struct numericCluster **aCluster, int assignedDoc){
	 		
     struct numericCluster *temp;
     temp=(struct numericCluster *) MYmalloc(sizeof(struct numericCluster));
     temp->docNum=assignedDoc;
     temp->next=*aCluster;
     *aCluster=temp;
}

void initClusters(struct numericCluster *clusters[], int numberOfClusters){
     
     int i;
     for(i=0;i<=numberOfClusters;i++)clusters[i]=NULL;
     
}     


void printClusters(struct numericCluster *clusters[], struct docDB *db, 
		   int numberOfClusters, struct oracleCell *oracle, int numOfDoc){
     struct numericCluster *temp;
     int i,idDocPos,classIndex;
     
     for(i=0;i<numberOfClusters;i++){
	printf("Cluster %d: ",i);
	temp=clusters[i];
    	while(temp!=NULL){
	    idDocPos=searchIdDoc(db[temp->docNum].docId,oracle,0,numOfDoc-1);
	    printf("%s:",db[temp->docNum].docId);
	    classIndex=0;
	    while(oracle[idDocPos].targetClass[classIndex]!=NULL){
		    printf("%s ",oracle[idDocPos].targetClass[classIndex]);
		    classIndex++;
	    }
	    printf("\n");
	    temp=temp->next;
     	}
	printf("\n\n");
     }
}


void evaluation(struct numericCluster *clusters[], struct docDB *db, 
		   int numberOfClusters, struct oracleCell *oracle, int numOfDoc){
     struct numericCluster *temp;
     int 
	 idDocPos,
	 classIndex,
	 clusterIndex,
	 labelIndex,
	 counters[3][10],
	 som;

     
     char clusterName[3][20]={"ship","coffee","rice"};
     
     
     
     for(clusterIndex=0;clusterIndex<3;clusterIndex++)
	     for(classIndex=0;classIndex<3;classIndex++)counters[clusterIndex][classIndex]=0;
     
     for(clusterIndex=0;clusterIndex<numberOfClusters;clusterIndex++){
	/**/printf("Cluster %d\n",clusterIndex);
	temp=clusters[clusterIndex];
    	while(temp!=NULL){
	    idDocPos=searchIdDoc(db[temp->docNum].docId,oracle,0,numOfDoc-1);
	    /**/ printf("%s:",db[temp->docNum].docId);
	    labelIndex=0;
	    if(idDocPos>=0)
	    while(oracle[idDocPos].targetClass[labelIndex]!=NULL){
		  /**/  printf("%s ",oracle[idDocPos].targetClass[labelIndex]);
		    for(classIndex=0;classIndex<3;classIndex++)
		       if(strcmp(oracle[idDocPos].targetClass[labelIndex],clusterName[classIndex])==0)
				counters[clusterIndex][classIndex]++;
		    labelIndex++;
	    }
	   /**/ printf("\n");
	    temp=temp->next;
     	}
	/**/ printf("\n\n");
     }
     printf("\n\nAccuracy COMPUTATION\n\n");
     for(clusterIndex=0;clusterIndex<3;clusterIndex++){
	     som=0;
	     for(classIndex=0;classIndex<3;classIndex++)som+=counters[clusterIndex][classIndex];
	     printf("cluster %s: %f\n",clusterName[clusterIndex],
			     (float)counters[clusterIndex][clusterIndex]/(float)som);
     }     
			     		     
}

void cleanClusters(struct numericCluster *aCluster[], int numOfClusters){
     
     struct numericCluster *temp;
     int clusterInd;
     
     for(clusterInd=0;clusterInd<numOfClusters;clusterInd++)
	  while((aCluster[clusterInd])!=NULL){
		 temp=(aCluster[clusterInd])->next;
		 MYfree(aCluster[clusterInd]);
		 (aCluster[clusterInd])=temp;
	 }
    
}	    	

void clusterAclass (struct clustering_env my_env, int whichClass, char *extension)
{ 
  char className [PATH_LENGHT];
  char classFileAbsName[PATH_LENGHT];
  struct CKB_env my_CKB_env;
  
  struct docVect docCentroid[MAX_DOC_SIZE];
  int 	 centroidSize;
  
  FILE *classFile;
  int numberOfDoc;
  
  char 	docId[PATH_LENGHT];
  
  int someCriteria=0;
  
  char 	temp[PATH_LENGHT],
	command[PATH_LENGHT];
  
  long pos; /*position of class file pointer */
  
  int assignedCluster;
    
  int *centroidSizes;

  struct numericCluster 
		  	**outClusters,
		  	**inClusters,
  			**tempClusterPointer;
  
  struct docDB *db;
  int docIndex;

  struct oracleCell *oracle;
  int numOfCorpusDoc;
   
  /* -------------------------------------------- */
  /*    Load the classification Knowledge Bases   */
  /* -------------------------------------------- */

   loadClassificationKB(my_env,extension,&my_CKB_env);
    
  /* -------------------------------------------- */
  /*    Load the set of document to be clusterd   */
  /* -------------------------------------------- */
  
  sprintf(classFileAbsName,"%s/%s.%s",my_env.splitted_class_path,my_CKB_env.centroidNames[whichClass],extension);
    if((classFile = fopen(classFileAbsName,"r"))==NULL) {
	     printf("ERROR : file %s to be clustered does not found\n",classFileAbsName);
	     exit(-1);
    }
   
  
  /* count the document with the function for load a single doc */
  numberOfDoc=0;pos=0; 

  centroidSizes=(int *) MYmalloc(sizeof(int)*MAX_DOCS_IN_CLASS);
  
  while(!feof(classFile)){
	centroidSizes[numberOfDoc]=
	buildExtractedDocumentCentroid(classFile,docCentroid,&pos,docId,my_CKB_env);numberOfDoc++;
  }
  
  db=(struct docDB *) MYmalloc(sizeof(struct docDB)*numberOfDoc);
  
  inClusters=(struct numericCluster **) MYmalloc(sizeof(struct numericCluster *)*my_env.numberOfClusters);
  outClusters=(struct numericCluster **) MYmalloc(sizeof(struct numericCluster *)*my_env.numberOfClusters);


  initClusters(inClusters,my_env.numberOfClusters);
  putDocInCluster(&(inClusters[0]),0);  /*doc 0 assigned to cluster 0*/
  putDocInCluster(&(inClusters[1]),5); /*doc 10 assigned to cluster 1*/
  putDocInCluster(&(inClusters[2]),12); /*doc 20 assigned to cluster 2*/

  
  pos=0; strcpy(db[docIndex].docId,"");
  fseek(classFile,pos,SEEK_SET);   
  for(docIndex=0;docIndex<numberOfDoc;docIndex++){
	 db[docIndex].docCentroid=(struct docVect *) MYmalloc(sizeof(struct docVect)*centroidSizes[docIndex]);
	 db[docIndex].size = 
	   buildExtractedDocumentCentroid(classFile,db[docIndex].docCentroid,&pos,db[docIndex].docId);
  }	 
  
  MYfree(centroidSizes);
  
   /* -------------------------------------------- */
  /*    Load the Oracle				   */
  /* -------------------------------------------- */

   numOfCorpusDoc=loadOracle(&oracle,"ordOraClust");
    
  while(someCriteria++<=30){

	  	  
    printf("\n\n\n*************************************\n");
    printf("Iteration %d\n",someCriteria);
    printf("*************************************\n");  
    
    initClusters(outClusters,my_env.numberOfClusters);
    for(docIndex=0;docIndex<numberOfDoc;docIndex++){
	   	 
	 if (my_env.verbose_switch==ON){
  	   printf ("\n_____________________________________________________________________\n");		 
           printf ("\nProcessing document %s of class: <%s> \n",db[docIndex].docId,
				 my_CKB_env.centroidNames[whichClass]);
   	 }  

	 assignedCluster=chooseClusterForDoc(inClusters,db,
			 		     db[docIndex].docId,db[docIndex].docCentroid,db[docIndex].size,
			 		     my_env);
	 putDocInCluster(&outClusters[assignedCluster],docIndex);

    }
    tempClusterPointer=inClusters;
    inClusters=outClusters;
    outClusters=tempClusterPointer;
 /*   printClusters(inClusters,db,my_env.numberOfClusters, oracle, numOfCorpusDoc);*/
    evaluation(inClusters,db,my_env.numberOfClusters, oracle, numOfCorpusDoc);
    cleanClusters(outClusters,my_env.numberOfClusters);
      
  }
  fclose(classFile);
   
  /* Shutdown the Classification Knowledge Bases */
  shutdownClassificationKB(&my_CKB_env);
  shutdownDB(db,numberOfDoc);
  
}



void Clustering(struct clustering_env my_env, int classId) 
  {
   int i,
       classNumber,
       NumberOfClasses;
   float		   
	threshold[MAX_CLASS_NUMBER];
   float RP_Difference;
		
   struct results perClassPR[MAX_CLASS_NUMBER];
   short finish;
   int count;
   char  learningType[4];

      /* show the environment*/
   if (my_env.verbose_switch==ON) {
   printf("       dict_path = %s\n",my_env.dict_path);
   printf("       output_path = %s\n",my_env.output_path);
   printf("       out_clusters_path = %s\n",my_env.out_clusters_path);
   printf("       in_clusters_path = %s\n",my_env.in_clusters_path);
   printf("       tmp_path = %s\n\n",my_env.tmp_path);
   printf("       class path to be clustered (splitted_class_path) = %s\n\n",
		   my_env.splitted_class_path);
   
   
   
   printf("      verbose_switch = %s\n\n",(my_env.verbose_switch==ON?"ON":"OFF"));
   
   printf("           le_switch = %s\n",(my_env.le_switch==ON?"ON":"OFF"));
   printf("           ne_switch = %s\n",(my_env.ne_switch==ON?"ON":"OFF"));
   printf("           ev_switch = %s\n\n",(my_env.ev_switch==ON?"ON":"OFF"));
   
   printf("     number of clusters required = %d\n\n",my_env.numberOfClusters);
   
   
  
   } 
   
   if (my_env.le_switch==ON)strcpy(learningType,"le");
   if (my_env.ne_switch==ON)strcpy(learningType,"ne");
   if (my_env.ev_switch==ON)strcpy(learningType,"ev");
   
   my_env.numberOfClusters=3;
   clusterAclass(my_env,classId,learningType);
}
