/***********************************************************************/
/*   Text Categorization Framework                                     */
/*                                                                     */
/*                                                                     */
/*   Author: Alessandro Moschitti                                      */
/*   moschitti@info.uniroma2.it                                        */
/*   Date: 30.11.03                                                    */
/*                                                                     */
/*   Copyright (c) 2003  Alessandro Moschitti - All rights reserved    */
/*                                                                     */
/*   This software is available for non-commercial use only. It must   */
/*   not be modified and distributed without prior permission of the   */
/*   author. The author is not responsible for implications from the   */
/*   use of this software.                                             */
/*                                                                     */
/***********************************************************************/


#include "classifier.h"
#include "base_lib.h"
#include "string.h"


int readLastDocumentId(FILE *classFile)
{ 
  int	documentId;
  
  char documentIdString[TERM_LENGHT];
  char documentIdStringOld[TERM_LENGHT];
  char term[TERM_LENGHT];
  char syntClass[TERM_LENGHT];
  char *p;
  char freq[TERM_LENGHT];
   
 fscanf(classFile,"%s",documentIdString);

 while(!feof(classFile)){
   strcpy(documentIdStringOld,documentIdString);
   fscanf(classFile,"%s",term);
   fscanf(classFile,"%s",syntClass); 
   fscanf(classFile,"%s",freq);
   fscanf(classFile,"%s",documentIdString);
   }
   
   p=strchr(documentIdStringOld,'#')+1;
   /*TEST printf("documentIdStringOld=%s\n",documentIdStringOld);
          printf("valore intero = %d\n", atoi(p));*/
   if(p==NULL){
     printf("ERROR: Character '#' not found while reading the last document");
     return 0;
   }  
   return atoi(p);
}  


int classSplitting(struct class_ce_env my_env,int whichClass,char *extension)
{
  FILE  *classFile,
  	*splittedClassFile;

  char 	classAbsFileName[PATH_LENGHT],
	splittedClassAbsFileName[PATH_LENGHT],
	extractedDocumentAbsFIleName[PATH_LENGHT];
		
  int	documentNumber,
  	docIndex,
	pos;

  
  if (my_env.verbose_switch==ON) 
    printf("\n Split class '%s' using %s information and %d
			  percentage\n\n",my_env.class_names[whichClass],extension,my_env.test_set_percentage);


  sprintf(classAbsFileName,"%s/%s.%s",my_env.class_path,my_env.class_names[whichClass],extension);  
  if((classFile=fopen(classAbsFileName,"r"))==NULL){
     if (my_env.verbose_switch==ON) printf("    Warning : file %s does not found\n",classAbsFileName);
     return 0;
  }
  
    
  sprintf(splittedClassAbsFileName,"%s/%s%s.%s",my_env.tmp_path,SPLITTED,my_env.class_names[whichClass],extension);    
  
  if((splittedClassFile=fopen(splittedClassAbsFileName,"w"))==NULL){
     if (my_env.verbose_switch==ON) printf("   Warning : file %s does not found\n\n",splittedClassAbsFileName);
     return 0;
  }
      
  
  /*to improve performace fseek(file,0,SEEK_END);*/
 
 rewind(classFile);
 pos=0;
 /* here is the modify for splitting all documents of a class */
 documentNumber=readLastDocumentId(classFile);
 for(docIndex=0;docIndex<documentNumber;docIndex++){
 printf("dati dello split doc %s/%s%d.%s\n",
	                              my_env.test_doc_path,
	                              my_env.class_names[whichClass],docIndex,extension);
 sprintf(extractedDocumentAbsFIleName,"%s/%s%d.%s",
	                              my_env.test_doc_path,
	                              my_env.class_names[whichClass],docIndex,extension);

    if (extractOneDocumentForClass(classFile,splittedClassFile,
          			extractedDocumentAbsFIleName,docIndex,my_env.verbose_switch,
				&pos)){

       if (my_env.verbose_switch==ON) printf("      document %s created!\n",extractedDocumentAbsFIleName);

    }
    else printf("ERROR: document %s has not beeen created!\n",extractedDocumentAbsFIleName);
 }
 
 if (my_env.verbose_switch==ON) printf("      class %s created!\n",splittedClassAbsFileName);
 else printf("ERROR: class %s has not beeen created!\n",splittedClassAbsFileName);
 fclose(splittedClassFile);
 return 1;
}

int extractOneDocumentForClass(FILE *classFile,FILE *splittedClassFile, char *extractedDocumentAbsFIleName, 
			       int requiredDocument, int verbose,long *pos)
 {	 

  int documentId;
  int freq;

  
  char term[TERM_LENGHT];
  char documentIdString[TERM_LENGHT];
  char syntClass[SYNTATCTIC_CLASS_DIGIT];

  FILE 	*extractedDocumentFile,
	*temporaryClassFile;
  
  char *p;
  
  if(strcmp(extractedDocumentAbsFIleName,"non scrivere")!=0)
     if((extractedDocumentFile=fopen(extractedDocumentAbsFIleName,"w"))==NULL){
        if (verbose==ON) printf("   Warning : file %s write permission denied\n",extractedDocumentAbsFIleName);
        return 0;
     }
    
  /* Insert the procedure to find the document number documentId */
  fseek(classFile,*pos,SEEK_SET);
  fscanf(classFile,"%s",documentIdString);
  documentId=requiredDocument;
  
  while(!feof(classFile)&& documentId<=requiredDocument) {    
	  
     /*TEST printf("documentIdString %s\n",documentIdString);   */
        
	/* read from the class */
     
     fscanf(classFile,"%s",term);
     fscanf(classFile,"%s",syntClass); 
     fscanf(classFile,"%d\n",&freq);
     

     /*TEST printf("term=%s syntClass=%s freq=%d\n",term,syntClass,freq);*/

     
     p=strchr(documentIdString,'#');
     p++;
     if(p==NULL){
     printf("ERROR: Character '#' not found while extracting document");
     return 0;
     }  
     documentId=atoi(p);
        
     if(documentId==requiredDocument && strcmp(extractedDocumentAbsFIleName,"non scrivere")!=0){
       fprintf(extractedDocumentFile,"%s\t",documentIdString);
       fprintf(extractedDocumentFile,"%s\t",term);
       fprintf(extractedDocumentFile,"%s\t",syntClass); 
       fprintf(extractedDocumentFile,"%d\n",freq);
       *pos=ftell(classFile);

     }
     else
      if(documentId<requiredDocument) {
       fprintf(splittedClassFile,"%s\t",documentIdString);
       fprintf(splittedClassFile,"%s\t",term);
       fprintf(splittedClassFile,"%s\t",syntClass); 
       fprintf(splittedClassFile,"%d\n",freq);
       *pos=ftell(classFile);
     }
    
     fscanf(classFile,"%s",documentIdString);
     
  }  
   
  fclose(extractedDocumentFile);
 
  return 1;
}
     

/*void SplittingOfClasses(struct class_ce_env my_env) 
{
   int whichClass;
   if (my_env.verbose_switch==ON) {
     // show the environment 
     int i;
     printf("----------------------------------------------------\n\n");
     
     printf("          class_path = %s\n",my_env.class_path);
     printf("       centroid_path = %s\n",my_env.centroid_path);
     printf("       tmp_file_path = %s\n\n",my_env.tmp_path);
   
     printf("      verbose_switch = %s\n\n",(my_env.verbose_switch==ON?"ON":"OFF"));
   
     printf("           le_switch = %s\n",(my_env.le_switch==ON?"ON":"OFF"));
     printf("           ne_switch = %s\n",(my_env.ne_switch==ON?"ON":"OFF"));
     printf("           ev_switch = %s\n\n",(my_env.ev_switch==ON?"ON":"OFF"));
   
     printf(" test_set_percentage = %d\n",my_env.test_set_percentage);
     printf("       test_doc_path = %s\n\n",my_env.test_doc_path);
     
     printf("        class_number = %d\n\n",my_env.class_number);
     for (i=0; i<my_env.class_number ; i++)
       printf("   class_names[%d] = %s\n",i,my_env.class_names[i]);
     
     printf("\n----------------------------------------------------\n\n");
  }
  for(whichClass=0;whichClass<my_env.class_number;whichClass++) {
    if (my_env.le_switch==ON)
      classSplitting( my_env,whichClass,"le");
    if (my_env.ne_switch==ON)
      classSplitting( my_env,whichClass,"ne");
    if (my_env.ev_switch==ON)
      classSplitting( my_env,whichClass,"ev");
  }
  if (my_env.le_switch==ON) printf("end Splitting Task\n");
}
*/
