/***********************************************************************/
/*   Text Categorization Framework                                     */
/*                                                                     */
/*                                                                     */
/*   Author: Alessandro Moschitti                                      */
/*   moschitti@info.uniroma2.it                                        */
/*   Date: 30.11.03                                                    */
/*                                                                     */
/*   Copyright (c) 2003  Alessandro Moschitti - All rights reserved    */
/*                                                                     */
/*   This software is available for non-commercial use only. It must   */
/*   not be modified and distributed without prior permission of the   */
/*   author. The author is not responsible for implications from the   */
/*   use of this software.                                             */
/*                                                                     */
/***********************************************************************/


#include "classifier.h"
#include "base_lib.h"
#include "string.h"


int countDocument(FILE *classFile)
{ 
  int	docCount;
  
  char documentIdString[TERM_LENGHT];
  char documentIdStringOld[TERM_LENGHT];
  char term[TERM_LENGHT];
  char freq[TERM_LENGHT];
   
 fscanf(classFile,"%s",documentIdString);
 strcpy(documentIdStringOld,documentIdString);
 docCount=1;

 while(!feof(classFile)){
   if(strcmp(documentIdStringOld,documentIdString)!=0){
	strcpy(documentIdStringOld,documentIdString);
	docCount++;
   } 	
   fscanf(classFile,"%s",term);
   fscanf(classFile,"%s",freq);
   fscanf(classFile,"%s",documentIdString);
 }
   
 return docCount;
  
}  
  


int classSplitting(struct class_ce_env my_env,int whichClass,char *extension)
{
  FILE  *classFile,
  	*splittedClassFile;

  char 	classAbsFileName[PATH_LENGHT],
	splittedClassAbsFileName[PATH_LENGHT],
	extractedDocumentAbsFIleName[PATH_LENGHT],
	copyClass[PATH_LENGHT];
		
  int	numberOfDoc,
  	docIndex,
	pos,
	docCounter=0;
  
 /*****************************************************************
 /** percentuali di splitting originali PER REUTERS VERSIION 3******
 ** float percentage[]=
 ** {31,39,25,30,30,33,28,25,33,33,22,29,26,50,36,30,25,33,33,29,28,29,50,45,33,26,25,27,44,33,30,20,47,16,30,20,22,29,20,33,25,50,20,50,29,46,40,27,21,66,30,20,33,50,30,40,33,29,33,39,61,25,33,57,50,33,37,34,26,8,44,28,31,27,36,50,48,32,5,39,25,37,35,33,30,47,26,31,27,50,34,34,38};
 **  my_env.test_set_percentage=percentage[whichClass];*
 *******************************************************/ 
		 
  if (my_env.verbose_switch==ON) 
    printf("\n Split class '%s' using %s information and %d percentage\n\n",my_env.class_names[whichClass],extension,my_env.test_set_percentage);


  sprintf(classAbsFileName,"%s/%s.%s",my_env.class_path,my_env.class_names[whichClass],extension);  
  if((classFile=fopen(classAbsFileName,"r"))==NULL){
     if (my_env.verbose_switch==ON) printf("    Warning : file %s does not found\n",classAbsFileName);
     return 0;
  }
  
    
  sprintf(splittedClassAbsFileName,"%s/%s.%s",my_env.splitted_class_path,my_env.class_names[whichClass],extension);    
  
  if((splittedClassFile=fopen(splittedClassAbsFileName,"w"))==NULL){
     if (my_env.verbose_switch==ON) printf("   Warning : file %s does not found\n\n",splittedClassAbsFileName);
     return 0;
  }
      
  
  /*to improve performace fseek(file,0,SEEK_END);*/

  numberOfDoc=countDocument(classFile);

  if(numberOfDoc*my_env.test_set_percentage/100>0){
	rewind(classFile);
	pos=0;
	for(docIndex=0;docIndex<my_env.test_set_percentage*numberOfDoc/100;docIndex++){
  	
      	sprintf(extractedDocumentAbsFIleName,"%s/",
	                                     my_env.test_doc_path);
	                                  
      	   if (extractOneDocumentForClass(classFile,splittedClassFile,
          			       extractedDocumentAbsFIleName,docIndex,my_env.verbose_switch,
				       &pos,extension,&docCounter)){
      	   
	      if (my_env.verbose_switch==ON) printf("      document %s created!\n",extractedDocumentAbsFIleName);
	  
           }
	   else printf("ERROR: document %s has not beeen created!\n",extractedDocumentAbsFIleName);
        }
	
	if (extractOneDocumentForClass(classFile,splittedClassFile,
          			       "non scrivere",numberOfDoc+1,my_env.verbose_switch,
				       &pos,extension,&docCounter))	   
	if (my_env.verbose_switch==ON) printf("      class %s created!\n",splittedClassAbsFileName);
	else printf("ERROR: class %s has not beeen created!\n",splittedClassAbsFileName);
	fclose(splittedClassFile);
  } 
  else{
    fclose(splittedClassFile);  
    printf("Warning: too few documents in class %s, splitting faild\n",classAbsFileName);
    fclose(classFile);
    sprintf(copyClass,"cp %s %s",classAbsFileName,splittedClassAbsFileName);
    system (copyClass);        
    return 0;
  }  
 
 fclose(classFile);
 return 1;
}

int extractOneDocumentForClass(FILE *classFile,FILE *splittedClassFile, char *extractedDocumentAbsFIleName, 
			       int requiredDocument, int verbose,long *pos,char *extension, int *docCounter)
 {	 

  int freq;

  
  char term[TERM_LENGHT];
  char documentIdString[TERM_LENGHT];
  char documentIdStringOld[TERM_LENGHT];

  FILE 	*extractedDocumentFile,
	*temporaryClassFile;
   		  
  
     
  /* Insert the procedure to find the document number documentId */
  fseek(classFile,*pos,SEEK_SET);
  fscanf(classFile,"%s",documentIdString);
  
  
  if(strcmp(extractedDocumentAbsFIleName,"non scrivere")!=0){
	  sprintf(extractedDocumentAbsFIleName,"%s%s.%s",extractedDocumentAbsFIleName,documentIdString,extension);
	  
     if((extractedDocumentFile=fopen(extractedDocumentAbsFIleName,"w"))==NULL){
        if (verbose==ON) printf("   Warning : file %s write permission denied\n",extractedDocumentAbsFIleName);
        return 0;
     }
  }
  
  while(!feof(classFile)&& *docCounter<=requiredDocument) {    
	  
     /*TEST printf("documentIdString %s\n",documentIdString);   */
        
	/* read from the class */
     
     fscanf(classFile,"%s",term);
     fscanf(classFile,"%d\n",&freq);
        
     if(*docCounter==requiredDocument && strcmp(extractedDocumentAbsFIleName,"non scrivere")!=0){
       fprintf(extractedDocumentFile,"%s\t",documentIdString);
       fprintf(extractedDocumentFile,"%s\t",term);
       fprintf(extractedDocumentFile,"%d\n",freq);
       *pos=ftell(classFile);

     }
     else
      if(*docCounter<requiredDocument) {
       fprintf(splittedClassFile,"%s\t",documentIdString);
       fprintf(splittedClassFile,"%s\t",term);
       fprintf(splittedClassFile,"%d\n",freq);
       *pos=ftell(classFile);
     }
     
     strcpy(documentIdStringOld,documentIdString);
     fscanf(classFile,"%s",documentIdString);
     if(strcmp(documentIdStringOld,documentIdString)!=0){
	    /*TEST printf("siamp al doc : %d\n",*docCounter);*/(*docCounter)++; 
     }
     
  }  
   
  fclose(extractedDocumentFile);

  return 1;
}
     

/*void SplittingOfClasses(struct class_ce_env my_env) 
{
   int whichClass;
   if (my_env.verbose_switch==ON) {
     // show the environment 
     int i;
     printf("----------------------------------------------------\n\n");
     
     printf("          class_path = %s\n",my_env.class_path);
     printf("       centroid_path = %s\n",my_env.centroid_path);
     printf("       tmp_file_path = %s\n\n",my_env.tmp_path);
   
     printf("      verbose_switch = %s\n\n",(my_env.verbose_switch==ON?"ON":"OFF"));
   
     printf("           le_switch = %s\n",(my_env.le_switch==ON?"ON":"OFF"));
     printf("           ne_switch = %s\n",(my_env.ne_switch==ON?"ON":"OFF"));
     printf("           ev_switch = %s\n\n",(my_env.ev_switch==ON?"ON":"OFF"));
   
     printf(" test_set_percentage = %d\n",my_env.test_set_percentage);
     printf("       test_doc_path = %s\n\n",my_env.test_doc_path);
     
     printf("        class_number = %d\n\n",my_env.class_number);
     for (i=0; i<my_env.class_number ; i++)
       printf("   class_names[%d] = %s\n",i,my_env.class_names[i]);
     
     printf("\n----------------------------------------------------\n\n");
  }
  for(whichClass=0;whichClass<my_env.class_number;whichClass++) {
    if (my_env.le_switch==ON)
      classSplitting( my_env,whichClass,"le");
    if (my_env.ne_switch==ON)
      classSplitting( my_env,whichClass,"ne");
    if (my_env.ev_switch==ON)
      classSplitting( my_env,whichClass,"ev");
  }
  if (my_env.le_switch==ON) printf("end Splitting Task\n");
}
*/
