/***********************************************************************/
/*   Text Categorization Framework                                     */
/*                                                                     */
/*                                                                     */
/*   Author: Alessandro Moschitti                                      */
/*   moschitti@info.uniroma2.it                                        */
/*   Date: 30.11.03                                                    */
/*                                                                     */
/*   Copyright (c) 2003  Alessandro Moschitti - All rights reserved    */
/*                                                                     */
/*   This software is available for non-commercial use only. It must   */
/*   not be modified and distributed without prior permission of the   */
/*   author. The author is not responsible for implications from the   */
/*   use of this software.                                             */
/*                                                                     */
/***********************************************************************/



#include "classifier.h"
#include "base_lib.h"



int unifieLemmaDoc(struct class_env my_env, char *classFileName, char *rugClassFIleName)
 {	 
      
  char documentNumber[TERM_LENGHT];
  
  char *stopWord[MAX_STOP_WORD];
  int numberOfSW;
  char stopListFileName[PATH_LENGHT];     
  
  char key[FREAD_BUFFER_SIZE];
  int i;
  int idTerm;
  
  struct dictionary docCentroid[MAX_DOC_SIZE];
  int 	 centroidSize;
   
  long pos;
  
  FILE *f1;
  FILE *class;
    
  sprintf(stopListFileName,"%s/%s",my_env.stopList_path,STOP_WORD_LIST);    

  numberOfSW=loadStopList(stopListFileName,stopWord,my_env.verbose_switch);
  
  if((f1=fopen(rugClassFIleName,"r"))==NULL) {
   if (my_env.verbose_switch==ON)printf("   Warning : file %s does not found\n",rugClassFIleName);
      return 0;
  }
    
  if((class=fopen(classFileName,"w"))==NULL) {
   if (my_env.verbose_switch==ON)printf("   Warning : file %s does not found\n",classFileName);
      return 0;
  }

  pos=0;
  while(!feof(f1)){
	  
     centroidSize = buildCleanedDocumentCentroid(f1,docCentroid,
                    &pos,documentNumber,stopWord,numberOfSW);
     /*TESTprintf("Doc. number %d built, size=%d\n",documentNumber,centroidSize);*/
		     
     for(idTerm=0;idTerm<centroidSize;idTerm++){
        fprintf(class,"%s\t",documentNumber);
	fprintf(class,"%s\t",docCentroid[idTerm].term);
	fprintf(class,"%d\n",docCentroid[idTerm].freq); 
	 
	 }
  }		 

  fclose(f1);
  fclose(class);
  shutdownStopList(stopWord,numberOfSW);
  return 1;
}


void task0(struct class_env my_env,int whichClass,char *extension)
{
  char classFileName[PATH_LENGHT];
  char rugClassFileName[PATH_LENGHT];
  char centroidFileName[PATH_LENGHT];
  char orderedCentroidFileName[PATH_LENGHT];
  char sortcommand[200],
       erasecommand[200];
  
  if (my_env.verbose_switch==ON) 
    printf("\nPurification from stop-words and Unification Lemma for class '%s' using '%s' information\n\n",my_env.class_names[whichClass],extension);

  sprintf(classFileName,"%s/%s.%s",my_env.class_path,my_env.class_names[whichClass],extension);

  sprintf(rugClassFileName,"%s/%s.%s",my_env.rug_class_path,my_env.class_names[whichClass],extension);
   
  if (unifieLemmaDoc(my_env,classFileName,rugClassFileName)==0 )
	return; /*the class file does not exist*/
  if (my_env.verbose_switch==ON) printf("          class file=%s written\n",centroidFileName);
  
}


void ClassUnificationProcessing(struct class_env my_env) 
{
   int whichClass;

   if (my_env.verbose_switch==ON) {
     /* show the environment */
     int i;
     printf("----------------------------------------------------\n\n");
     
     printf("          class_path = %s\n",my_env.class_path);
     printf("       rug_class_path = %s\n",my_env.rug_class_path);
     printf("       tmp_file_path = %s\n\n",my_env.tmp_path);
     printf("       stopList_path = %s\n\n",my_env.stopList_path);

     printf("      verbose_switch = %s\n\n",(my_env.verbose_switch==ON?"ON":"OFF"));
   
     printf("           le_switch = %s\n",(my_env.le_switch==ON?"ON":"OFF"));
     printf("           ne_switch = %s\n",(my_env.ne_switch==ON?"ON":"OFF"));
     printf("           ev_switch = %s\n\n",(my_env.ev_switch==ON?"ON":"OFF"));
   
     printf("        class_number = %d\n\n",my_env.class_number);
     for (i=0; i<my_env.class_number ; i++)
       printf("   class_names[%d] = %s\n",i,my_env.class_names[i]);
     
     printf("\n----------------------------------------------------\n\n");
  }
  for(whichClass=0;whichClass<my_env.class_number;whichClass++) {
    if (my_env.le_switch==ON)
      task0( my_env,whichClass,"le");
    if (my_env.ne_switch==ON)
      task0( my_env,whichClass,"ne");
    if (my_env.ev_switch==ON)
      task0( my_env,whichClass,"ev");
  }
}
/* end */
