/***********************************************************************/
/*   Text Categorization Framework                                     */
/*                                                                     */
/*                                                                     */
/*   Author: Alessandro Moschitti                                      */
/*   moschitti@info.uniroma2.it                                        */
/*   Date: 30.11.03                                                    */
/*                                                                     */
/*   Copyright (c) 2003  Alessandro Moschitti - All rights reserved    */
/*                                                                     */
/*   This software is available for non-commercial use only. It must   */
/*   not be modified and distributed without prior permission of the   */
/*   author. The author is not responsible for implications from the   */
/*   use of this software.                                             */
/*                                                                     */
/***********************************************************************/


#include "classifier.h"
#include <math.h>
#include <string.h>


struct statInfo{

  float positiv,
        squarePositiv,
	negativ,
        squareNegativ;
	
  int 	posNum,
	negNum;
  
  float posAverage,
	negAverage,
	posVariance,
        negVariance;
 };
		  



int loadThresholdKB(struct dict_env my_env,char *extension,struct CKB_env *clKB) {
  char etichetta[TERM_LENGHT];
  char dictAbsName[PATH_LENGHT];
  char weightDictAbsName[PATH_LENGHT];
  char infoDictionariesAbsName[PATH_LENGHT];

  int i=0;

    
  FILE *infoDictionariesFile;
  FILE *offsetDictFile;
  FILE *weightDictFile;
   
  
  
  /* -------------------------------- */
  /* Reading Dictionaries information */
  /* -------------------------------- */
  
  
  if (my_env.verbose_switch==ON) printf ("Reading Dictionaries information\n");
  sprintf(infoDictionariesAbsName,"%s/%s.%s",my_env.dict_path,INFO_DICTIONARY,extension);
  if((infoDictionariesFile = fopen(infoDictionariesAbsName,"r"))==NULL) {
      printf("file %s does not found\n",infoDictionariesAbsName);
      return 0;
  }
  
  
  fscanf(infoDictionariesFile,"%s\n%d\n%s\n%d\n",
		              etichetta,&(clKB->totalDocumentNumber),
		              etichetta,&(clKB->differentWordNumber));
  clKB->centroidNumber=0;
  fscanf(infoDictionariesFile,"%s\n",clKB->centroidNames[clKB->centroidNumber]);
  while(!feof(infoDictionariesFile)&& strcmp(clKB->centroidNames[clKB->centroidNumber],"Max_Weight=")!=0){
    if (my_env.verbose_switch==ON) 
	   printf("centroidNames[%d]=%s\n",clKB->centroidNumber,clKB->centroidNames[clKB->centroidNumber]);
    clKB->centroidNumber++;
    fscanf(infoDictionariesFile,"%s\n",clKB->centroidNames[clKB->centroidNumber]);
  }
  
  fscanf(infoDictionariesFile,"%f\n%s",&(clKB->maxWeight),etichetta);
  
  i=0;
 
  fscanf(infoDictionariesFile,"%f\n",&clKB->centroidTotalFreqs[i]);
  while(!feof(infoDictionariesFile)){	  
    if (my_env.verbose_switch==ON) 
	   printf("centroidTotalFreq[%d]=%f\n",i,clKB->centroidTotalFreqs[i]);
    i++;
    fscanf(infoDictionariesFile,"%f\n",&clKB->centroidTotalFreqs[i]);
		    
 /*   fscanf(infoDictionariesFile,"%s",infoDictionariesAbsName);
    printf("%s\n",infoDictionariesAbsName);*/
  }
  if (my_env.verbose_switch==ON) 
	   printf("centroidTotalFreq[%d]=%f\n",i,clKB->centroidTotalFreqs[i]);
  fclose(infoDictionariesFile);
  
  
  /* -------------------------------- */
  /* Reading 			      */
  /* -------------------------------- */

  sprintf(dictAbsName,"%s/%s.%s",my_env.dict_path,OFFSET_DICTIONARY,extension);
  if((offsetDictFile = fopen(dictAbsName,"r"))==NULL) {
      printf("file %s does not found\n",dictAbsName);
      return 0;
  }
  sprintf(weightDictAbsName,"%s/%s.%s",my_env.dict_path,WEIGHT_DICTIONARY,extension);
  if((clKB->weightDictFile = fopen(weightDictAbsName,"r"))==NULL) {
      printf("file %s does not found\n",weightDictAbsName);
      return 0;
  }

  if (my_env.verbose_switch==ON) printf ("loading Offset Dictionary ... "); 
  loadDictionaryOffset(offsetDictFile,clKB->differentWordNumber,&clKB->offsetDictArray);
  if (my_env.verbose_switch==ON)      printf ("done !\n");    

  fclose(offsetDictFile);
  
}
  



/* Compute sums */

void statisticBuilding(float scalarProduct[], struct statInfo *stat,int rightClassIndex, 
		int classNumber){	
  int i=0;
  float rowMeans=0;
  
  for(i=0;i<classNumber;i++)
     if(i!=rightClassIndex)
	rowMeans+=(scalarProduct[rightClassIndex]-scalarProduct[i]);

  rowMeans/=(classNumber-1);
  stat[rightClassIndex].positiv+=rowMeans;
  stat[rightClassIndex].squarePositiv+=(rowMeans*rowMeans);
  stat[rightClassIndex].posNum++;  
}   
	  
/* Compute Averages and Variances */

   int computeAveragesAndVariances(struct statInfo *stat, int classNumber){


     int i=0;

     for(i=0;i<classNumber;i++){
	   stat[i].posAverage=stat[i].positiv/stat[i].posNum;
  	   stat[i].posVariance=
	      sqrt(stat[i].squarePositiv/stat[i].posNum-stat[i].posAverage*stat[i].posAverage);
	   
	   stat[i].negAverage=
  	   stat[i].negVariance=0;
     }
     
   }   	  


/* write results in info dictionary */

 writeResults(struct statInfo stat[], int classNumber, struct dict_env my_env, char *extension){
    int i;
    char infoDictionariesAbsName[PATH_LENGHT];
    FILE *infoDictionariesFile;   
	   
    if (my_env.verbose_switch==ON) 
	    printf ("\n Writing Thresholds and Statistic Parameters on Dictionary Info ... ");
    sprintf(infoDictionariesAbsName,"%s/%s.%s",my_env.dict_path,INFO_DICTIONARY,extension);
    if((infoDictionariesFile = fopen(infoDictionariesAbsName,"a"))==NULL) 
	printf("file %s does not found\n",infoDictionariesAbsName);
      
    fprintf(infoDictionariesFile,"Average_StdDeviation_Threshold:\n");
    for(i=0;i<classNumber;i++)
      	fprintf(infoDictionariesFile,"%5.8f %5.8f\n",stat[i].posAverage,stat[i].posVariance);

    fprintf(infoDictionariesFile,"NegativeScores:\n");
    for(i=0;i<classNumber;i++)
      	fprintf(infoDictionariesFile,"%5.4f %5.4f\n",stat[i].negAverage,stat[i].negVariance);

    fclose(infoDictionariesFile);
    
    if (my_env.verbose_switch==ON) printf ("done !\n");

}  
  
  
  
thresholdComputation(struct dict_env my_env, char *extension)
{ 
  int whichClass;
  int i;
  int r;

  int documentNumber;
  char documentCentroidAbsName[PATH_LENGHT];
  char  scalarProductsAbsName[PATH_LENGHT];
  char  NumericScalarProductsAbsName[PATH_LENGHT];
  char  rmcommand[PATH_LENGHT];
  struct CKB_env my_CKB_env;
  FILE *scalarProductsFile;

  float scalarProducts[MAX_CLASS_NUMBER];
  struct statInfo statistic[MAX_CLASS_NUMBER];

  FILE *classFile;
  char classFileAbsName[PATH_LENGHT];
  
  struct dictionary docCentroid[MAX_DOC_SIZE];
  int 	 centroidSize;
  
  long pos;
   
   
  /* -------------------------------------------- */
  /*    Init statisticl structures	          */
  /* -------------------------------------------- */
  
  for(i=0;i<my_env.class_number;i++)
     statistic[i].positiv=statistic[i].squarePositiv=
     statistic[i].negativ=statistic[i].squareNegativ=
     statistic[i].posNum=statistic[i].negNum=0;
  
#ifndef RDS
  return;		   /* se non stiamo operando con RDS non calcolare le
soglie*/
#endif
		  
#ifndef AUTOMATIC_THRESHOLD_COMPUTATION
  return;		   /* se non stiamo operando con RDS non calcolare le
soglie*/
#endif
		  
		  

  /* -------------------------------------------- */
  /*    Load the Threshold Knowledge Bases   */
  /* -------------------------------------------- */
  
  loadThresholdKB(my_env,extension,&my_CKB_env);
  
  /* -------------------------------------------- */
  /* Open Scalar Products file and create Header  */
  /* -------------------------------------------- */
 
  sprintf(scalarProductsAbsName,"%s/%s.%s",my_env.dict_path,SCALAR_PRODUCTS,"txt");
  if((scalarProductsFile = fopen(scalarProductsAbsName,"w"))==NULL) {
      if (my_env.verbose_switch==ON) printf("file %s write permission denied\n",scalarProductsAbsName);
      exit(-1);
  }
  
  
  /* ----------------------------------- */
  /* compute all document Scalar Products*/
  /* ----------------------------------- */
  
  if (my_env.verbose_switch==ON) printf("compute all document Scalar Products\n");
  
  sprintf(documentCentroidAbsName,"%s/%s.%s.ce",my_env.tmp_path,GENERAL_DOCUMENT,extension);

  for(whichClass=0;whichClass<my_env.class_number;whichClass++) {
    
    
    sprintf(classFileAbsName,"%s/%s.%s",my_env.splitted_class_path,my_env.class_names[whichClass],extension);
    if((classFile = fopen(classFileAbsName,"r"))==NULL) {
	     printf("Warning : file %s does not found\n",classFileAbsName);
	     return 0;
    }	
    
    
    /* file pointer points to the first document of splitted_class*/  
    pos=0;    
    while (!feof(classFile)) {

	 centroidSize = buildExtractedDocumentCentroid(classFile,docCentroid,&pos,&documentNumber);
	 if (my_env.verbose_switch==ON)
           printf ("\n Document centroid %d of class: <%s> created\n",documentNumber,my_env.class_names[whichClass]);


	 if (my_env.verbose_switch==ON) printf ("  Scalar Products Computations ... "); 

	 aDocumentScalarProduct(docCentroid,centroidSize,extension,&my_CKB_env,scalarProducts);

	 if (my_env.verbose_switch==ON) printf ("done !\n");


	 if (my_env.verbose_switch==ON) printf ("  Writing output ... "); 

	  /*** Output of Classifier ***/

	 fprintf(scalarProductsFile,"%s\t%d",my_env.class_names[whichClass],documentNumber);

	 for(i=0;i<my_env.class_number;i++)
	      fprintf(scalarProductsFile,"%14.4f",scalarProducts[i]);
	 fprintf(scalarProductsFile,"\n");

	 statisticBuilding(scalarProducts,statistic,whichClass,my_env.class_number); 	  	

	 if (my_env.verbose_switch==ON) printf ("done !\n"); 

    }
     
   fclose(classFile);
  }
  /*close the Scalar Product file */
  fclose(scalarProductsFile);
 
   computeAveragesAndVariances(statistic,my_env.class_number);
  
  /* write results in dictionary Info */
  writeResults(statistic,my_env.class_number,my_env,extension);

   printf("\n%s written\n",scalarProductsAbsName);
  
  /* Shutdown the Classification Knowledge Bases */
  
  shutdownDictionary(my_CKB_env.offsetDictArray,my_CKB_env.differentWordNumber);
}

 


